@@ -58,7 +58,7 @@ int ivtv_audio_set_io(struct ivtv *itv)
 
 void ivtv_audio_set_route(struct ivtv *itv, struct v4l2_routing *route)
 {
-	ivtv_i2c_hw(itv, itv->card->hw_audio, VIDIOC_INT_S_AUDIO_ROUTING, &route);
+	ivtv_i2c_hw(itv, itv->card->hw_audio, VIDIOC_INT_S_AUDIO_ROUTING, route);
 }
 
 void ivtv_audio_set_audio_clock_freq(struct ivtv *itv, u8 freq)
