@@ -50,18 +50,7 @@
 
 /* Hauppauge PVR-250 cards */
 
-static const struct ivtv_card_pci_info ivtv_pci_pvr250[] = {
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x0001 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x0003 },
-	{ PCI_DEVICE_ID_IVTV15, IVTV_PCI_ID_HAUPPAUGE, 0x4001 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x4008 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x4009 },
-	{ PCI_DEVICE_ID_IVTV15, IVTV_PCI_ID_HAUPPAUGE, 0x4801 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x4801 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x4803 },
-	{ 0, 0, 0 }
-};
-
+/* Note: for Hauppauge cards the tveeprom information is used instead of PCI IDs */
 static const struct ivtv_card ivtv_card_pvr250 = {
 	.type = IVTV_CARD_PVR_250,
 	.name = "Hauppauge WinTV PVR-250",
@@ -85,7 +74,6 @@ static const struct ivtv_card ivtv_card_pvr250 = {
 		{ IVTV_CARD_INPUT_LINE_IN2,   MSP_SCART3 },
 	},
 	.radio_input = { IVTV_CARD_INPUT_AUD_TUNER, MSP_SCART2 },
-	.pci_list = ivtv_pci_pvr250,
 };
 
 /* ------------------------------------------------------------------------- */
@@ -115,13 +103,6 @@ static struct ivtv_card_output ivtv_pvr350_outputs[] = {
         }
 };
 
-static const struct ivtv_card_pci_info ivtv_pci_pvr350[] = {
-	{ PCI_DEVICE_ID_IVTV15, IVTV_PCI_ID_HAUPPAUGE, 0x4000 },
-	{ PCI_DEVICE_ID_IVTV15, IVTV_PCI_ID_HAUPPAUGE, 0x4800 },
-	{ PCI_DEVICE_ID_IVTV15, IVTV_PCI_ID_HAUPPAUGE, 0 },
-	{ 0, 0, 0 }
-};
-
 static const struct ivtv_card ivtv_card_pvr350 = {
 	.type = IVTV_CARD_PVR_350,
 	.name = "Hauppauge WinTV PVR-350",
@@ -147,7 +128,6 @@ static const struct ivtv_card ivtv_card_pvr350 = {
 		{ IVTV_CARD_INPUT_LINE_IN2,   MSP_SCART3 },
 	},
 	.radio_input = { IVTV_CARD_INPUT_AUD_TUNER, MSP_SCART2 },
-	.pci_list = ivtv_pci_pvr350,
 };
 
 /* PVR-350 V1 boards have a different audio tuner input and use a
@@ -185,26 +165,6 @@ static const struct ivtv_card ivtv_card_pvr350_v1 = {
 
 /* Hauppauge PVR-150/PVR-500 cards */
 
-static const struct ivtv_card_pci_info ivtv_pci_pvr150[] = {
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x0009 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x0801 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x0807 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x17e7 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x17f7 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x37f1 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x37f3 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x8001 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x8003 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0x8801 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0xc801 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0xe807 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE, 0xe817 },
-        // Alternate Hauppauge vendor IDs
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE_ALT1, 0x0801 },
-	{ PCI_DEVICE_ID_IVTV16, IVTV_PCI_ID_HAUPPAUGE_ALT2, 0x8801 },
-	{ 0, 0, 0 }
-};
-
 static const struct ivtv_card ivtv_card_pvr150 = {
 	.type = IVTV_CARD_PVR_150,
 	.name = "Hauppauge WinTV PVR-150",
@@ -234,7 +194,6 @@ static const struct ivtv_card ivtv_card_pvr150 = {
 	       		 CX25840_AUDIO_SERIAL, WM8775_AIN4 },
 	/* apparently needed for the IR blaster */
 	.gpio_init = { .direction = 0x1f01, .initial_value = 0x26f3 },
-	.pci_list = ivtv_pci_pvr150,
 };
 
 /* ------------------------------------------------------------------------- */
@@ -501,7 +460,7 @@ static const struct ivtv_card ivtv_card_tg5000tv = {
  	.gpio_audio_mode   = { .mask = 0x4300, .mono   = 0x4000, .stereo = 0x0200,
  			      .lang1 = 0x0300, .lang2  = 0x0000, .both   = 0x0200 },
  	.gpio_video_input  = { .mask = 0x0030, .tuner  = 0x0000,
- 			  .composite = 0x0010, .svideo = 0x0020 },
+			  .composite = 0x0010, .svideo = 0x0020 },
 	.tuners = {
 		{ .std = V4L2_STD_525_60, .tuner = TUNER_PHILIPS_FQ1286 },
 	},
@@ -555,7 +514,7 @@ static const struct ivtv_card ivtv_card_cx23416gyc = {
 	.v4l2_capabilities = IVTV_CAP_ENCODER,
 	.hw_video = IVTV_HW_SAA717X | IVTV_HW_GPIO |
 		IVTV_HW_UPD64031A | IVTV_HW_UPD6408X,
-	.hw_audio = IVTV_HW_SAA717X,// | IVTV_HW_GPIO,
+	.hw_audio = IVTV_HW_SAA717X,
 	.hw_audio_ctrl = IVTV_HW_SAA717X,
 	.hw_all = IVTV_HW_GPIO | IVTV_HW_SAA717X | IVTV_HW_TUNER |
 		  IVTV_HW_UPD64031A | IVTV_HW_UPD6408X,
@@ -587,7 +546,7 @@ static const struct ivtv_card ivtv_card_cx23416gyc_nogr = {
 	.name = "Yuan MPG600GR, Kuroutoshikou CX23416GYC-STVLP (no GR)",
 	.v4l2_capabilities = IVTV_CAP_ENCODER,
 	.hw_video = IVTV_HW_SAA717X | IVTV_HW_GPIO | IVTV_HW_UPD6408X,
-	.hw_audio = IVTV_HW_SAA717X | IVTV_HW_GPIO,
+	.hw_audio = IVTV_HW_SAA717X,
 	.hw_audio_ctrl = IVTV_HW_SAA717X,
 	.hw_all = IVTV_HW_GPIO | IVTV_HW_SAA717X | IVTV_HW_TUNER |
 		  IVTV_HW_UPD6408X,
@@ -617,7 +576,7 @@ static const struct ivtv_card ivtv_card_cx23416gyc_nogrycs = {
 	.name = "Yuan MPG600GR, Kuroutoshikou CX23416GYC-STVLP (no GR/YCS)",
 	.v4l2_capabilities = IVTV_CAP_ENCODER,
 	.hw_video = IVTV_HW_SAA717X | IVTV_HW_GPIO,
-	.hw_audio = IVTV_HW_SAA717X | IVTV_HW_GPIO,
+	.hw_audio = IVTV_HW_SAA717X,
 	.hw_audio_ctrl = IVTV_HW_SAA717X,
 	.gpio_audio_freq   = { .mask = 0xc000, .f32000 = 0x0000,
 	         	     .f44100 = 0x4000, .f48000 = 0x8000 },
@@ -659,7 +618,7 @@ static const struct ivtv_card ivtv_card_gv_mvprx = {
 	.name = "I/O Data GV-MVP/RX, GV-MVP/RX2W (dual tuner)",
 	.v4l2_capabilities = IVTV_CAP_ENCODER,
 	.hw_video = IVTV_HW_SAA7115 | IVTV_HW_UPD64031A | IVTV_HW_UPD6408X,
-	.hw_audio = IVTV_HW_WM8739 | IVTV_HW_GPIO,
+	.hw_audio = IVTV_HW_GPIO,
 	.hw_audio_ctrl = IVTV_HW_WM8739,
 	.hw_all = IVTV_HW_GPIO | IVTV_HW_SAA7115 | IVTV_HW_TVAUDIO |
 	       	  IVTV_HW_TUNER | IVTV_HW_WM8739 |
@@ -696,7 +655,7 @@ static const struct ivtv_card ivtv_card_gv_mvprx2e = {
 	.name = "I/O Data GV-MVP/RX2E",
 	.v4l2_capabilities = IVTV_CAP_ENCODER,
 	.hw_video = IVTV_HW_SAA7115,
-	.hw_audio = IVTV_HW_WM8739 | IVTV_HW_GPIO,
+	.hw_audio = IVTV_HW_GPIO,
 	.hw_audio_ctrl = IVTV_HW_WM8739,
 	.hw_all = IVTV_HW_GPIO | IVTV_HW_SAA7115 | IVTV_HW_TUNER |
 		  IVTV_HW_TVAUDIO | IVTV_HW_WM8739,
