@@ -19,132 +19,32 @@
 
 #include "ivtv-driver.h"
 #include "ivtv-cards.h"
+#include "ivtv-ioctl.h"
 #include "ivtv-audio.h"
 #include "ivtv-i2c.h"
+#include "ivtv-mailbox.h"
 #include "ivtv-controls.h"
 
-static int ivtv_querymenu(struct ivtv *itv, struct v4l2_querymenu *qmenu)
-{
-	const char * const *menu;
-	u32 i = qmenu->index;
+static const u32 user_ctrls[] = {
+	V4L2_CID_USER_CLASS,
+	V4L2_CID_BRIGHTNESS,
+	V4L2_CID_CONTRAST,
+	V4L2_CID_SATURATION,
+	V4L2_CID_HUE,
+	V4L2_CID_AUDIO_VOLUME,
+	V4L2_CID_AUDIO_BALANCE,
+	V4L2_CID_AUDIO_BASS,
+	V4L2_CID_AUDIO_TREBLE,
+	V4L2_CID_AUDIO_MUTE,
+	V4L2_CID_AUDIO_LOUDNESS,
+	0
+};
 
-	switch (qmenu->id) {
-	case V4L2_CID_IVTV_FREQ:{
-		static const char * const menu_freq[] = {
-			"44.1 kHz",
-			"48 kHz",
-			"32 kHz",
-			NULL
-		};
-		menu = menu_freq;
-		break;
-	}
-	case V4L2_CID_IVTV_ENC:{
-		static const char * const menu_layer[] = {
-			"Layer 1",
-			"Layer 2",
-			"Layer 3 (?)",
-			NULL
-		};
-		menu = menu_layer;
-		break;
-	}
-	case V4L2_CID_IVTV_BITRATE:{
-		static const char * const menu_bitrate[] = {
-			"[L1/L2] Free fmt",
-			"[L1/L2] 32k/32k",
-			"[L1/L2] 64k/48k",
-			"[L1/L2] 96k/56k",
-			"[L1/L2] 128k/64k",
-			"[L1/L2] 160k/80k",
-			"[L1/L2] 192k/96k",
-			"[L1/L2] 224k/112k",
-			"[L1/L2] 256k/128k",
-			"[L1/L2] 288k/160k",
-			"[L1/L2] 320k/192k",
-			"[L1/L2] 352k/224k",
-			"[L1/L2] 384k/256k",
-			"[L1/L2] 416k/320k",
-			"[L1/L2] 448k/384k",
-			NULL
-		};
-		menu = menu_bitrate;
-		break;
-	}
-	case V4L2_CID_IVTV_MONO:{
-		static const char * const menu_mono[] = {
-			"Stereo",
-			"JointStereo",
-			"Dual",
-			"Mono",
-			NULL
-		};
-		menu = menu_mono;
-		break;
-	}
-	case V4L2_CID_IVTV_JOINT:{
-		static const char * const menu_joint[] = {
-			"Subbands 4-31/bound=4",
-			"Subbands 8-31/bound=8",
-			"Subbands 12-31/bound=12",
-			"Subbands 16-31/bound=16",
-			NULL
-		};
-		menu = menu_joint;
-		break;
-	}
-	case V4L2_CID_IVTV_EMPHASIS:{
-		static const char * const menu_emph[] = {
-			"None",
-			"50/15uS",
-			"CCITT J.17",
-			NULL
-		};
-		menu = menu_emph;
-		break;
-	}
-	case V4L2_CID_IVTV_CRC:{
-		static const char * const menu_crc[] = {
-			"CRC off",
-			"CRC on",
-			NULL
-		};
-		menu = menu_crc;
-		break;
-	}
-	case V4L2_CID_IVTV_COPYRIGHT:{
-		static const char * const menu_copyright[] = {
-			"Copyright off",
-			"Copyright on",
-			NULL
-		};
-		menu = menu_copyright;
-		break;
-	}
-	case V4L2_CID_IVTV_GEN:{
-		static const char * const menu_gen[] = {
-			"Copy",
-			"Original",
-			NULL
-		};
-		menu = menu_gen;
-		break;
-	}
-	default:
-		IVTV_DEBUG_IOCTL("invalid control %x\n", qmenu->id);
-		return -EINVAL;
-	}
-	while (i-- && *menu) {
-		menu++;
-	}
-	if (*menu == NULL) {
-		IVTV_DEBUG_IOCTL("invalid menu index %d (control %x)\n",
-                                 qmenu->index, qmenu->id);
-		return -EINVAL;
-	}
-	strncpy(qmenu->name, *menu, sizeof(qmenu->name));
-	return 0;
-}
+static const u32 *ctrl_classes[] = {
+	user_ctrls,
+	cx2341x_mpeg_ctrls,
+	NULL
+};
 
 static void ivtv_init_queryctrl(struct v4l2_queryctrl *qctrl,
 				int type, int min, int max, int step, int def)
@@ -155,8 +55,7 @@ static void ivtv_init_queryctrl(struct v4l2_queryctrl *qctrl,
 	qctrl->step = step;
 	qctrl->default_value = def;
 	qctrl->flags = 0;
-	qctrl->reserved[0] = 0;
-	qctrl->reserved[1] = 0;
+	qctrl->reserved[0] = qctrl->reserved[1] = 0;
 }
 
 static int ivtv_queryctrl(struct ivtv *itv, struct v4l2_queryctrl *qctrl)
@@ -165,45 +64,11 @@ static int ivtv_queryctrl(struct ivtv *itv, struct v4l2_queryctrl *qctrl)
 
 	IVTV_DEBUG_IOCTL("VIDIOC_QUERYCTRL(%08x)\n", qctrl->id);
 
-	switch (qctrl->id) {
-		/* Audio controls */
-	case V4L2_CID_IVTV_FREQ:
-		name = "Frequency";
-		ivtv_init_queryctrl(qctrl, V4L2_CTRL_TYPE_MENU, 0, 2, 1, 1);
-		break;
-	case V4L2_CID_IVTV_ENC:
-		name = "Encoding";
-		ivtv_init_queryctrl(qctrl, V4L2_CTRL_TYPE_MENU, 0, 2, 1, 1);
-		break;
-	case V4L2_CID_IVTV_BITRATE:
-		name = "Audio Bitrate";
-		ivtv_init_queryctrl(qctrl, V4L2_CTRL_TYPE_MENU, 0, 14, 1, 14);
-		break;
-	case V4L2_CID_IVTV_MONO:
-		name = "Mono/Stereo";
-		ivtv_init_queryctrl(qctrl, V4L2_CTRL_TYPE_MENU, 0, 3, 1, 0);
-		break;
-	case V4L2_CID_IVTV_JOINT:
-		name = "Joint extension";
-		ivtv_init_queryctrl(qctrl, V4L2_CTRL_TYPE_MENU, 0, 3, 1, 0);
-		break;
-	case V4L2_CID_IVTV_EMPHASIS:
-		name = "Emphasis";
-		ivtv_init_queryctrl(qctrl, V4L2_CTRL_TYPE_MENU, 0, 2, 1, 0);
-		break;
-	case V4L2_CID_IVTV_CRC:
-		name = "Audio CRC";
-		ivtv_init_queryctrl(qctrl, V4L2_CTRL_TYPE_MENU, 0, 1, 1, 0);
-		break;
-	case V4L2_CID_IVTV_COPYRIGHT:
-		name = "Copyright";
-		ivtv_init_queryctrl(qctrl, V4L2_CTRL_TYPE_MENU, 0, 1, 1, 0);
-		break;
-	case V4L2_CID_IVTV_GEN:
-		name = "Generation";
-		ivtv_init_queryctrl(qctrl, V4L2_CTRL_TYPE_MENU, 0, 1, 1, 0);
-		break;
+	qctrl->id = v4l2_ctrl_next(ctrl_classes, qctrl->id);
+	if (qctrl->id == 0)
+		return -EINVAL;
 
+	switch (qctrl->id) {
 		/* Decoder controls */
 	case V4L2_CID_IVTV_DEC_SMOOTH_FF:
 		name = "Smooth Slow/FF";
@@ -239,7 +104,9 @@ static int ivtv_queryctrl(struct ivtv *itv, struct v4l2_queryctrl *qctrl)
 	case V4L2_CID_HUE:
 	case V4L2_CID_SATURATION:
 	case V4L2_CID_CONTRAST:
-		return itv->video_dec_func(itv, VIDIOC_QUERYCTRL, qctrl);
+		if (itv->video_dec_func(itv, VIDIOC_QUERYCTRL, qctrl))
+			qctrl->flags |= V4L2_CTRL_FLAG_DISABLED;
+		return 0;
 
 	case V4L2_CID_AUDIO_VOLUME:
 	case V4L2_CID_AUDIO_MUTE:
@@ -247,17 +114,29 @@ static int ivtv_queryctrl(struct ivtv *itv, struct v4l2_queryctrl *qctrl)
 	case V4L2_CID_AUDIO_BASS:
 	case V4L2_CID_AUDIO_TREBLE:
 	case V4L2_CID_AUDIO_LOUDNESS:
-		return ivtv_i2c_hw(itv, itv->card->hw_audio_ctrl, VIDIOC_QUERYCTRL, qctrl);
+		if (ivtv_i2c_hw(itv, itv->card->hw_audio_ctrl, VIDIOC_QUERYCTRL, qctrl))
+			qctrl->flags |= V4L2_CTRL_FLAG_DISABLED;
+		return 0;
 
 	default:
-		IVTV_DEBUG_IOCTL("invalid control %x\n", qctrl->id);
-		return -EINVAL;
+		if (cx2341x_ctrl_query(&itv->params, qctrl))
+			qctrl->flags |= V4L2_CTRL_FLAG_DISABLED;
+		return 0;
 	}
 	strncpy(qctrl->name, name, sizeof(qctrl->name) - 1);
 	qctrl->name[sizeof(qctrl->name) - 1] = 0;
 	return 0;
 }
 
+static int ivtv_querymenu(struct ivtv *itv, struct v4l2_querymenu *qmenu)
+{
+	struct v4l2_queryctrl qctrl;
+
+	qctrl.id = qmenu->id;
+	ivtv_queryctrl(itv, &qctrl);
+	return v4l2_ctrl_query_menu(qmenu, &qctrl, cx2341x_ctrl_get_menu(qmenu->id));
+}
+
 static int ivtv_s_ctrl(struct ivtv *itv, struct v4l2_control *vctrl)
 {
 	s32 v = vctrl->value;
@@ -265,63 +144,6 @@ static int ivtv_s_ctrl(struct ivtv *itv, struct v4l2_control *vctrl)
 	IVTV_DEBUG_IOCTL("VIDIOC_S_CTRL(%08x, %x)\n", vctrl->id, v);
 
 	switch (vctrl->id) {
-		/* Audio controls */
-	case V4L2_CID_IVTV_FREQ:
-		if ((v < 0) || (v > 2))
-			return -ERANGE;
-		itv->codec.audio_bitmask &= ~3;
-		itv->codec.audio_bitmask |= v;
-		ivtv_audio_set_audio_clock_freq(itv, v);
-		break;
-	case V4L2_CID_IVTV_ENC:
-		if ((v < 0) || (v > 2))
-			return -ERANGE;
-		itv->codec.audio_bitmask &= ~(3 << 2);
-		itv->codec.audio_bitmask |= (v + 1) << 2;
-		break;
-	case V4L2_CID_IVTV_BITRATE:
-		if ((v < 0) || (v > 14))
-			return -ERANGE;
-		itv->codec.audio_bitmask &= ~(15 << 4);
-		itv->codec.audio_bitmask |= v << 4;
-		break;
-	case V4L2_CID_IVTV_MONO:
-		if ((v < 0) || (v > 3))
-			return -ERANGE;
-		itv->codec.audio_bitmask &= ~(3 << 8);
-		itv->codec.audio_bitmask |= v << 8;
-		break;
-	case V4L2_CID_IVTV_JOINT:
-		if ((v < 0) || (v > 3))
-			return -ERANGE;
-		itv->codec.audio_bitmask &= ~(3 << 10);
-		itv->codec.audio_bitmask |= v << 10;
-		break;
-	case V4L2_CID_IVTV_EMPHASIS:
-		if ((v < 0) || (v > 2))
-			return -ERANGE;
-		itv->codec.audio_bitmask &= ~(3 << 12);
-		itv->codec.audio_bitmask |= v << 12;
-		break;
-	case V4L2_CID_IVTV_CRC:
-		if ((v < 0) || (v > 1))
-			return -ERANGE;
-		itv->codec.audio_bitmask &= ~(1 << 14);
-		itv->codec.audio_bitmask |= v << 14;
-		break;
-	case V4L2_CID_IVTV_COPYRIGHT:
-		if ((v < 0) || (v > 1))
-			return -ERANGE;
-		itv->codec.audio_bitmask &= ~(1 << 15);
-		itv->codec.audio_bitmask |= v << 15;
-		break;
-	case V4L2_CID_IVTV_GEN:
-		if ((v < 0) || (v > 1))
-			return -ERANGE;
-		itv->codec.audio_bitmask &= ~(1 << 16);
-		itv->codec.audio_bitmask |= v << 16;
-		break;
-
 		/* Decoder controls */
 	case V4L2_CID_IVTV_DEC_SMOOTH_FF:
 		if ((v < 0) || (v > 1))
@@ -386,35 +208,6 @@ static int ivtv_g_ctrl(struct ivtv *itv, struct v4l2_control *vctrl)
 	IVTV_DEBUG_IOCTL("VIDIOC_G_CTRL(%08x)\n", vctrl->id);
 
 	switch (vctrl->id) {
-		/* Audio controls */
-	case V4L2_CID_IVTV_FREQ:
-		vctrl->value = itv->codec.audio_bitmask & 3;
-		break;
-	case V4L2_CID_IVTV_ENC:
-		vctrl->value = ((itv->codec.audio_bitmask >> 2) & 3) - 1;
-		break;
-	case V4L2_CID_IVTV_BITRATE:
-		vctrl->value = (itv->codec.audio_bitmask >> 4) & 15;
-		break;
-	case V4L2_CID_IVTV_MONO:
-		vctrl->value = (itv->codec.audio_bitmask >> 8) & 3;
-		break;
-	case V4L2_CID_IVTV_JOINT:
-		vctrl->value = (itv->codec.audio_bitmask >> 10) & 3;
-		break;
-	case V4L2_CID_IVTV_EMPHASIS:
-		vctrl->value = (itv->codec.audio_bitmask >> 12) & 3;
-		break;
-	case V4L2_CID_IVTV_CRC:
-		vctrl->value = (itv->codec.audio_bitmask >> 14) & 1;
-		break;
-	case V4L2_CID_IVTV_COPYRIGHT:
-		vctrl->value = (itv->codec.audio_bitmask >> 15) & 1;
-		break;
-	case V4L2_CID_IVTV_GEN:
-		vctrl->value = (itv->codec.audio_bitmask >> 16) & 1;
-		break;
-
 		/* Decoder controls */
 	case V4L2_CID_IVTV_DEC_SMOOTH_FF:
 		vctrl->value = itv->dec_options.speed.smooth;
@@ -459,8 +252,51 @@ static int ivtv_g_ctrl(struct ivtv *itv, struct v4l2_control *vctrl)
 	return 0;
 }
 
+static int ivtv_setup_vbi_fmt(struct ivtv *itv, enum v4l2_mpeg_stream_vbi_fmt fmt)
+{
+	if (!(itv->v4l2_cap & V4L2_CAP_SLICED_VBI_CAPTURE))
+		return -EINVAL;
+	if (atomic_read(&itv->capturing) > 0)
+		return -EBUSY;
+
+	// First try to allocate sliced VBI buffers if needed.
+	if (fmt && itv->vbi_sliced_mpeg_data[0] == NULL) {
+		int i;
+
+		for (i = 0; i < IVTV_VBI_FRAMES; i++) {
+			// Yuck, hardcoded. Needs to be a define
+			itv->vbi_sliced_mpeg_data[i] =
+				kmalloc(2049, GFP_KERNEL);
+			if (itv->vbi_sliced_mpeg_data[i] == NULL) {
+				while (--i >= 0) {
+					kfree(itv->vbi_sliced_mpeg_data[i]);
+					itv->vbi_sliced_mpeg_data[i] = NULL;
+				}
+				return -ENOMEM;
+			}
+		}
+	}
+
+	itv->vbi_insert_mpeg = fmt;
+
+	if (itv->vbi_insert_mpeg == 0) {
+		return 0;
+	}
+	/* Need sliced data for mpeg insertion */
+	if (get_service_set(itv->vbi_sliced_in) == 0) {
+		if (itv->is_60hz)
+			itv->vbi_sliced_in->service_set = V4L2_SLICED_CAPTION_525;
+		else
+			itv->vbi_sliced_in->service_set = V4L2_SLICED_WSS_625;
+		expand_service_set(itv->vbi_sliced_in, itv->is_50hz);
+	}
+	return 0;
+}
+
 int ivtv_control_ioctls(struct ivtv *itv, unsigned int cmd, void *arg)
 {
+	struct v4l2_control ctrl;
+
 	switch (cmd) {
 	case VIDIOC_QUERYMENU:
 	        IVTV_DEBUG_IOCTL("VIDIOC_QUERYMENU\n");
@@ -475,6 +311,98 @@ int ivtv_control_ioctls(struct ivtv *itv, unsigned int cmd, void *arg)
 	case VIDIOC_G_CTRL:
 		return ivtv_g_ctrl(itv, arg);
 
+	case VIDIOC_S_EXT_CTRLS:
+	{
+		struct v4l2_ext_controls *c = arg;
+
+		if (c->ctrl_class == V4L2_CTRL_CLASS_USER) {
+			int i;
+			int err = 0;
+
+			for (i = 0; i < c->count; i++) {
+				ctrl.id = c->controls[i].id;
+				ctrl.value = c->controls[i].value;
+				err = ivtv_s_ctrl(itv, &ctrl);
+				c->controls[i].value = ctrl.value;
+				if (err) {
+					c->error_idx = i;
+					break;
+				}
+			}
+			return err;
+		}
+	        IVTV_DEBUG_IOCTL("VIDIOC_S_EXT_CTRLS\n");
+		if (c->ctrl_class == V4L2_CTRL_CLASS_MPEG) {
+			struct cx2341x_mpeg_params p = itv->params;
+			int err = cx2341x_ext_ctrls(&p, arg, cmd);
+
+			if (err)
+				return err;
+
+			if (p.video_encoding != itv->params.video_encoding) {
+				int is_mpeg1 = p.video_encoding ==
+				       		V4L2_MPEG_VIDEO_ENCODING_MPEG_1;
+				struct v4l2_format fmt;
+
+				/* fix videodecoder resolution */
+				fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+				fmt.fmt.pix.width = itv->params.width / (is_mpeg1 ? 2 : 1);
+				fmt.fmt.pix.height = itv->params.height;
+//				if (atomic_read(&itv->capturing) > 0) {
+//					IVTV_DEBUG_WARN(
+//							"v4l2 ioctl: set size failed, "
+//							"capture in progress\n");
+//				} else {
+					itv->video_dec_func(itv, VIDIOC_S_FMT, &fmt);
+//				}
+			}
+			err = cx2341x_update(itv, ivtv_api_func, &itv->params, &p);
+			if (!err && itv->params.stream_vbi_fmt != p.stream_vbi_fmt) {
+				err = ivtv_setup_vbi_fmt(itv, p.stream_vbi_fmt);
+			}
+			itv->params = p;
+			ivtv_audio_set_audio_clock_freq(itv, p.audio_properties & 0x03);
+			return err;
+		}
+		return -EINVAL;
+	}
+
+	case VIDIOC_G_EXT_CTRLS:
+	{
+		struct v4l2_ext_controls *c = arg;
+
+		if (c->ctrl_class == V4L2_CTRL_CLASS_USER) {
+			int i;
+			int err = 0;
+
+			for (i = 0; i < c->count; i++) {
+				ctrl.id = c->controls[i].id;
+				ctrl.value = c->controls[i].value;
+				err = ivtv_g_ctrl(itv, &ctrl);
+				c->controls[i].value = ctrl.value;
+				if (err) {
+					c->error_idx = i;
+					break;
+				}
+			}
+			return err;
+		}
+	        IVTV_DEBUG_IOCTL("VIDIOC_G_EXT_CTRLS\n");
+		if (c->ctrl_class == V4L2_CTRL_CLASS_MPEG)
+			return cx2341x_ext_ctrls(&itv->params, arg, cmd);
+		return -EINVAL;
+	}
+
+	case VIDIOC_TRY_EXT_CTRLS:
+	{
+		struct v4l2_ext_controls *c = arg;
+
+	        IVTV_DEBUG_IOCTL("VIDIOC_TRY_EXT_CTRLS\n");
+		if (c->ctrl_class == V4L2_CTRL_CLASS_MPEG)
+			return cx2341x_ext_ctrls(&itv->params, arg, cmd);
+		return -EINVAL;
+	}
+
 	default:
 		return -EINVAL;
 	}
