@@ -50,7 +50,6 @@
 #include "ivtv-queue.h"
 #include "ivtv-irq.h"
 #include "ivtv-mailbox.h"
-#include "ivtv-reset.h"
 #include "ivtv-streams.h"
 #include "ivtv-ioctl.h"
 #include "ivtv-kthreads.h"
@@ -68,26 +67,6 @@
 
 #include <media/tveeprom.h>
 
-/* Version info*/
-#define IVTV_DEFINE_VERSION_INTERNAL(name, major, minor, patchlevel, comment) \
-	uint32_t IVTV_VERSION_NUMBER_INTERNAL(name) = \
-		((major << 16) | (minor << 8) | (patchlevel)); \
-	const char * const IVTV_VERSION_STRING_INTERNAL(name) = \
-		#major"."#minor"."#patchlevel;\
-	const char * const IVTV_VERSION_COMMENT_INTERNAL(name) = \
-		comment;
-
-#define IVTV_DEFINE_VERSION(name, major, minor, patchlevel, comment) \
-		IVTV_DEFINE_VERSION_INTERNAL(name, major, minor, patchlevel, comment)
-
-IVTV_DEFINE_VERSION(IVTV_VERSION_INFO_NAME,
-		    IVTV_DRIVER_VERSION_MAJOR,
-		    IVTV_DRIVER_VERSION_MINOR,
-		    IVTV_DRIVER_VERSION_PATCHLEVEL,
-		    IVTV_DRIVER_VERSION_COMMENT);
-
-/* mini header */
-
 /* var to keep track of the number of array elements in use */
 int ivtv_cards_active = 0;
 
@@ -147,16 +126,6 @@ static int dec_yuv_buffers = IVTV_DEFAULT_DEC_YUV_BUFFERS;
 static int dec_vbi_buffers = IVTV_DEFAULT_DEC_VBI_BUFFERS;
 static int dec_osd_buffers = IVTV_DEFAULT_DEC_OSD_BUFFERS;
 
-static int max_mpg_buffers = IVTV_MAX_MPG_BUFFERS;
-static int max_yuv_buffers = IVTV_MAX_YUV_BUFFERS;
-static int max_vbi_buffers = IVTV_MAX_VBI_BUFFERS;
-static int max_pcm_buffers = IVTV_MAX_PCM_BUFFERS;
-static int max_dec_mpg_buffers = IVTV_MAX_DEC_MPG_BUFFERS;
-static int max_dec_yuv_buffers = IVTV_MAX_DEC_YUV_BUFFERS;
-static int max_dec_vbi_buffers = IVTV_MAX_DEC_VBI_BUFFERS;
-static int max_dec_osd_buffers = IVTV_MAX_DEC_OSD_BUFFERS;
-
-static int ivtv_dynbuf = 0;
 static int ivtv_yuv_mode = 0;
 static int ivtv_yuv_threshold=480;
 static int ivtv_pci_latency = 1;
@@ -167,7 +136,7 @@ char *ivtv_dfw = NULL;
 
 int ivtv_debug = IVTV_DBGFLG_WARN;
 
-int newi2c = 1;
+int newi2c = -1;
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
 module_param(yuv_fixup, int, 0644);
@@ -184,7 +153,6 @@ module_param_string(pal, pal, sizeof(pal), 0644);
 module_param_string(secam, secam, sizeof(secam), 0644);
 module_param_string(ntsc, ntsc, sizeof(ntsc), 0644);
 module_param(ivtv_debug, int, 0644);
-module_param(ivtv_dynbuf, int, 0644);
 module_param(ivtv_pci_latency, int, 0644);
 module_param(ivtv_yuv_mode, int, 0644);
 module_param(ivtv_yuv_threshold, int, 0644);
@@ -201,14 +169,6 @@ module_param(dec_yuv_buffers, int, 0644);
 module_param(dec_vbi_buffers, int, 0644);
 module_param(dec_osd_buffers, int, 0644);
 
-module_param(max_mpg_buffers, int, 0644);
-module_param(max_yuv_buffers, int, 0644);
-module_param(max_vbi_buffers, int, 0644);
-module_param(max_pcm_buffers, int, 0644);
-module_param(max_dec_mpg_buffers, int, 0644);
-module_param(max_dec_yuv_buffers, int, 0644);
-module_param(max_dec_vbi_buffers, int, 0644);
-module_param(max_dec_osd_buffers, int, 0644);
 module_param(newi2c, int, 0644);
 #else
 MODULE_PARM(yuv_fixup, "i");
@@ -219,7 +179,6 @@ MODULE_PARM(pal, "s");
 MODULE_PARM(secam, "s");
 MODULE_PARM(ntsc, "s");
 MODULE_PARM(ivtv_debug, "i");
-MODULE_PARM(ivtv_dynbuf, "i");
 MODULE_PARM(ivtv_pci_latency, "i");
 MODULE_PARM(ivtv_yuv_mode, "i");
 MODULE_PARM(ivtv_yuv_threshold, "i");
@@ -236,14 +195,6 @@ MODULE_PARM(dec_yuv_buffers, "i");
 MODULE_PARM(dec_vbi_buffers, "i");
 MODULE_PARM(dec_osd_buffers, "i");
 
-MODULE_PARM(max_mpg_buffers, "i");
-MODULE_PARM(max_yuv_buffers, "i");
-MODULE_PARM(max_vbi_buffers, "i");
-MODULE_PARM(max_pcm_buffers, "i");
-MODULE_PARM(max_dec_mpg_buffers, "i");
-MODULE_PARM(max_dec_yuv_buffers, "i");
-MODULE_PARM(max_dec_vbi_buffers, "i");
-MODULE_PARM(max_dec_osd_buffers, "i");
 MODULE_PARM(newi2c, "i");
 #endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0) */
 
@@ -284,9 +235,6 @@ MODULE_PARM_DESC(ntsc, "Set NTSC standard: M, J, K");
 MODULE_PARM_DESC(ivtv_debug,
 		 "Debug level (bitmask). Default: errors only\n"
 		 "\t\t\t(ivtv_debug = 511 gives full debugging)");
-MODULE_PARM_DESC(ivtv_dynbuf,
-		 "Specify using Dynamic Buffers: 0 = No, 1 = Yes,\n"
-		 "\t\t\tDefault: No");
 MODULE_PARM_DESC(ivtv_pci_latency,
 		 "Change the PCI latency to 64 if lower: 0 = No, 1 = Yes,\n"
 		 "\t\t\tDefault: Yes");
@@ -306,55 +254,32 @@ MODULE_PARM_DESC(ivtv_dfw,
 MODULE_PARM_DESC(mpg_buffers,
 		 "Enc MPG Buffers (in megs)\n"
 		 "\t\t\tDefault: " IVTV_DEFAULT_MPG_BUFFERS_CNT);
-MODULE_PARM_DESC(max_mpg_buffers,
-		 "Max Enc MPG Buffers (in megs)\n"
-		 "\t\t\tDefault: " IVTV_MAX_MPG_BUFFERS_CNT);
 MODULE_PARM_DESC(yuv_buffers,
 		 "Enc YUV Buffers (in megs)\n"
 		 "\t\t\tDefault: " IVTV_DEFAULT_YUV_BUFFERS_CNT);
-MODULE_PARM_DESC(max_yuv_buffers,
-		 "Max Enc YUV Buffers (in megs)\n"
-		 "\t\t\tDefault: " IVTV_MAX_YUV_BUFFERS_CNT);
 MODULE_PARM_DESC(vbi_buffers,
 		 "Enc VBI Buffers (in megs)\n"
 		 "\t\t\tDefault: " IVTV_DEFAULT_VBI_BUFFERS_CNT);
-MODULE_PARM_DESC(max_vbi_buffers,
-		 "Max Enc VBI Buffers (in megs)\n"
-		 "\t\t\tDefault: " IVTV_MAX_VBI_BUFFERS_CNT);
 MODULE_PARM_DESC(pcm_buffers,
 		 "Enc PCM buffers (in megs)\n"
 		 "\t\t\tDefault: " IVTV_DEFAULT_PCM_BUFFERS_CNT);
-MODULE_PARM_DESC(max_pcm_buffers,
-		 "Max Enc PCM Buffers (in megs)\n"
-		 "\t\t\tDefault: " IVTV_MAX_PCM_BUFFERS_CNT);
 MODULE_PARM_DESC(dec_mpg_buffers,
 		 "Dec MPG buffers (in megs)\n"
 		 "\t\t\tDefault: " IVTV_DEFAULT_DEC_MPG_BUFFERS_CNT);
-MODULE_PARM_DESC(max_dec_mpg_buffers,
-		 "Max Dec MPG Buffers (in megs)\n"
-		 "\t\t\tDefault: " IVTV_MAX_DEC_MPG_BUFFERS_CNT);
 MODULE_PARM_DESC(dec_yuv_buffers,
 		 "Dec YUV buffers (in megs)\n"
 		 "\t\t\tDefault: " IVTV_DEFAULT_DEC_YUV_BUFFERS_CNT);
-MODULE_PARM_DESC(max_dec_yuv_buffers,
-		 "Max Dec YUV Buffers (in megs)\n"
-		 "\t\t\tDefault: " IVTV_MAX_DEC_YUV_BUFFERS_CNT);
 MODULE_PARM_DESC(dec_vbi_buffers,
 		 "Dec VBI buffers (in megs)\n"
 		 "\t\t\tDefault: " IVTV_DEFAULT_DEC_VBI_BUFFERS_CNT);
-MODULE_PARM_DESC(max_dec_vbi_buffers,
-		 "Max Dec VBI Buffers (in megs)\n"
-		 "\t\t\tDefault: " IVTV_MAX_DEC_VBI_BUFFERS_CNT);
 MODULE_PARM_DESC(dec_osd_buffers,
 		 "Dec OSD buffers (in megs)\n"
 		 "\t\t\tDefault: " IVTV_DEFAULT_DEC_OSD_BUFFERS_CNT);
-MODULE_PARM_DESC(max_dec_osd_buffers,
-		 "Max Dec OSD Buffers (in megs)\n"
-		 "\t\t\tDefault: " IVTV_MAX_DEC_OSD_BUFFERS_CNT);
 
 MODULE_PARM_DESC(newi2c,
 		 "Use new I2C implementation\n"
-		 "\t\t\t default is 1 (yes)");
+		 "\t\t\t-1 is autodetect, 0 is off, 1 is on\n"
+		 "\t\t\tDefault is autodetect");
 
 MODULE_PARM_DESC(ivtv_first_minor, "Set minor assigned to first card");
 
@@ -362,7 +287,7 @@ MODULE_AUTHOR("Kevin Thayer, Chris Kennedy, Hans Verkuil");
 MODULE_DESCRIPTION("CX23415/CX23416 driver");
 MODULE_SUPPORTED_DEVICE
     ("CX23415/CX23416 MPEG2 encoder (WinTV PVR-150/250/350/500,\n"
-		"\t\t\t Yuan MPG series and similar)");
+		"\t\t\tYuan MPG series and similar)");
 MODULE_LICENSE("GPL");
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
@@ -464,40 +389,41 @@ void ivtv_read_eeprom(struct ivtv *itv, struct tveeprom *tv)
 
 static void ivtv_process_eeprom(struct ivtv *itv)
 {
-	const struct ivtv_card *card = itv->card;
 	struct tveeprom tv;
+	int pci_slot = PCI_SLOT(itv->dev->devfn);
 
 	ivtv_read_eeprom(itv, &tv);
 
-        /* if the model is a 23xxx (PVR500), and this is the 'PVR150' on
-           that card (subsystem device ID has bit 4 set), then the eeprom
-           reported incorrectly that a radio is present. A radio is only
-           available for the first 'PVR150'. 
-         
-           Many thanks to Hauppauge for providing this data. */
-        if (tv.model / 1000 == 23) {
-                int is_first = (itv->dev->subsystem_device & 0x0010) == 0;
-
-                IVTV_INFO("This is the %s unit of a PVR500\n",
-                                is_first ? "first" : "second");
-                itv->card_name = is_first ? "WinTV PVR 500 (unit #1)" :
-                                            "WinTV PVR 500 (unit #2)";
-                if (!is_first) {
-                        IVTV_INFO("Correcting tveeprom data: no radio present on second unit\n");
-                        tv.has_radio = 0;
-                }
-        }
-
+	/* Many thanks to Steven Toth from Hauppauge for providing the
+	   model numbers */
 	switch (tv.model) {
 		// In a few cases the PCI subsystem IDs do not correctly
 		// identify the card. A better method is to check the
 		// model number from the eeprom instead.
-                case 26000 ... 26199:
-			itv->card = ivtv_get_card(IVTV_CARD_PVR_150);
-			break;
-		case 32000 ... 32099:
+                case 32000 ... 32999:
+                case 48000 ... 48099:  /* 48??? range are PVR250s with a cx23415 */
+                case 48400 ... 48599:
 			itv->card = ivtv_get_card(IVTV_CARD_PVR_250);
 			break;
+                case 48100 ... 48399:
+                case 48600 ... 48999:
+			itv->card = ivtv_get_card(IVTV_CARD_PVR_350);
+			break;
+                case 23000 ... 23999:  /* PVR500 */
+                case 25000 ... 25999:  /* Low profile PVR150 */
+                case 26000 ... 26999:  /* Regular PVR150 */
+			itv->card = ivtv_get_card(IVTV_CARD_PVR_150);
+			break;
+		case 0:
+			IVTV_ERR("Invalid EEPROM\n");
+			return;
+		default:
+			IVTV_ERR("Unknown model %d, defaulting to PVR-150\n", tv.model);
+			itv->card = ivtv_get_card(IVTV_CARD_PVR_150);
+			break;
+	}
+
+	switch (tv.model) {
 		/* Old style PVR350 (with an saa7114) uses this input for
 		   the tuner. */
 		case 48254:
@@ -507,11 +433,35 @@ static void ivtv_process_eeprom(struct ivtv *itv)
 			break;
 	}
 
-	if (card != itv->card) {
-        	itv->card_name = itv->card->name;
-		IVTV_INFO("Corrected autodetection from %s to %s\n",
-				card->name, itv->card_name);
-	}
+	itv->v4l2_cap = itv->card->v4l2_capabilities;
+        itv->card_name = itv->card->name;
+
+        /* If this is a PVR500 then it should be possible to detect whether it is the
+	   first or second unit by looking at the subsystem device ID: is bit 4 is
+	   set, then it is the second unit (according to info from Hauppauge).
+
+	   However, while this works for most cards, I have seen a few PVR500 cards
+	   where both units have the same subsystem ID.
+
+	   So instead I look at the reported 'PCI slot' (which is the slot on the PVR500
+	   PCI bridge) and if it is 8, then it is assumed to be the first unit, otherwise
+	   it is the second unit. It is possible that it is a different slot when ivtv is
+	   used in Xen, in that case I ignore this card here. The worst that can happen
+	   is that the card presents itself with a non-working radio device.
+
+	   This detection is needed since the eeprom reports incorrectly that a radio is
+	   present on the second unit. */
+        if (tv.model / 1000 == 23 && (pci_slot == 8 || pci_slot == 9)) {
+                int is_first = (pci_slot & 1) == 0;
+
+                itv->card_name = is_first ? "WinTV PVR 500 (unit #1)" :
+                                            "WinTV PVR 500 (unit #2)";
+                if (!is_first) {
+                        IVTV_INFO("Correcting tveeprom data: no radio present on second unit\n");
+                        tv.has_radio = 0;
+                }
+        }
+	IVTV_INFO("Autodetected %s\n", itv->card_name);
 
 	switch (tv.tuner_hauppauge_model) {
 		case 85:
@@ -522,11 +472,22 @@ static void ivtv_process_eeprom(struct ivtv *itv)
 		default:
 			break;
 	}
+	if (tv.tuner_type == TUNER_ABSENT)
+		IVTV_ERR("tveeprom cannot autodetect tuner!");
 
 	if (itv->options.tuner == -1)
 		itv->options.tuner = tv.tuner_type;
 	if (itv->options.radio == -1)
 		itv->options.radio = (tv.has_radio != 0);
+	/* only enable newi2c if an IR blaster is present */
+	if (itv->options.newi2c == -1 && tv.has_ir != 2) {
+		itv->options.newi2c = (tv.has_ir & 2) ? 1 : 0;
+                if (itv->options.newi2c) {
+                    IVTV_INFO("reopen i2c bus for IR-blaster support\n");
+                    exit_ivtv_i2c(itv);
+                    init_ivtv_i2c(itv);
+                }
+	}
 
 	if (itv->std != 0)
                 // user specified tuner standard
@@ -535,16 +496,16 @@ static void ivtv_process_eeprom(struct ivtv *itv)
         /* autodetect tuner standard */
         if (tv.tuner_formats & V4L2_STD_PAL) {
                 IVTV_DEBUG_INFO("PAL tuner detected\n");
-                itv->std |= V4L2_STD_PAL;
+                itv->std |= V4L2_STD_PAL_BG | V4L2_STD_PAL_H;
         } else if (tv.tuner_formats & V4L2_STD_NTSC) {
                 IVTV_DEBUG_INFO("NTSC tuner detected\n");
-                itv->std |= V4L2_STD_NTSC;
+                itv->std |= V4L2_STD_NTSC_M;
         } else if (tv.tuner_formats & V4L2_STD_SECAM) {
                 IVTV_DEBUG_INFO("SECAM tuner detected\n");
-                itv->std |= V4L2_STD_SECAM;
+                itv->std |= V4L2_STD_SECAM_L;
         } else {
-                IVTV_INFO("No tuner detected, default to NTSC\n");
-                itv->std |= V4L2_STD_NTSC;
+                IVTV_INFO("No tuner detected, default to NTSC-M\n");
+                itv->std |= V4L2_STD_NTSC_M;
         }
 }
 
@@ -636,36 +597,37 @@ static void ivtv_process_options(struct ivtv *itv)
 	/* Buffers are now dynamically allocated, could specify limit though */
 	itv->options.yuv_fixup = yuv_fixup;
 	itv->options.mpg_buffers = mpg_buffers;
-	itv->options.max_mpg_buffers = max_mpg_buffers;
 	itv->options.yuv_buffers = yuv_buffers;
-	itv->options.max_yuv_buffers = max_yuv_buffers;
 	itv->options.vbi_buffers = vbi_buffers;
-	itv->options.max_vbi_buffers = max_vbi_buffers;
 	itv->options.pcm_buffers = pcm_buffers;
-	itv->options.max_pcm_buffers = max_pcm_buffers;
 	itv->options.dec_mpg_buffers = dec_mpg_buffers;
-	itv->options.max_dec_mpg_buffers = max_dec_mpg_buffers;
 	itv->options.dec_yuv_buffers = dec_yuv_buffers;
-	itv->options.max_dec_yuv_buffers = max_dec_yuv_buffers;
 	itv->options.dec_osd_buffers = dec_osd_buffers;
-	itv->options.max_dec_osd_buffers = max_dec_osd_buffers;
 	itv->options.dec_vbi_buffers = dec_vbi_buffers;
-	itv->options.max_dec_vbi_buffers = max_dec_vbi_buffers;
 	itv->options.cardtype = cardtype[itv->num];
 	itv->options.tuner = tuner[itv->num];
 	itv->options.radio = radio[itv->num];
-	itv->options.dynbuf = ivtv_dynbuf;
 	itv->options.newi2c = newi2c;
 
+	itv->std = ivtv_parse_std(itv);
         itv->has_cx23415 = (itv->dev->device == PCI_DEVICE_ID_IVTV15);
         chipname = itv->has_cx23415 ? "cx23415" : "cx23416";
 	if ((itv->card = ivtv_get_card(itv->options.cardtype - 1))) {
 		IVTV_INFO("User specified %s card (detected %s based chip)\n",
                                 itv->card->name, chipname);
-	} else {
-                if (itv->options.cardtype != 0) {
-		        IVTV_ERR("Unknown user specified type, trying to autodetect card\n");
-                }
+	} else if (itv->options.cardtype != 0) {
+	        IVTV_ERR("Unknown user specified type, trying to autodetect card\n");
+        }
+	if (itv->card == NULL) {
+		if (itv->dev->subsystem_vendor == IVTV_PCI_ID_HAUPPAUGE ||
+		    itv->dev->subsystem_vendor == IVTV_PCI_ID_HAUPPAUGE_ALT1 ||
+		    itv->dev->subsystem_vendor == IVTV_PCI_ID_HAUPPAUGE_ALT2) {
+			itv->card = ivtv_get_card(itv->has_cx23415 ? IVTV_CARD_PVR_350 : IVTV_CARD_PVR_150);
+			IVTV_INFO("Autodetected Hauppauge card (%s based)\n",
+					chipname);
+		}
+	}
+	if (itv->card == NULL) {
 		for (i = 0; (itv->card = ivtv_get_card(i)); i++) {
 			if (itv->card->pci_list == NULL)
 				continue;
@@ -684,24 +646,23 @@ static void ivtv_process_options(struct ivtv *itv)
 				goto done;
 			}
 		}
+	}
+done:
 
-	      done:
-		if (itv->card == NULL) {
-			itv->card = ivtv_get_card(IVTV_CARD_PVR_150);
-			IVTV_ERR("Unknown card: vendor/device: %04x/%04x\n",
-			     itv->dev->vendor, itv->dev->device);
-                        IVTV_ERR("              subsystem vendor/device: %04x/%04x\n",
-			     itv->dev->subsystem_vendor, itv->dev->subsystem_device);
-                        IVTV_ERR("              %s based\n", chipname);
-			IVTV_ERR("Defaulting to %s card\n", itv->card->name);
-			IVTV_ERR("Please mail the vendor/device and subsystem vendor/device IDs and what kind of\n");
-			IVTV_ERR("card you have to the ivtv-devel mailinglist (www.ivtvdriver.org)\n");
-			IVTV_ERR("Prefix your subject line with [UNKNOWN CARD].\n");
-		}
+	if (itv->card == NULL) {
+		itv->card = ivtv_get_card(IVTV_CARD_PVR_150);
+		IVTV_ERR("Unknown card: vendor/device: %04x/%04x\n",
+		     itv->dev->vendor, itv->dev->device);
+		IVTV_ERR("              subsystem vendor/device: %04x/%04x\n",
+		     itv->dev->subsystem_vendor, itv->dev->subsystem_device);
+		IVTV_ERR("              %s based\n", chipname);
+		IVTV_ERR("Defaulting to %s card\n", itv->card->name);
+		IVTV_ERR("Please mail the vendor/device and subsystem vendor/device IDs and what kind of\n");
+		IVTV_ERR("card you have to the ivtv-devel mailinglist (www.ivtvdriver.org)\n");
+		IVTV_ERR("Prefix your subject line with [UNKNOWN CARD].\n");
 	}
 	itv->v4l2_cap = itv->card->v4l2_capabilities;
         itv->card_name = itv->card->name;
-	itv->std = ivtv_parse_std(itv);
 }
 
 /* Precondition: the ivtv structure has been memset to 0. Only
@@ -744,21 +705,9 @@ static int __devinit ivtv_init_struct1(struct ivtv *itv)
 	itv->open_id = 1;
 
 	/* Initial settings */
-	itv->codec.bitrate_mode = 0;
-	itv->codec.bitrate = 8000000;
-	itv->codec.bitrate_peak = 9600000;
-	itv->codec.stream_type = IVTV_STREAM_DVD_S2;
-	itv->codec.bframes = 3;
-	itv->codec.gop_closure = 1;
-	itv->codec.dnr_mode = 0;
-	itv->codec.dnr_type = 0;
-	itv->codec.dnr_spatial = 0;
-	itv->codec.dnr_temporal = 8;
-	itv->codec.aspect = 2;
-	itv->codec.pulldown = 0;
-	/* Freq 48 kHz, Layer II, 384 kbit/s */
-	itv->codec.audio_bitmask = 1 | (2 << 2) | (14 << 4);
-
+	cx2341x_fill_defaults(&itv->params);
+	itv->params.port = CX2341X_PORT_MEMORY;
+	itv->params.capabilities = CX2341X_CAP_HAS_SLICED_VBI;
 	init_waitqueue_head(&itv->cap_w);
 	init_waitqueue_head(&itv->vsync_w_fs);
 	init_waitqueue_head(&itv->vsync_w_osd);
@@ -779,13 +728,6 @@ static int __devinit ivtv_init_struct1(struct ivtv *itv)
 
 	itv->dma_cfg.dec_vbi_buf_size = IVTV_DMA_DEC_VBI_BUF_SIZE;
 
-	itv->dma_cfg.max_mpg_buf = max_mpg_buffers;
-	itv->dma_cfg.max_yuv_buf = max_yuv_buffers;
-	itv->dma_cfg.max_vbi_buf = max_vbi_buffers;
-	itv->dma_cfg.max_pcm_buf = max_pcm_buffers;
-	itv->dma_cfg.max_dec_mpg_buf = max_dec_mpg_buffers;
-	itv->dma_cfg.max_dec_yuv_buf = max_dec_yuv_buffers;
-
 	itv->dma_cfg.fw_dec_dma_xfer = FW_DEC_DMA_XFER;
 	itv->dma_cfg.fw_enc_dma_xfer = FW_ENC_DMA_XFER;
 	itv->dma_cfg.fw_enc_dma_type = FW_ENC_DMA_TYPE;
@@ -950,6 +892,10 @@ static int ivtv_setup_pci(struct ivtv *itv, struct pci_dev *dev,
 		pci_write_config_byte(dev, PCI_LATENCY_TIMER, 64);
 		pci_read_config_byte(dev, PCI_LATENCY_TIMER, &pci_latency);
 	}
+	/* This config space value relates to DMA latencies. The
+	   default value 0x8080 is too low however and will lead 
+	   to DMA errors. 0xffff is the max value which solves
+	   these problems. */
 	pci_write_config_dword(dev, 0x40, 0xffff);
 
 	IVTV_DEBUG_INFO("%d (rev %d) at %02x:%02x.%x, "
@@ -977,38 +923,47 @@ static void ivtv_load_and_init_modules(struct ivtv *itv)
 	int i;
 
 	/* load modules */
-#ifndef CONFIG_VIDEO_TUNER
-	if (hw & IVTV_HW_TUNER) {
+#ifdef CONFIG_VIDEO_TUNER_MODULE
+	if (hw & IVTV_HW_TUNER)
 		ivtv_request_module(itv, "tuner");
-		ivtv_request_module(itv, "tda9887");
-	}
 #endif
-
-#ifndef CONFIG_VIDEO_DECODER
+#ifdef CONFIG_VIDEO_CX25840_MODULE
 	if (hw & IVTV_HW_CX25840)
 		ivtv_request_module(itv, "cx25840");
+#endif
+#ifdef CONFIG_VIDEO_SAA711X_MODULE
 	if (hw & IVTV_HW_SAA711X)
 		ivtv_request_module(itv, "saa7115");
+#endif
+#ifdef CONFIG_VIDEO_SAA7127_MODULE
 	if (hw & IVTV_HW_SAA7127)
 		ivtv_request_module(itv, "saa7127");
+#endif
 	if (hw & IVTV_HW_SAA717X)
 		ivtv_request_module(itv, "saa717x");
+#ifdef CONFIG_VIDEO_UPD64031A_MODULE
 	if (hw & IVTV_HW_UPD64031A)
 		ivtv_request_module(itv, "upd64031a");
+#endif
+#ifdef CONFIG_VIDEO_UPD64083_MODULE
 	if (hw & IVTV_HW_UPD6408X)
 		ivtv_request_module(itv, "upd64083");
 #endif
-#ifndef CONFIG_VIDEO_BT848
+#ifdef CONFIG_VIDEO_MSP3400_MODULE
 	if (hw & IVTV_HW_MSP34XX)
 		ivtv_request_module(itv, "msp3400");
+#endif
 	if (hw & IVTV_HW_TVAUDIO)
 		ivtv_request_module(itv, "tvaudio");
-#endif
-#ifndef CONFIG_VIDEO_AUDIO_DECODER
+#ifdef CONFIG_VIDEO_WM8775_MODULE
 	if (hw & IVTV_HW_WM8775)
 		ivtv_request_module(itv, "wm8775");
+#endif
+#ifdef CONFIG_VIDEO_WM8739_MODULE
 	if (hw & IVTV_HW_WM8739)
 		ivtv_request_module(itv, "wm8739");
+#endif
+#ifdef CONFIG_VIDEO_CS53L32A_MODULE
 	if (hw & IVTV_HW_CS53L32A)
 		ivtv_request_module(itv, "cs53l32a");
 #endif
@@ -1091,7 +1046,6 @@ static int __devinit ivtv_probe(struct pci_dev *dev,
 {
 	int retval = 0;
 	int video_input;
-	int cfg, top;
 	struct ivtv *itv;
         struct v4l2_frequency vf;
 
@@ -1177,6 +1131,14 @@ static int __devinit ivtv_probe(struct pci_dev *dev,
 		goto free_io;
 	}
 
+	/* write firmware */
+	retval = ivtv_firmware_init(itv);
+	if (retval) {
+		IVTV_ERR("Error %d initializing firmware.\n", retval);
+		retval = -ENOMEM;
+		goto free_irq;
+	}
+
 	itv->gpio_dir = itv->reg_mem + IVTV_REG_GPIO_DIR_OFFSET;
 	itv->gpio_out = itv->reg_mem + IVTV_REG_GPIO_OUT_OFFSET;
 	itv->gpio_in  = itv->reg_mem + IVTV_REG_GPIO_IN_OFFSET;
@@ -1192,16 +1154,16 @@ static int __devinit ivtv_probe(struct pci_dev *dev,
 	IVTV_DEBUG_INFO("Active card count: %d.\n", ivtv_cards_active);
 
 	if (itv->card->hw_all & IVTV_HW_TVEEPROM) {
-#ifndef CONFIG_VIDEO_TVEEPROM
+#ifdef CONFIG_VIDEO_TVEEPROM_MODULE
 		ivtv_request_module(itv, "tveeprom");
-#endif /* CONFIG_VIDEO_TVEEPROM */
+#endif
                 // Based on the model number the cardtype may be changed.
                 // The PCI IDs are not always reliable.
 		ivtv_process_eeprom(itv);
 	}
 
 	if (itv->std == 0) {
-		itv->std = V4L2_STD_NTSC;
+		itv->std = V4L2_STD_NTSC_M;
 	}
 
         if (itv->options.tuner == -1) {
@@ -1233,7 +1195,7 @@ static int __devinit ivtv_probe(struct pci_dev *dev,
         } else {
                 itv->is_50hz = 1;
         }
-	itv->codec.framespergop = itv->is_60hz ? 15 : 12;
+	itv->params.video_gop_size = itv->is_60hz ? 15 : 12;
 
         /* Setup YUV Encoding/Decoding Buffers */
         if (itv->is_60hz) {
@@ -1266,14 +1228,6 @@ static int __devinit ivtv_probe(struct pci_dev *dev,
 	if (itv->options.radio > 0)
 		itv->v4l2_cap |= V4L2_CAP_RADIO;
 
-	/* write firmware */
-	retval = ivtv_firmware_init(itv);
-	if (retval) {
-		IVTV_ERR("Error %d initializing firmware.\n", retval);
-		retval = -ENOMEM;
-		goto free_i2c;
-	}
-
 	/* search for encoder/decoder mailboxes */
 	IVTV_DEBUG_INFO("About to search for mailboxes\n");
 	ivtv_find_firmware_mailbox(itv);
@@ -1340,7 +1294,7 @@ static int __devinit ivtv_probe(struct pci_dev *dev,
 			     SA_SHIRQ | SA_INTERRUPT, itv->name, (void *)itv);
 	if (retval) {
 		IVTV_ERR("Failed to register irq %d\n", retval);
-		goto free_i2c;
+		goto free_streams;
 	}
 
 	/* On a cx23416 this seems to be able to enable DMA to the chip? */
@@ -1387,32 +1341,6 @@ static int __devinit ivtv_probe(struct pci_dev *dev,
 	   in one place. */
 	itv->std++;		// Force full standard initialization
 	ivtv_v4l2_ioctls(itv, NULL, 0, VIDIOC_S_STD, &itv->tuner_std);
-	top = 0;
-
-	switch (itv->options.tuner) {
-	case TUNER_PHILIPS_FM1216ME_MK3:
-	case TUNER_PHILIPS_FM1236_MK3:
-                cfg = TDA9887_PORT1_ACTIVE|TDA9887_PORT2_ACTIVE|TDA9887_TOP(top);
-                ivtv_call_i2c_clients(itv, TDA9887_SET_CONFIG, &cfg);
-		itv->set_fm_high_sensitivity = 1;
-		break;
-
-	case TUNER_PHILIPS_FQ1216AME_MK4:
-		if (itv->tuner_std == V4L2_STD_SECAM_L ||
-		    itv->tuner_std == V4L2_STD_SECAM_LC)
-			top = -2;
-		/* fall through */
-	case TUNER_PHILIPS_FQ1236A_MK4:
-                cfg = TDA9887_PORT1_ACTIVE|TDA9887_PORT2_INACTIVE|TDA9887_TOP(top);
-                ivtv_call_i2c_clients(itv, TDA9887_SET_CONFIG, &cfg);
-		itv->set_fm_high_sensitivity = 1;
-		break;
-
-	case TUNER_PANASONIC_VP27:
-		cfg = TDA9887_INTERCARRIER;
-                ivtv_call_i2c_clients(itv, TDA9887_SET_CONFIG, &cfg);
-		break;
-	}
 	ivtv_v4l2_ioctls(itv, NULL, 0, VIDIOC_S_FREQUENCY, &vf);
 
 	IVTV_INFO("Initialized %s, card #%d\n", itv->card_name, itv->num);
@@ -1421,6 +1349,8 @@ static int __devinit ivtv_probe(struct pci_dev *dev,
 
       free_irq:
 	free_irq(itv->dev->irq, (void *)itv);
+      free_streams:
+	ivtv_streams_cleanup(itv);
       free_i2c:
 	exit_ivtv_i2c(itv);
       free_io:
@@ -1432,6 +1362,8 @@ static int __devinit ivtv_probe(struct pci_dev *dev,
 	destroy_workqueue(itv->vbi_work_queues);
 #endif
       err:
+	if (retval == 0)
+		retval = -ENODEV;
 	IVTV_ERR("Error %d on initialization\n", retval);
 
 	spin_lock(&ivtv_cards_lock);
@@ -1551,9 +1483,7 @@ static struct pci_driver ivtv_pci_driver = {
 static int module_start(void)
 {
 	printk(KERN_INFO "ivtv:  ==================== START INIT IVTV ====================\n");
-	printk(KERN_INFO "ivtv:  version %s %s loading\n",
-		       IVTV_VERSION_STRING(IVTV_VERSION_INFO_NAME),
-		       IVTV_VERSION_COMMENT(IVTV_VERSION_INFO_NAME));
+	printk(KERN_INFO "ivtv:  version %s loading\n", IVTV_VERSION);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	printk(KERN_INFO "ivtv:  Linux version: " VERMAGIC_STRING "\n");
 #else
