@@ -38,11 +38,8 @@
 #include "ivtv-compat.h"
 
 #include <linux/module.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
 #include <linux/moduleparam.h>
-#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0) */
 #include <linux/init.h>
-#include <linux/config.h>
 #include <linux/delay.h>
 #include <linux/sched.h>
 #include <linux/fs.h>
@@ -61,7 +58,7 @@
 
 #include <media/v4l2-common.h>
 #include <media/tuner.h>
-#include "cx2341x.h"
+#include <media/cx2341x.h>
 
 #define IVTV_INTERNAL
 #include "ivtv.h"
@@ -137,9 +134,16 @@ extern const u32 yuv_offset[4];
 #define IVTV_CARD_CX23416GYC_NOGR    19	/* Kuroutoshikou CX23416GYC-STVLP (Yuan MPG600GR OEM) */
 #define IVTV_CARD_CX23416GYC_NOGRYCS 20	/* Kuroutoshikou CX23416GYC-STVLP (Yuan MPG600GR OEM) */
 
-#define IVTV_AUDIO_32000 0x2	/* 32 kHz audio */
-#define IVTV_AUDIO_44100 0x0	/* 44.1 kHz audio */
-#define IVTV_AUDIO_48000 0x1	/* 48 kHz audio */
+#define IVTV_ENC_STREAM_TYPE_MPG 0
+#define IVTV_ENC_STREAM_TYPE_YUV 1
+#define IVTV_ENC_STREAM_TYPE_VBI 2
+#define IVTV_ENC_STREAM_TYPE_PCM 3
+#define IVTV_ENC_STREAM_TYPE_RAD 4
+#define IVTV_DEC_STREAM_TYPE_MPG 5
+#define IVTV_DEC_STREAM_TYPE_VBI 6
+#define IVTV_DEC_STREAM_TYPE_VOUT 7
+#define IVTV_DEC_STREAM_TYPE_YUV 8
+#define IVTV_DEC_STREAM_TYPE_OSD 9
 
 #define IVTV_V4L2_DEC_MPG_OFFSET 16	/* offset from 0 to register decoder mpg v4l2 minors on */
 #define IVTV_V4L2_ENC_PCM_OFFSET 24	/* offset from 0 to register pcm v4l2 minors on */
@@ -167,7 +171,6 @@ extern const u32 yuv_offset[4];
 #define IVTV_PCI_ID_YUAN3 		0xffab
 #define IVTV_PCI_ID_YUAN4 		0xfbab
 #define IVTV_PCI_ID_DIAMONDMM 		0xff92
-#define IVTV_PCI_ID_XTREME550 		0x0070
 #define IVTV_PCI_ID_IODATA 		0x10fc
 #define IVTV_PCI_ID_MELCO 		0x1154
 #define IVTV_PCI_ID_GOTVIEW 		0xffac
@@ -176,7 +179,6 @@ extern const u32 yuv_offset[4];
 #define IVTV_MBOX_API_BOXES 6
 #define IVTV_MBOX_DMA_START 6
 #define IVTV_MBOX_DMA_END 8
-#define IVTV_MBOX_MAX_DATA 16
 #define IVTV_MBOX_DMA 9
 #define IVTV_MBOX_FIELD_DISPLAYED 8
 #define IVTV_MBOX_SIZE 80
@@ -268,16 +270,6 @@ extern const u32 yuv_offset[4];
 #define IVTV_DEFAULT_DEC_VBI_BUFFERS_CNT "1"
 #define IVTV_DEFAULT_DEC_OSD_BUFFERS_CNT "1"
 
-/* DMA Buffers Limits */
-#define IVTV_MAX_MPG_BUFFERS_CNT "16"
-#define IVTV_MAX_YUV_BUFFERS_CNT "32"
-#define IVTV_MAX_VBI_BUFFERS_CNT "16"
-#define IVTV_MAX_PCM_BUFFERS_CNT "16"
-#define IVTV_MAX_DEC_MPG_BUFFERS_CNT "1"
-#define IVTV_MAX_DEC_YUV_BUFFERS_CNT "1"
-#define IVTV_MAX_DEC_VBI_BUFFERS_CNT "1"
-#define IVTV_MAX_DEC_OSD_BUFFERS_CNT "2"
-
 
 /* Decoder DMA Settings */
 struct ivtv_dma_settings {
@@ -298,14 +290,6 @@ struct ivtv_dma_settings {
 	int vbi_buf_size;
 	int dec_vbi_buf_size;
 
-	/* Max growable amount of buffers */
-	int max_yuv_buf;
-	int max_pcm_buf;
-	int max_mpg_buf;
-	int max_dec_yuv_buf;
-	int max_dec_mpg_buf;
-	int max_vbi_buf;
-
 	/* Chip DMA Xfer Settings */
 	int fw_dec_dma_xfer;
 	int fw_enc_dma_xfer;
@@ -327,8 +311,8 @@ struct ivtv_dec_dma {
 	u32 total_xfer;
 	u32 sg_bytes;
 	atomic_t type;		/* 0=mpg, 1=osd, 2=yuv */
-	u32 data[IVTV_MBOX_MAX_DATA];
-	u32 speed_data[IVTV_MBOX_MAX_DATA];
+	u32 data[CX2341X_MBOX_MAX_DATA];
+	u32 speed_data[CX2341X_MBOX_MAX_DATA];
 	atomic_t vsync_flag_osd;
 	unsigned long ts;
 	u32 wq_runs;
@@ -468,8 +452,9 @@ struct ivtv_dec_dma {
 
 /*Used for locating the firmware mailboxes*/
 #define IVTV_DECODE_INIT_MPEG_FILENAME "v4l-cx2341x-init.mpg"
+#define IVTV_DECODE_INIT_MPEG_SIZE (152*1024)
 
-#define IVTV_FIRM_IMAGE_SIZE 256*1024
+#define IVTV_FIRM_IMAGE_SIZE (256*1024)
 #define IVTV_FIRM_SEARCH_ENCODER_START 0x00000000
 #define IVTV_FIRM_SEARCH_DECODER_START 0x00000000
 #define IVTV_FIRM_SEARCH_ENCODER_END (IVTV_ENCODER_OFFSET + IVTV_ENCODER_SIZE - 1)
@@ -554,25 +539,16 @@ extern int ivtv_debug;
 struct ivtv_options {
 	int yuv_fixup;		/* Should we re-work YUV to a standard format? */
 	int yuv_buffers;	/* How many yuv buffers to allocate? */
-	int max_yuv_buffers;	/* How many yuv buffers to allocate? */
 	int pcm_buffers;	/* How many pcm buffers to allocate? */
-	int max_pcm_buffers;	/* How many pcm buffers to allocate? */
 	int mpg_buffers;	/* how many mpg buffers to allocate? */
-	int max_mpg_buffers;	/* how many mpg buffers to allocate? */
 	int vbi_buffers;	/* how many vbi buffers to allocate? */
-	int max_vbi_buffers;	/* how many vbi buffers to allocate? */
 	int dec_vbi_buffers;	/* how many vbi buffers to allocate? */
-	int max_dec_vbi_buffers;	/* how many vbi buffers to allocate? */
 	int dec_mpg_buffers;	/* how many decoder mpg buffers to allocate? */
-	int max_dec_mpg_buffers;	/* how many decoder mpg buffers to allocate? */
 	int dec_yuv_buffers;	/* How many decoder yuv buffers to allocate? */
-	int max_dec_yuv_buffers;	/* How many decoder yuv buffers to allocate? */
 	int dec_osd_buffers;	/* How many decoder osd buffers to allocate? */
-	int max_dec_osd_buffers;	/* How many decoder osd buffers to allocate? */
 	int cardtype;		/* force card type on load */
 	int tuner;		/* set tuner on load */
 	int radio;		/* enable/disable radio */
-	int dynbuf;		/* Enable/Disable dynamic buffers */
 	int newi2c;		/* New I2C algorithm */
 };
 
@@ -598,7 +574,7 @@ struct ivtv_mailbox {
 	u32 cmd;
 	u32 retval;
 	u32 timeout;
-	u32 data[IVTV_MBOX_MAX_DATA];
+	u32 data[CX2341X_MBOX_MAX_DATA];
 };
 
 /* per buffer bit flags */
@@ -715,9 +691,10 @@ struct ivtv_stream {
 
 	/* Buffer Stats */
 	atomic_t allocated_buffers;
+	u32 buf_backup;
+	u32 last_offset;
 	int buffers;
 	u32 buf_min;
-	u32 buf_max;
 	int bufsize;
 	u32 buf_total;
 	u32 buf_fill;
@@ -791,8 +768,8 @@ struct api_cmd {
 	int marked;		/* is this used */
 	unsigned long jiffies;		/* last command issued */
 
-	u32 s_data[IVTV_MBOX_MAX_DATA];	/* send api data */
-	u32 r_data[IVTV_MBOX_MAX_DATA];	/* returned api data */
+	u32 s_data[CX2341X_MBOX_MAX_DATA];	/* send api data */
+	u32 r_data[CX2341X_MBOX_MAX_DATA];	/* returned api data */
 };
 
 struct yuv_playback_info
@@ -882,7 +859,6 @@ struct ivtv {
         u8 is_50hz; 
         u8 is_60hz;
 	u8 pvr150_workaround;
-	u8 set_fm_high_sensitivity; /* the tuner requires PORT1=1 for radio */
 	u8 nof_inputs;
 	u8 nof_audio_inputs;
 	u32 v4l2_cap;		/* V4L2 capabilities of card */
@@ -1007,13 +983,13 @@ struct ivtv {
 	/* v4l2 and User settings */
 
 	/* codec settings */
-	struct ivtv_ioctl_codec codec;
+	struct cx2341x_mpeg_params params;
 	u32 audio_input;
 	u32 active_input;
 	u32 active_output;
 	v4l2_std_id std;
 	v4l2_std_id tuner_std;	/* The norm of the tuner (fixed) */
-	u32 width, height;
+	//u32 width, height;
 	u32 vbi_start[2], vbi_count;
 	int streamcount;	/* Number of elements in streams */
 	struct ivtv_stream *streams;
