@@ -23,7 +23,6 @@
 #include "ivtv-i2c.h"
 #include "ivtv-queue.h"
 #include "ivtv-irq.h"
-#include "ivtv-reset.h"
 #include "ivtv-vbi.h"
 #include "ivtv-mailbox.h"
 #include "ivtv-kthreads.h"
@@ -36,16 +35,6 @@
 
 typedef unsigned long uintptr_t;
 
-static int allocate_stream(struct ivtv *itv, int type)
-{
-	if (!itv->options.dynbuf) {
-                // statically allocated
-                return 0;
-        }
-	// allocate stream
-	return ivtv_stream_alloc(itv, type);
-}
-
 /* This function tries to claim the stream for a specific file descriptor.
    If no one else is using this stream then the stream is claimed, any
    memory is allocated and associated VBI streams are also automatically
@@ -77,10 +66,6 @@ int ivtv_claim_stream(struct ivtv_open_id *id, int type)
                 IVTV_DEBUG_INFO("Stream %d is busy\n", type);
                 return -EBUSY;
         }
-        if (allocate_stream(itv, type)) {
-                clear_bit(IVTV_F_S_IN_USE, &stream->s_flags);
-                return -ENOMEM;
-        }
 	stream->id = id->open_id;
 	if (type == IVTV_DEC_STREAM_TYPE_VBI) {
                 // Enable reinsertion interrupt
@@ -102,13 +87,6 @@ int ivtv_claim_stream(struct ivtv_open_id *id, int type)
 	vbi_stream = &itv->streams[vbi_type];
 
         if (!test_and_set_bit(IVTV_F_S_IN_USE, &vbi_stream->s_flags)) {
-                if (allocate_stream(itv, vbi_type)) {
-                        // could not allocate VBI stream, so free the lot
-                        clear_bit(IVTV_F_S_IN_USE, &vbi_stream->s_flags);
-                        ivtv_stream_free(itv, type);
-                        clear_bit(IVTV_F_S_IN_USE, &stream->s_flags);
-                        return -ENOMEM;
-                }
                 // Enable reinsertion interrupt
                 if (vbi_type == IVTV_DEC_STREAM_TYPE_VBI)
                         ivtv_clear_irq_mask(itv, IVTV_IRQ_DEC_VBI_RE_INSERT);
@@ -139,9 +117,6 @@ void ivtv_release_stream(struct ivtv *itv, int type)
         }
 
         ivtv_flush_queues_lock(itv, type);
-        if (itv->options.dynbuf) {
-                ivtv_stream_free(itv, type);
-        }
 
 	/* IVTV_DEC_STREAM_TYPE_MPG needs to release IVTV_DEC_STREAM_TYPE_VBI,
 	   IVTV_ENC_STREAM_TYPE_MPG needs to release IVTV_ENC_STREAM_TYPE_VBI,
@@ -169,9 +144,6 @@ void ivtv_release_stream(struct ivtv *itv, int type)
                 ivtv_set_irq_mask(itv, IVTV_IRQ_DEC_VBI_RE_INSERT);
         clear_bit(IVTV_F_S_IN_USE, &vbi_stream->s_flags);
         ivtv_flush_queues_lock(itv, vbi_type);
-
-        if (itv->options.dynbuf)
-                ivtv_stream_free(itv, vbi_type);
 }
 
 static int ivtv_check_digitizer_sync(struct ivtv *itv)
@@ -207,7 +179,7 @@ static int ivtv_reset_digitizer(struct ivtv *itv)
 	ivtv_set_irq_mask(itv, IVTV_IRQ_MASK_CAPTURE);
 
         /* Initialize Digitizer for Capture */
-        ivtv_init_digitizer(itv);
+        ivtv_vapi(itv, CX2341X_ENC_REFRESH_INPUT, 0);
 
         /*IVTV_DEBUG_INFO("Sleeping for 100ms\n"); */
         ivtv_sleep_timeout(HZ / 10, 0);
@@ -239,8 +211,8 @@ static int ivtv_YUV_fixup(struct ivtv *itv, struct ivtv_stream *st, int count,
 	int start;		/* position in buf to read from */
 	int buf_start;		/* byte offset of first byte in this *buf */
 
-	height = itv->height;
-	width = itv->width;
+	height = itv->params.height;
+	width = itv->params.width;
 	maxblock = (width + 0xf) >> 4;
 	maxline = (int)height + (height/2);	/* 1 for Y, .5 for UV */
 	/* Offset is always bufsize * buffer index 
@@ -344,20 +316,6 @@ static int gather_free_buffers(struct ivtv *itv, int streamtype,
 				IVTV_DEBUG_DMA(
 					   "%d: Got FREE Buff with %d bytes.\n",
 					   enc, buf->buffer.bytesused);
-			} else if (!buf) {
-				/* Too many allocated? */
-				if (st->buf_total >= st->buf_max)
-					break;
-
-				buf = ivtv_init_buffer(itv, st);
-				buf->buffer.index = x;
-				if (buf) {
-					IVTV_DEBUG_DMA(
-						   "%d: Alloc Buffer. (%d)\n",
-						   enc,
-						   atomic_read(&st->
-							       allocated_buffers));
-				}
 			}
 			if (!buf)
 				break;
@@ -866,7 +824,8 @@ ssize_t ivtv_v4l2_read(struct file * filp, char *buf, size_t count,
 
 	/* If capture is already in progress, then we also have to
 	   do nothing extra. */
-	if (test_and_set_bit(IVTV_F_S_CAPTURING, &stream->s_flags))
+	if (test_bit(IVTV_F_S_STREAMOFF, &stream->s_flags) ||
+	    test_and_set_bit(IVTV_F_S_CAPTURING, &stream->s_flags))
 		return ivtv_read(filp, buf, count, pos);
 
 	/* Start VBI capture if required */
@@ -946,7 +905,7 @@ static int ivtv_wait_dec_buffer(struct ivtv *itv, struct ivtv_stream *st,
 		      (ivtv_buf_fill_lock(st, 0, BUF_USED) >=
 		       itv->dma_cfg.dec_max_xfer &&
 		       !test_and_set_bit(IVTV_F_S_DECODING, &st->s_flags)))) ||
-		    (ivtv_buf_fill_lock(st, 0, BUF_USED) >= st->buf_max &&
+		    (ivtv_buf_fill_lock(st, 0, BUF_USED) >= st->buf_min &&
 		     test_bit(IVTV_F_S_OVERFLOW, &st->s_flags))) {
 			if (test_and_clear_bit(IVTV_F_S_OVERFLOW, &st->s_flags)) 
 			{
@@ -1182,9 +1141,9 @@ unsigned int ivtv_v4l2_dec_poll(struct file *filp, poll_table * wait)
 
 	/* If we need to start decoding, do so after the first write */
 	if ((!test_bit(FW_RESET_LOCK, &itv->r_flags) &&
-	     (ivtv_buf_fill_lock(st, 0, BUF_USED) >= st->buf_max &&
+	     (ivtv_buf_fill_lock(st, 0, BUF_USED) >= st->buf_min &&
 	      !test_and_set_bit(IVTV_F_S_DECODING, &st->s_flags))) ||
-	    (ivtv_buf_fill_lock(st, 0, BUF_USED) >= st->buf_max &&
+	    (ivtv_buf_fill_lock(st, 0, BUF_USED) >= st->buf_min &&
 	     test_bit(IVTV_F_S_OVERFLOW, &st->s_flags))) {
 		if (test_and_clear_bit(IVTV_F_S_OVERFLOW, &st->s_flags)) {
 			ivtv_stop_v4l2_decode_stream(itv, type);
@@ -1203,13 +1162,11 @@ unsigned int ivtv_v4l2_dec_poll(struct file *filp, poll_table * wait)
 
 	/* Allow write if buffers are not at the limit or some available */
 	if (ivtv_buf_fill_lock(st, 0, BUF_USED) <=
-	    st->buf_max - itv->dma_cfg.dec_min_xfer) {
+	    st->buf_min - itv->dma_cfg.dec_min_xfer) {
 		mask |= POLLOUT | POLLWRNORM;	/* Writable */
 	}
 
 	if (mask) {
-		IVTV_DEBUG_IRQ("DEC: Poll returning 0x%x\n", mask);
-
 		IVTV_DEBUG_INFO(
 			   "DEC: %d bufs, 0x%08x fill; %d free %d dma %d full %d io\n",
 			   atomic_read(&st->allocated_buffers),
@@ -1244,8 +1201,6 @@ unsigned int ivtv_v4l2_enc_poll(struct file *filp, poll_table * wait)
 	}
 
 	if (mask) {
-		IVTV_DEBUG_IRQ("ENC: Poll returning 0x%x\n", mask);
-
 		IVTV_DEBUG_INFO(
 			   "ENC: %d bufs, 0x%08x fill; %d free %d dma %d full %d io\n",
 			   atomic_read(&st->allocated_buffers),
@@ -1285,16 +1240,12 @@ int ivtv_v4l2_close(struct inode *inode, struct file *filp)
 
 	/* Stop radio */
 	if (id->type == IVTV_ENC_STREAM_TYPE_RAD) {
-                int cfg = TDA9887_PORT1_ACTIVE;
-
 		// Closing radio device, return to TV mode
 		ivtv_mute(itv);
 		/* Mark that the radio is no longer in use */
 		clear_bit(IVTV_F_I_RADIO_USER, &itv->i_flags);
 		/* Switch tuner to TV */
 		ivtv_call_i2c_clients(itv, VIDIOC_S_STD, &itv->std);
-		if (itv->set_fm_high_sensitivity)
-                	ivtv_call_i2c_clients(itv, TDA9887_SET_CONFIG, &cfg);
 		/* Select correct audio input (i.e. TV tuner or Line in) */
 		ivtv_audio_set_io(itv);
 		/* Done! Unmute and continue. */
@@ -1325,8 +1276,17 @@ int ivtv_v4l2_close(struct inode *inode, struct file *filp)
 				"close stopping embedded VBI capture\n");
 			ivtv_stop_capture(itv, IVTV_ENC_STREAM_TYPE_VBI);
 		}
-		ivtv_stop_capture(itv, id->type);
+		if ((id->type == IVTV_DEC_STREAM_TYPE_VBI ||
+		     id->type == IVTV_ENC_STREAM_TYPE_VBI) &&
+		    test_bit(IVTV_F_S_INTERNAL_USE, &stream->s_flags)) {
+			/* Also used internally, don't stop capturing */
+			stream->id = -1;
+		}
+		else {
+			ivtv_stop_capture(itv, id->type);
+		}
 	}
+	clear_bit(IVTV_F_S_STREAMOFF, &stream->s_flags);
 	if (id->type == IVTV_DEC_STREAM_TYPE_YUV  && test_bit(IVTV_F_S_DECODING_YUV, &stream->s_flags)) { /* Reset registers we have changed so mpeg playback works */
                 int h_filter, v_filter_1, v_filter_2;
 
@@ -1482,8 +1442,6 @@ int ivtv_v4l2_open(struct inode *inode, struct file *filp)
 	filp->private_data = item;
 
 	if (item->type == IVTV_ENC_STREAM_TYPE_RAD) {
-                int cfg = TDA9887_PORT1_INACTIVE;
-
 		/* Try to claim this stream */
 		if (ivtv_claim_stream(item, item->type)) {
 			/* No, it's already in use */
@@ -1495,8 +1453,6 @@ int ivtv_v4l2_open(struct inode *inode, struct file *filp)
 		ivtv_mute(itv);
 		/* Switch tuner to radio */
 		ivtv_call_i2c_clients(itv, AUDC_SET_RADIO, 0);
-		if (itv->set_fm_high_sensitivity)
-                	ivtv_call_i2c_clients(itv, TDA9887_SET_CONFIG, &cfg);
 		/* Mark that the radio is being used. */
 		set_bit(IVTV_F_I_RADIO_USER, &itv->i_flags);
 		/* Select the correct audio input (i.e. radio tuner) */
@@ -1551,8 +1507,9 @@ void ivtv_unmute(struct ivtv *itv)
         if (atomic_read(&itv->capturing) == 0)
                 ivtv_vapi(itv, CX2341X_ENC_INITIALIZE_INPUT, 0);
 
+	ivtv_sleep_timeout(HZ / 10, 0);
+
 	if (atomic_read(&itv->capturing)) {
-		ivtv_sleep_timeout(HZ / 10, 0);
 		ivtv_vapi(itv, CX2341X_ENC_MISC, 12);
 		ivtv_vapi(itv, CX2341X_ENC_MUTE_AUDIO, 0);
 	}
