@@ -84,31 +84,37 @@ int ivtv_check_firmware(struct ivtv *itv)
 static int load_fw_direct(const char *fn, char *mem, struct ivtv *itv, long size)
 {
 	const struct firmware *fw = NULL;
-	int retval = -ENOMEM;
+	int retries = 3;
 
-	if (request_firmware(&fw, fn, FWDEV(itv)) == 0) {
+retry:
+	if (retries && request_firmware(&fw, fn, FWDEV(itv)) == 0) {
 		int i;
 		u32 *dst = (u32 *)mem;
 		const u32 *src = (const u32 *)fw->data;
 
-		if (fw->size >= size) {
-			retval = size;
-		} else {
-			retval = fw->size;
+		if (fw->size < size) {
+			// Due to race conditions in firmware loading (esp. with udev <0.95)
+			// the wrong file was sometimes loaded. So we check filesizes to
+			// see if at least the right-sized file was loaded. If not, then we
+			// retry.
+			IVTV_INFO("retry: file loaded was not %s (expected size %ld, got %zd)\n", fn, size, fw->size);
+			release_firmware(fw);
+			retries--;
+			goto retry;
 		}
-		for (i = 0; i < retval; i += 4) {
-			writel(*src, dst);
+		for (i = 0; i < size; i += 4) {
+			/* no need for endianness conversion on the ppc */
+			__raw_writel(*src, dst);
 			dst++;
 			src++;
 		}
 		release_firmware(fw);
-		IVTV_INFO("loaded %s firmware (%d bytes)\n", fn, retval);
-	} else {
-		IVTV_INFO("unable to open firmware %s\n", fn);
-		IVTV_INFO("did you put the firmware in the hotplug firmware directory?\n");
+		IVTV_INFO("loaded %s firmware (%ld bytes)\n", fn, size);
+		return size;
 	}
-
-	return retval;
+	IVTV_INFO("unable to open firmware %s (must be %ld bytes)\n", fn, size);
+	IVTV_INFO("did you put the firmware in the hotplug firmware directory?\n");
+	return -ENOMEM;
 }
 
 static int ivtv_enc_firmware_copy(struct ivtv *itv)
@@ -139,7 +145,7 @@ static int ivtv_dec_firmware_copy(struct ivtv *itv)
 /* stops decoder firmware NICELY */
 static int ivtv_stop_firmware_dec(struct ivtv *itv)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	int x = 0, sleep = 0;
 
 	IVTV_DEBUG_INFO("Stopping decoder firmware\n");
@@ -164,7 +170,7 @@ static int ivtv_stop_firmware_dec(struct ivtv *itv)
 /* stops encoder firmware NICELY */
 static int ivtv_stop_firmware_enc(struct ivtv *itv)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	int x = 0, sleep = 0;
 
 	IVTV_DEBUG_INFO("Stopping decoder firmware\n");
@@ -249,7 +255,7 @@ void ivtv_halt_firmware(struct ivtv *itv, int mode)
 	}
 
 	IVTV_DEBUG_INFO("Sleeping for %dms (600 recommended)\n",
-		   (int)IVTV_SDRAM_SLEEPTIME);
+		   (int)(IVTV_SDRAM_SLEEPTIME * 1000 / HZ));
 	ivtv_sleep_timeout(IVTV_SDRAM_SLEEPTIME, 0);
 }
 
@@ -373,7 +379,7 @@ int ivtv_find_dec_firmware_mailbox(struct ivtv *itv)
 
 int ivtv_firmware_versions(struct ivtv *itv)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	int x;
 
 	/* Encoder */
@@ -471,7 +477,7 @@ int ivtv_firmware_init(struct ivtv *itv)
 	       (IVTV_REG_DEC_SDRAM_REFRESH + itv->reg_mem));
 
 	IVTV_DEBUG_INFO("Sleeping for %dms (600 recommended)\n",
-		   (int)IVTV_SDRAM_SLEEPTIME);
+		   (int)(IVTV_SDRAM_SLEEPTIME * 1000 / HZ));
 	ivtv_sleep_timeout(IVTV_SDRAM_SLEEPTIME, 0);
 
 	IVTV_DEBUG_INFO("Card ready for firmware!\n");
@@ -557,13 +563,13 @@ int ivtv_find_firmware_mailbox(struct ivtv *itv)
 void
 ivtv_init_mpeg_decoder(struct ivtv *itv)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	long readbytes;
 	unsigned char *mem_offset;
 	data[0] = 0;
-	data[1] = itv->width;	/* YUV source width */
-	data[2] = itv->height;
-	data[3] = itv->codec.audio_bitmask & 0xffff;	/* Audio settings to use,
+	data[1] = itv->params.width;	/* YUV source width */
+	data[2] = itv->params.height;
+	data[3] = itv->params.audio_properties;	/* Audio settings to use,
 							   bitmap. see docs. */
 	if ( ivtv_api(itv, itv->dec_mbox,
                       &itv->dec_msem,
@@ -581,19 +587,15 @@ ivtv_init_mpeg_decoder(struct ivtv *itv)
 		return;
 	}
 	ivtv_api_getresult_nosleep(itv, &itv->dec_mbox[9], &result, &data[0]);
-	mem_offset = itv->dec_mem+ data[1];
+	mem_offset = itv->dec_mem + data[1];
 
-	if ((readbytes = load_fw_direct(IVTV_DECODE_INIT_MPEG_FILENAME, (char *)mem_offset, itv,data[2])) <= 0)
+	if ((readbytes = load_fw_direct(IVTV_DECODE_INIT_MPEG_FILENAME, (char *)mem_offset, itv, IVTV_DECODE_INIT_MPEG_SIZE)) <= 0)
 	{
-		IVTV_INFO("failed to read mpeg decoder initialisation file %s\n",IVTV_DECODE_INIT_MPEG_FILENAME);
-	} else
-	{
-		ivtv_vapi(itv, CX2341X_DEC_SCHED_DMA_FROM_HOST, 3,
-                          0, readbytes, 0);
-		ivtv_sleep_timeout(HZ/10,0);
+		IVTV_INFO("failed to read mpeg decoder initialisation file %s\n", IVTV_DECODE_INIT_MPEG_FILENAME);
+	} else {
+		ivtv_vapi(itv, CX2341X_DEC_SCHED_DMA_FROM_HOST, 3, 0, readbytes, 0);
+		ivtv_sleep_timeout(HZ/10, 0);
 	}
-	ivtv_vapi(itv, CX2341X_DEC_STOP_PLAYBACK, 4,
-                  0,
-                  itv->dec_options.pts_low,
-                  itv->dec_options.pts_hi, 1);
+	ivtv_vapi(itv, CX2341X_DEC_STOP_PLAYBACK, 4, 0,
+                  itv->dec_options.pts_low, itv->dec_options.pts_hi, 1);
 }
