@@ -108,6 +108,25 @@ static const u8 hw_driverids[] = {
 	0 		/* IVTV_HW_GPIO dummy driver ID */
 };
 
+/* This array should match the IVTV_HW_ defines */
+static const char * const hw_drivernames[] = {
+	"cx2584x",
+	"saa7115",
+	"saa7127",
+	"msp3400",
+	"tuner",
+	"wm8775",
+	"cs53l32a",
+	"tveeprom",
+	"saa7114",
+	"tvaudio",
+	"upd64031a",
+	"upd64083",
+	"saa717x",
+	"wm8739",
+	"gpio",
+};
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 static void ivtv_i2c_inc(struct i2c_adapter *adapter)
 {
@@ -591,17 +610,14 @@ static int ivtv_call_i2c_client(struct ivtv *itv, int addr, unsigned int cmd,
 	return -ENODEV;
 }
 
-/* Find the i2c device matching the IVTV_HW_ flag and return
+/* Find the i2c device based on the driver ID and return
    its i2c address or -ENODEV if no matching device was found. */
-int ivtv_i2c_hw_addr(struct ivtv *itv, u32 hw)
+int ivtv_i2c_id_addr(struct ivtv *itv, u32 id)
 {
 	struct i2c_client *client;
-	int i = long_log2(hw);
-	int id = 0;
 	int retval = -ENODEV;
+	int i;
 
-	if (i < ARRAY_SIZE(hw_driverids))
-		id = hw_driverids[i];
 	down(&itv->i2c_lock);
 	for (i = 0; i < I2C_CLIENTS_MAX; i++) {
 		client = itv->i2c_clients[i];
@@ -616,6 +632,40 @@ int ivtv_i2c_hw_addr(struct ivtv *itv, u32 hw)
 	return retval;
 }
 
+/* Find the i2c device name matching the DRIVERID */
+static const char *ivtv_i2c_id_name(u32 id)
+{
+	int i;
+
+	for (i = 0; i < ARRAY_SIZE(hw_driverids); i++)
+		if (hw_driverids[i] == id)
+			return hw_drivernames[i];
+	return "unknown device";
+}
+
+/* Find the i2c device name matching the IVTV_HW_ flag */
+static const char *ivtv_i2c_hw_name(u32 hw)
+{
+	int i;
+
+	for (i = 0; i < ARRAY_SIZE(hw_driverids); i++)
+		if (1 << i == hw)
+			return hw_drivernames[i];
+	return "unknown device";
+}
+
+/* Find the i2c device matching the IVTV_HW_ flag and return
+   its i2c address or -ENODEV if no matching device was found. */
+int ivtv_i2c_hw_addr(struct ivtv *itv, u32 hw)
+{
+	int i;
+
+	for (i = 0; i < ARRAY_SIZE(hw_driverids); i++)
+		if (1 << i == hw)
+			return ivtv_i2c_id_addr(itv, hw_driverids[i]);
+	return -ENODEV;
+}
+
 /* Calls i2c device based on IVTV_HW_ flag. If hw == 0, then do nothing.
    If hw == IVTV_HW_GPIO then call the gpio handler. */
 int ivtv_i2c_hw(struct ivtv *itv, u32 hw, unsigned int cmd, void *arg)
@@ -629,7 +679,22 @@ int ivtv_i2c_hw(struct ivtv *itv, u32 hw, unsigned int cmd, void *arg)
 
 	addr = ivtv_i2c_hw_addr(itv, hw);
 	if (addr < 0) {
-		IVTV_ERR("i2c hardware 0x%08x not found for command 0x%x!\n", hw, cmd);
+		IVTV_ERR("i2c hardware 0x%08x (%s) not found for command 0x%x!\n",
+			       hw, ivtv_i2c_hw_name(hw), cmd);
+		return addr;
+	}
+	return ivtv_call_i2c_client(itv, addr, cmd, arg);
+}
+
+/* Calls i2c device based on I2C driver ID. */
+int ivtv_i2c_id(struct ivtv *itv, u32 id, unsigned int cmd, void *arg)
+{
+	int addr;
+
+	addr = ivtv_i2c_id_addr(itv, id);
+	if (addr < 0) {
+		IVTV_ERR("i2c ID 0x%08x (%s) not found for command 0x%x!\n",
+				id, ivtv_i2c_id_name(id), cmd);
 		return addr;
 	}
 	return ivtv_call_i2c_client(itv, addr, cmd, arg);
@@ -687,7 +752,7 @@ int __devinit init_ivtv_i2c(struct ivtv *itv)
 {
 	IVTV_DEBUG_I2C("i2c init\n");
 
-	if (itv->options.newi2c) {
+	if (itv->options.newi2c > 0) {
 		memcpy(&itv->i2c_adap, &ivtv_i2c_adap_hw_template,
 	 	       sizeof(struct i2c_adapter));
 	} else {
@@ -711,7 +776,7 @@ int __devinit init_ivtv_i2c(struct ivtv *itv)
 	ivtv_setscl(itv, 1);
 	ivtv_setsda(itv, 1);
 
-        if (itv->options.newi2c)
+        if (itv->options.newi2c > 0)
                 return i2c_add_adapter(&itv->i2c_adap);
         else
                 return i2c_bit_add_bus(&itv->i2c_adap);
@@ -721,7 +786,7 @@ void __devexit exit_ivtv_i2c(struct ivtv *itv)
 {
 	IVTV_DEBUG_I2C("i2c exit\n");
 
-        if (itv->options.newi2c) {
+        if (itv->options.newi2c > 0) {
                 i2c_del_adapter(&itv->i2c_adap);
         } else {
                 i2c_bit_del_bus(&itv->i2c_adap);
