@@ -25,8 +25,10 @@ int ivtv_msp34xx(struct ivtv *itv, unsigned int cmd, void *arg);
 int ivtv_upd64031a(struct ivtv *itv, unsigned int cmd, void *arg);
 int ivtv_upd64083(struct ivtv *itv, unsigned int cmd, void *arg);
 
+int ivtv_i2c_id_addr(struct ivtv *itv, u32 id);
 int ivtv_i2c_hw_addr(struct ivtv *itv, u32 hw);
 int ivtv_i2c_hw(struct ivtv *itv, u32 hw, unsigned int cmd, void *arg);
+int ivtv_i2c_id(struct ivtv *itv, u32 id, unsigned int cmd, void *arg);
 void ivtv_call_i2c_clients(struct ivtv *itv, unsigned int cmd, void *arg);
 
 /* init + register i2c algo-bit adapter */
