@@ -19,7 +19,6 @@
 
 #include "ivtv-driver.h"
 #include "ivtv-version.h"
-#include "ivtv-reset.h"
 #include "ivtv-mailbox.h"
 #include "ivtv-i2c.h"
 #include "ivtv-queue.h"
@@ -38,6 +37,12 @@
 #include <media/tveeprom.h>
 #include <linux/i2c-id.h>
 
+/* Differs from the VIDIOC_INT_S_REGISTER from v4l2-common.h in the
+   2.6.18 kernel due to a bug in that header. ivtvctl issues the
+   corrected version below, but the VIDIOC_INT_S_REGISTER ioctl is
+   what must be passed to the i2c modules */
+#define IVTV_INT_S_REGISTER           _IOW ('d', 100, struct v4l2_register)
+
 u16 service2vbi(int type)
 {
         switch (type) {
@@ -88,7 +93,7 @@ static u16 select_service_from_set(int field, int line, u16 set, int is_pal)
         return 0;
 }
 
-static void expand_service_set(struct v4l2_sliced_vbi_format *fmt, int is_pal)
+void expand_service_set(struct v4l2_sliced_vbi_format *fmt, int is_pal)
 {
         u16 set = fmt->service_set;
         int f, l;
@@ -174,12 +179,65 @@ static void ivtv_stream_off(struct ivtv *itv, struct ivtv_stream *stream)
            we've finished here. */
         set_bit(IVTV_F_S_STREAMOFF, &stream->s_flags);
         ivtv_stop_capture(itv, stream->type);
-        return;
+}
+
+static int ivtv_stream_on(struct ivtv *itv, struct ivtv_stream *stream)
+{
+       	struct ivtv_stream *vbi_stream;
+
+        IVTV_DEBUG_INFO("Entering STREAMON\n");
+
+	/* If capture is already in progress, then we have to
+	   do nothing. */
+	if (test_and_set_bit(IVTV_F_S_CAPTURING, &stream->s_flags))
+		return 0;
+
+	/* Start VBI capture if required */
+       	vbi_stream = &itv->streams[IVTV_ENC_STREAM_TYPE_VBI];
+        if (stream->type == IVTV_ENC_STREAM_TYPE_MPG &&
+            test_bit(IVTV_F_S_INTERNAL_USE, &vbi_stream->s_flags) &&
+            !test_and_set_bit(IVTV_F_S_CAPTURING, &vbi_stream->s_flags)) {
+                /* Note: the IVTV_ENC_STREAM_TYPE_VBI is claimed
+                   automatically when the MPG stream is claimed. 
+                   We only need to start the VBI capturing. */
+        	if (ivtv_start_v4l2_encode_stream(itv, IVTV_ENC_STREAM_TYPE_VBI)) {
+               		IVTV_DEBUG_WARN("VBI capture start failed\n");
+
+                	/* Failure, clean up and return an error */
+        		clear_bit(IVTV_F_S_CAPTURING, &vbi_stream->s_flags);
+			clear_bit(IVTV_F_S_CAPTURING, &stream->s_flags);
+                        /* also releases the associated VBI stream */
+			ivtv_release_stream(itv, stream->type);
+                	return -EIO;
+                }
+               	IVTV_DEBUG_INFO("VBI insertion started\n");
+	}
+
+	/* Tell the card to start capturing */
+	if (!ivtv_start_v4l2_encode_stream(itv, stream->type)) {
+        	/* We're done */
+	        return 0;
+        }
+
+        /* failure, clean up */
+        IVTV_DEBUG_WARN("Failed to start capturing for stream %d\n", stream->type);
+
+        /* Note: the IVTV_ENC_STREAM_TYPE_VBI is released
+           automatically when the MPG stream is released. 
+           We only need to stop the VBI capturing. */
+        if (stream->type == IVTV_ENC_STREAM_TYPE_MPG &&
+            test_bit(IVTV_F_S_CAPTURING, &vbi_stream->s_flags)) {
+                ivtv_stop_capture(itv, IVTV_ENC_STREAM_TYPE_VBI);
+                clear_bit(IVTV_F_S_CAPTURING, &vbi_stream->s_flags);
+        }
+        clear_bit(IVTV_F_S_CAPTURING, &stream->s_flags);
+        ivtv_release_stream(itv, stream->type);
+        return -EIO;
 }
 
 static int ivtv_s_speed(struct ivtv *itv, struct ivtv_speed *speed)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	int type;
 	struct ivtv_stream *st;
 
@@ -231,10 +289,10 @@ static int ivtv_s_speed(struct ivtv *itv, struct ivtv_speed *speed)
 			   "DEC: Warning, Reverse not supported\n");
 
 		/* B Frames when in reverse */
-		if (speed->b_per_gop > itv->codec.bframes)
-			speed->b_per_gop = itv->codec.bframes;
+		if (speed->b_per_gop > itv->params.video_b_frames)
+			speed->b_per_gop = itv->params.video_b_frames;
 		else if (speed->b_per_gop < 0)
-			speed->b_per_gop = itv->codec.bframes;
+			speed->b_per_gop = itv->params.video_b_frames;
 	}
 
 	data[1] = speed->direction;	/* Forward. Reverse not supported */
@@ -316,26 +374,6 @@ static int ivtv_s_speed(struct ivtv *itv, struct ivtv_speed *speed)
 	return 0;
 }
 
-static int ivtv_api_show_boxes(struct ivtv *itv, struct ivtv_mailbox *mbox,
-	       			int args, int start, int stop)
-{
-	int x, i;
-	struct ivtv_mailbox *local_box;
-
-	for (x = start; x <= stop; x++) {
-		local_box = &mbox[x];
-		IVTV_INFO("Mailbox[%d] flags 0x%08x cmd 0x%08x timeout 0x%08x\n",
-		       x, readl(&local_box->flags),
-		       readl(&local_box->cmd), readl(&local_box->timeout));
-
-		for (i = 0; i < args; i++) {
-			IVTV_INFO(" data[%d] = 0x%08x\n",
-			       i, readl(&local_box->data[i]));
-		}
-	}
-	return 0;
-}
-
 static int ivtv_itvc(struct ivtv *itv, unsigned int cmd, void *arg)
 {
 	struct v4l2_register *regs = arg;
@@ -352,7 +390,7 @@ static int ivtv_itvc(struct ivtv *itv, unsigned int cmd, void *arg)
                 return -EINVAL;
 
 	spin_lock_irqsave(&ivtv_cards_lock, flags);
-	if (cmd == IVTV_IOC_G_ITVC_REG) {
+	if (cmd == VIDIOC_INT_G_REGISTER) {
 		regs->val = readl(regs->reg + reg_start);
 	} else {
 		writel(regs->val, regs->reg + reg_start);
@@ -398,15 +436,18 @@ static int ivtv_get_fmt(struct ivtv *itv, int streamtype, struct v4l2_format *fm
         /* case 0:	 Works around a bug in MythTV (already fixed).
            This case should be removed once post 0.1.10
            versions of ivtv have become mainstream. */
-        case V4L2_BUF_TYPE_VIDEO_CAPTURE:
         case V4L2_BUF_TYPE_VIDEO_OUTPUT:
-                fmt->fmt.pix.width = itv->width;
-                fmt->fmt.pix.height = itv->height;
+		if (!(itv->v4l2_cap & V4L2_CAP_VIDEO_OUTPUT))
+			return -EINVAL;
+		/* fall through */
+        case V4L2_BUF_TYPE_VIDEO_CAPTURE:
+                fmt->fmt.pix.width = itv->params.width;
+                fmt->fmt.pix.height = itv->params.height;
                 fmt->fmt.pix.colorspace = V4L2_COLORSPACE_SMPTE170M;
                 fmt->fmt.pix.field = V4L2_FIELD_INTERLACED;
                 if (streamtype == IVTV_ENC_STREAM_TYPE_YUV ||
                                 streamtype == IVTV_DEC_STREAM_TYPE_YUV) {
-                        fmt->fmt.pix.pixelformat = V4L2_PIX_FMT_UYVY;
+                        fmt->fmt.pix.pixelformat = V4L2_PIX_FMT_HM12;
                         /* YUV size is (Y=(h*w) + UV=(h*(w/2))) */
                         fmt->fmt.pix.sizeimage =
                                 fmt->fmt.pix.height * fmt->fmt.pix.width +
@@ -438,6 +479,8 @@ static int ivtv_get_fmt(struct ivtv *itv, int streamtype, struct v4l2_format *fm
         {
                 struct v4l2_sliced_vbi_format *vbifmt = &fmt->fmt.sliced;
 
+		if (!(itv->v4l2_cap & V4L2_CAP_SLICED_VBI_OUTPUT))
+			return -EINVAL;
                 vbifmt->io_size = sizeof(struct v4l2_sliced_vbi_data) * 36;
                 memset(vbifmt->reserved, 0, sizeof(vbifmt->reserved));
                 memset(vbifmt->service_lines, 0, sizeof(vbifmt->service_lines));
@@ -495,25 +538,17 @@ static int ivtv_try_or_set_fmt(struct ivtv *itv, int streamtype,
                 if (!set_fmt)
                         return ivtv_get_fmt(itv, streamtype, fmt);
 
-                if ((itv->width != fmt->fmt.pix.width ||
-                     itv->height != fmt->fmt.pix.height) &&
+                if ((itv->params.width != fmt->fmt.pix.width ||
+                     itv->params.height != fmt->fmt.pix.height) &&
                     atomic_read(&itv->capturing) > 0) {
                         return -EBUSY;
                 }
 
                 /* FIXME: only sets resolution for now */
 		pix.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-                pix.fmt.pix.width = itv->width = fmt->fmt.pix.width;
-                pix.fmt.pix.height = itv->height = fmt->fmt.pix.height;
+                pix.fmt.pix.width = itv->params.width = fmt->fmt.pix.width;
+                pix.fmt.pix.height = itv->params.height = fmt->fmt.pix.height;
                 itv->video_dec_func(itv, VIDIOC_S_FMT, &pix);
-
-                if (itv->codec.stream_type == IVTV_STREAM_MPEG1 ||
-                    itv->codec.stream_type == IVTV_STREAM_VCD) {
-                        /* this is an MPEG1 stream */
-                        IVTV_DEBUG_INFO("VIDIOC_S_FMT: "
-                            "the current codec stream type is MPEG1 or VCD, "
-                            "you have to do a S_CODEC after this ioctl\n");
-                }
                 return ivtv_get_fmt(itv, streamtype, fmt);
         }
        
@@ -572,142 +607,30 @@ static int ivtv_internal_ioctls(struct ivtv *itv, int streamtype, unsigned int c
 	struct v4l2_register *reg = arg;
 
 	switch (cmd) {
-	case IVTV_IOC_FWAPI:{
-		struct ivtv_ioctl_fwapi *fwapi = arg;
-
-		IVTV_DEBUG_IOCTL("IVTV_IOC_FWAPI\n");
-
-		/* Encoder */
-		if (fwapi->cmd >= 128)
-			return ivtv_api(itv, itv->enc_mbox, &itv->enc_msem,
-					fwapi->cmd, &fwapi->result, fwapi->args,
-					fwapi->data);
-
-		/* Decoder */
-		return ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, fwapi->cmd,
-				&fwapi->result, fwapi->args, fwapi->data);
-	}
-
-	case IVTV_IOC_EVENT_SETUP:{
-		struct ivtv_ioctl_event *event = arg;
-		int x;
-		int mbox;
-
-		IVTV_DEBUG_IOCTL("IVTV_IOC_EVENT_SETUP\n");
-
-		mbox = event->mbox;
-
-		if (213 == event->api.cmd && 1 == event->api.data[1])
-			ivtv_clear_irq_mask(itv, event->api.data[2]);
-		else if (213 == event->api.cmd)
-			ivtv_set_irq_mask(itv, event->api.data[2]);
-
-		if (event->type) {
-			IVTV_DEBUG_WARN("sending event to encoder\n");
-			/* Encoder */
-			x = ivtv_api(itv, &itv->enc_mbox[mbox], &itv->enc_msem,
-				     event->api.cmd, &event->api.result,
-				     event->api.args, event->api.data);
-		} else {
-			IVTV_DEBUG_WARN("sending event to decoder\n");
-			/* Decoder */
-			x = ivtv_api(itv, &itv->dec_mbox[mbox], &itv->dec_msem,
-				     event->api.cmd, &event->api.result,
-				     event->api.args, event->api.data);
-		}
-
-		ivtv_api_show_boxes(itv, &itv->dec_mbox[mbox], 4, mbox, mbox);
-		return x;
-	}
-
-	case IVTV_IOC_RELOAD_FW:{
-		int ret = 0;
-
-		IVTV_DEBUG_IOCTL("IVTV_IOC_RELOAD_FW\n");
-
-		/* Force Full Reload */
-		ret = ivtv_reset_card(itv, 1, IVTV_CMD_FULL_RESET);
-
-		return ret;
-	}
-
-	/* ioctls to allow direct access to the ITVC registers for testing */
-	case IVTV_IOC_G_ITVC_REG:
-		IVTV_DEBUG_IOCTL("IVTV_IOC_G_ITVC_REG\n");
-		return ivtv_itvc(itv, cmd, arg);
-
-	case IVTV_IOC_S_ITVC_REG:
-		IVTV_DEBUG_IOCTL("IVTV_IOC_S_ITVC_REG\n");
-		return ivtv_itvc(itv, cmd, arg);
-
-	/* ioctls to allow direct access to the decoder registers for testing */
-	case IVTV_IOC_G_DECODER_REG:
-		IVTV_DEBUG_IOCTL("IVTV_IOC_G_DECODER_REG\n");
-		if (itv->hw_flags & IVTV_HW_CX25840)
-			reg->i2c_id = I2C_DRIVERID_CX25840;
-		else if (itv->hw_flags & IVTV_HW_SAA711X)
-			reg->i2c_id = I2C_DRIVERID_SAA711X;
-		else
-			reg->i2c_id = I2C_DRIVERID_SAA717X;
-		return itv->video_dec_func(itv, VIDIOC_INT_G_REGISTER, reg);
-
-	case IVTV_IOC_S_DECODER_REG:
-		IVTV_DEBUG_IOCTL("IVTV_IOC_S_DECODER_REG\n");
-		if (itv->hw_flags & IVTV_HW_CX25840)
-			reg->i2c_id = I2C_DRIVERID_CX25840;
-		else if (itv->hw_flags & IVTV_HW_SAA711X)
-			reg->i2c_id = I2C_DRIVERID_SAA711X;
-		else
-			reg->i2c_id = I2C_DRIVERID_SAA717X;
-		return itv->video_dec_func(itv, VIDIOC_INT_S_REGISTER, reg);
-
 	/* ioctls to allow direct access to the encoder registers for testing */
-	case IVTV_IOC_G_ENCODER_REG:
-		IVTV_DEBUG_IOCTL("IVTV_IOC_G_ENCODER_REG\n");
-		reg->i2c_id = I2C_DRIVERID_SAA7127;
-		return ivtv_saa7127(itv, VIDIOC_INT_G_REGISTER, reg);
+	case VIDIOC_INT_G_REGISTER:
+		IVTV_DEBUG_IOCTL("VIDIOC_INT_G_REGISTER\n");
+		if (reg->i2c_id)
+			return ivtv_i2c_id(itv, reg->i2c_id, cmd, reg);
+		return ivtv_itvc(itv, cmd, arg);
 
-	case IVTV_IOC_S_ENCODER_REG:
-		IVTV_DEBUG_IOCTL("IVTV_IOC_S_ENCODER_REG\n");
-		reg->i2c_id = I2C_DRIVERID_SAA7127;
-		return ivtv_saa7127(itv, VIDIOC_INT_S_REGISTER, reg);
+	case IVTV_INT_S_REGISTER:
+		IVTV_DEBUG_IOCTL("VIDIOC_INT_S_REGISTER\n");
+		cmd = VIDIOC_INT_S_REGISTER;	/* see comment at the top */
+		if (reg->i2c_id)
+			return ivtv_i2c_id(itv, reg->i2c_id, cmd, reg);
+		return ivtv_itvc(itv, cmd, arg);
 
-	case IVTV_IOC_ZCOUNT:{
-		/* Zeroes out usage count so it can be unloaded in case of
-		 * drastic error */
-
-		IVTV_DEBUG_IOCTL("ZCOUNT\n");
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
-		while (MOD_IN_USE)
-			MOD_DEC_USE_COUNT;
-
-		MOD_INC_USE_COUNT;
-#endif
-		break;
-	}
-
-	case IVTV_IOC_S_AUDIO_ROUTING: {
+	case VIDIOC_INT_S_AUDIO_ROUTING: {
 		struct v4l2_routing *route = arg;
 
-		IVTV_DEBUG_IOCTL("IVTV_IOC_S_AUDIO_ROUTING\n");
+		IVTV_DEBUG_IOCTL("VIDIOC_INT_S_AUDIO_ROUTING\n");
 		ivtv_audio_set_route(itv, route);
 		break;
 	}
 
-	case IVTV_IOC_G_DEBUG_LEVEL:
-		IVTV_DEBUG_IOCTL("IVTV_IOC_G_DEBUG_LEVEL\n");
-		*(int *)arg = ivtv_debug;
-		break;
-
-	case IVTV_IOC_S_DEBUG_LEVEL:
-		IVTV_DEBUG_IOCTL("IVTV_IOC_S_DEBUG_LEVEL\n");
-		ivtv_debug = (*(int *)arg) | IVTV_DBGFLG_WARN;
-		*(int *)arg = ivtv_debug;
-		break;
-
-	case IVTV_IOC_RESET_IR:
-		IVTV_DEBUG_IOCTL("IVTV_IOC_RESET_IR\n");
+	case VIDIOC_INT_RESET:
+		IVTV_DEBUG_IOCTL("VIDIOC_INT_RESET\n");
 		ivtv_reset_ir_gpio(itv);
 		break;	
         
@@ -834,6 +757,37 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
                 return ivtv_get_fmt(itv, streamtype, fmt);
         }
 
+	case VIDIOC_ENUM_FMT: {
+		static struct v4l2_fmtdesc formats[] = {
+			{ 0, 0, 0,
+			  "HM12 (YUV 4:1:1)", V4L2_PIX_FMT_HM12,
+			  { 0, 0, 0, 0 }
+			},
+			{ 1, 0, V4L2_FMT_FLAG_COMPRESSED,
+			  "MPEG", V4L2_PIX_FMT_MPEG,
+			  { 0, 0, 0, 0 }
+			}
+		};
+		struct v4l2_fmtdesc *fmt = arg;
+		enum v4l2_buf_type type = fmt->type;
+
+		switch (type) {
+		case V4L2_BUF_TYPE_VIDEO_CAPTURE:
+			break;
+		case V4L2_BUF_TYPE_VIDEO_OUTPUT:
+			if (!(itv->v4l2_cap & V4L2_CAP_VIDEO_OUTPUT))
+				return -EINVAL;
+			break;
+		default:
+			return -EINVAL;
+		}
+		if (fmt->index > 1)
+			return -EINVAL;
+		*fmt = formats[fmt->index];
+		fmt->type = type;
+		return 0;
+	}
+
 	case VIDIOC_G_INPUT:{
 		IVTV_DEBUG_IOCTL("VIDIOC_G_INPUT\n");
 
@@ -976,18 +930,20 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		}
 
 		itv->std = std;
-		itv->width = 720;
+		itv->params.width = 720;
 		if (itv->std & V4L2_STD_525_60) {
                         itv->is_50hz = 0;
                         itv->is_60hz = 1;
-			itv->height = 480;
+			itv->params.height = 480;
+			itv->params.is_50hz = 0;
 			itv->vbi_count = 12;
 			itv->vbi_start[0] = 10;
 			itv->vbi_start[1] = 273;
 		} else {	/* PAL/SECAM */
                         itv->is_50hz = 1;
                         itv->is_60hz = 0;
-			itv->height = 576;
+			itv->params.height = 576;
+			itv->params.is_50hz = 1;
 			itv->vbi_count = 18;
 			itv->vbi_start[0] = 6;
 			itv->vbi_start[1] = 318;
@@ -1051,6 +1007,14 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		break;
 	}
 
+	case VIDIOC_STREAMON:{
+		IVTV_DEBUG_IOCTL("VIDIOC_STREAMON\n");
+		if (id->open_id != stream->id)
+			return -EBUSY;
+
+		return ivtv_stream_on(itv, stream);
+	}
+
         case VIDIOC_G_SLICED_VBI_CAP: {
 		struct v4l2_sliced_vbi_cap *cap = arg;
                 int set = itv->is_50hz ? V4L2_SLICED_VBI_625 : V4L2_SLICED_VBI_525;
@@ -1068,6 +1032,8 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
                                 }
                         }
 		} else if (streamtype == IVTV_DEC_STREAM_TYPE_VOUT) {
+			if (!(itv->v4l2_cap & V4L2_CAP_SLICED_VBI_OUTPUT))
+				return -EINVAL;
                         if (itv->is_60hz) {
                                 cap->service_lines[0][21] = V4L2_SLICED_CAPTION_525;
                                 cap->service_lines[1][21] = V4L2_SLICED_CAPTION_525;
@@ -1101,6 +1067,7 @@ int ivtv_v4l2_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 			ivtv_read_eeprom(itv, &tv);
                 }
 		ivtv_call_i2c_clients(itv, VIDIOC_LOG_STATUS, 0);
+		cx2341x_log_status(&itv->params, itv->name);
                 IVTV_INFO("==================  END STATUS CARD #%d  ==================\n", itv->num);
                 break;
 
@@ -1148,7 +1115,7 @@ static int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		*fs = itv->dec_timestamp;
 
 		if (atomic_read(&itv->decoding) == 0) {
-			u32 data[IVTV_MBOX_MAX_DATA], result;
+			u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 			memset(fs, 0, sizeof(fs));
 			/* scr should be valid anytime */
@@ -1182,16 +1149,21 @@ static int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 
 	}
 
-	case IVTV_IOC_START_DECODE:
-	case IVTV_IOC_S_START_DECODE:{
+	case IVTV_IOC_STOP:{
+		IVTV_DEBUG_IOCTL("IVTV_IOC_STOP\n");
+		if (ivtv_vapi(itv, CX2341X_DEC_STOP_PLAYBACK, 3, 1, 0, 0))
+			IVTV_DEBUG_WARN(
+				   "DEC: error stopping playback\n");
+		break;
+
+	}
+
+	case IVTV_IOC_START_DECODE: {
 		struct ivtv_cfg_start_decode *sd = arg;
 		struct ivtv_stream *stream =
 		    &itv->streams[IVTV_DEC_STREAM_TYPE_MPG];
 
-                if (cmd == IVTV_IOC_STOP_DECODE)
-                        IVTV_DEBUG_IOCTL("IVTV_IOC_START_DECODE\n");
-                else
-                        IVTV_DEBUG_IOCTL("IVTV_IOC_S_START_DECODE\n");
+                IVTV_DEBUG_IOCTL("IVTV_IOC_START_DECODE\n");
 		if (!(itv->v4l2_cap & V4L2_CAP_VIDEO_OUTPUT))
 			return -EINVAL;
 		if ((sd->gop_offset < 0) || (sd->gop_offset > 15))
@@ -1208,9 +1180,6 @@ static int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		/*itv->dec_options.fast_stop = sd->fast_stop;
 		itv->dec_options.no_stop = sd->no_stop;*/
 
-		if (cmd == IVTV_IOC_S_START_DECODE)
-			break;
-
 		if (test_bit(IVTV_F_I_PASSTHROUGH, &itv->i_flags))
 			return -EBUSY;
 
@@ -1236,16 +1205,12 @@ static int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		return 0;
 	}
 
-	case IVTV_IOC_STOP_DECODE:
-	case IVTV_IOC_S_STOP_DECODE:{
+	case IVTV_IOC_STOP_DECODE: {
 		struct ivtv_cfg_stop_decode *sd = arg;
 		struct ivtv_stream *stream =
 		    &itv->streams[IVTV_DEC_STREAM_TYPE_MPG];
 
-                if (cmd == IVTV_IOC_STOP_DECODE)
-                        IVTV_DEBUG_IOCTL("IVTV_IOC_STOP_DECODE\n");
-                else
-                        IVTV_DEBUG_IOCTL("IVTV_IOC_S_STOP_DECODE\n");
+                IVTV_DEBUG_IOCTL("IVTV_IOC_STOP_DECODE\n");
 		if (!(itv->v4l2_cap & V4L2_CAP_VIDEO_OUTPUT))
 			return -EINVAL;
 		if ((sd->hide_last < 0) || (sd->hide_last > 1))
@@ -1255,9 +1220,6 @@ static int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		itv->dec_options.pts_low = (u32) (sd->pts_stop & 0xFFFFFFFF);
 		itv->dec_options.pts_hi = (u32) (sd->pts_stop >> 32);
 
-		if (cmd == IVTV_IOC_S_STOP_DECODE)
-			break;
-
 		if (!test_bit(IVTV_F_S_DECODING, &stream->s_flags))
 			break;
 
@@ -1385,9 +1347,37 @@ static int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		break;
 	}
 
+	case IVTV_IOC_PAUSE_BLACK:{
+        	struct ivtv_stream *st;
+		int type;
+
+		IVTV_DEBUG_IOCTL("IVTV_IOC_PAUSE\n");
+        	/* YUV or MPG decoding stream */
+        	if (test_bit(IVTV_F_I_DEC_YUV, &itv->i_flags))
+                	type = IVTV_DEC_STREAM_TYPE_YUV;
+        	else
+                	type = IVTV_DEC_STREAM_TYPE_MPG;
+        	st = &itv->streams[type];
+
+		/* We really don't want to get stuck inbetween DMA calls */
+		if ((atomic_read(&itv->decoding) > 0) &&
+			test_bit(IVTV_F_T_DEC_DONE, &itv->t_flags)) 
+		{
+			if (ivtv_vapi(itv, CX2341X_DEC_PAUSE_PLAYBACK, 1, 1)) {
+				IVTV_DEBUG_WARN(
+					"DEC: error pausing\n");
+			}
+		} else
+			IVTV_DEBUG_INFO(
+					"DEC: Pause Error, "
+					"Decoder isn't running\n");
+
+		break;
+	}
+
 	case IVTV_IOC_GET_TIMING:{
 		struct ivtv_ioctl_framesync *timing = arg;
-		u32 data[IVTV_MBOX_MAX_DATA], result;
+		u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 		IVTV_DEBUG_IOCTL("IVTV_IOC_GET_TIMING\n");
 		if (atomic_read(&itv->decoding) == 0) {
@@ -1423,57 +1413,6 @@ static int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		break;
 	}
 
-	case IVTV_IOC_S_VBI_EMBED:{
-                int embed = *(int *)arg;
-
-		IVTV_DEBUG_IOCTL("IVTV_IOC_S_VBI_EMBED\n");
-		if (!(itv->v4l2_cap & V4L2_CAP_SLICED_VBI_CAPTURE))
-			return -EINVAL;
-                if (embed != 0 && embed != 1)
-                        return -EINVAL;
-		if (atomic_read(&itv->capturing) > 0)
-			return -EBUSY;
-
-                // First try to allocate sliced VBI buffers if needed.
-                if (embed && itv->vbi_sliced_mpeg_data[0] == NULL) {
-                        int i;
-
-                        for (i = 0; i < IVTV_VBI_FRAMES; i++) {
-                                // Yuck, hardcoded. Needs to be a define
-                                itv->vbi_sliced_mpeg_data[i] =
-                                        kmalloc(2049, GFP_KERNEL);
-                                if (itv->vbi_sliced_mpeg_data[i] == NULL) {
-                                        while (--i >= 0) {
-                                                kfree(itv->vbi_sliced_mpeg_data[i]);
-                                                itv->vbi_sliced_mpeg_data[i] = NULL;
-                                        }
-                                        return -ENOMEM;
-                                }
-                        }
-                }
-
-		itv->vbi_insert_mpeg = embed;
-
-		if (itv->vbi_insert_mpeg == 0) {
-                        break;
-                }
-		/* Need sliced data for mpeg insertion */
-                if (get_service_set(itv->vbi_sliced_in) == 0) {
-                        if (itv->is_60hz)
-        			itv->vbi_sliced_in->service_set = V4L2_SLICED_CAPTION_525;
-                        else
-        			itv->vbi_sliced_in->service_set = V4L2_SLICED_WSS_625;
-        		expand_service_set(itv->vbi_sliced_in, itv->is_50hz);
-                }
-		break;
-	}
-
-	case IVTV_IOC_G_VBI_EMBED:{
-		IVTV_DEBUG_IOCTL("IVTV_IOC_G_VBI_EMBED\n");
-		*(int *)arg = itv->vbi_insert_mpeg;
-		break;
-	}
-
 	case IVTV_IOC_S_VBI_PASSTHROUGH:{
 		IVTV_DEBUG_IOCTL("IVTV_IOC_S_VBI_PASSTHROUGH\n");
 		if (!(itv->v4l2_cap & V4L2_CAP_SLICED_VBI_OUTPUT))
@@ -1488,68 +1427,6 @@ static int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		break;
 	}
 
-	case IVTV_IOC_G_CODEC:{
-		struct ivtv_ioctl_codec *codec = arg;
-
-		IVTV_DEBUG_IOCTL("IVTV_IOC_G_CODEC\n");
-		*codec = itv->codec;
-		codec->framerate = itv->codec.framerate;
-		codec->framespergop = itv->codec.framespergop;
-		break;
-	}
-
-	case IVTV_IOC_S_CODEC:{
-		struct ivtv_ioctl_codec *codec = arg;
-		struct v4l2_format fmt;
-
-		IVTV_DEBUG_IOCTL("IVTV_IOC_S_CODEC\n");
-		/* FIXME: insert abundant parameter validation here */
-		if ((codec->bitrate == 0) || (codec->bitrate_peak == 0) ||
-		    (codec->bitrate > codec->bitrate_peak)) {
-			IVTV_DEBUG_WARN("ivtv ioctl: set "
-				   "bitrate=%u < peak=%u: failed\n",
-				   codec->bitrate, codec->bitrate_peak);
-			return -EINVAL;
-		}
-
-                /* check if it is an MPEG1 stream */
-                if ((codec->stream_type == IVTV_STREAM_MPEG1) ||
-                    (codec->stream_type == IVTV_STREAM_VCD)) {
-                        /* this is an MPEG1 stream */
-                        int tmp_height = itv->is_60hz ? 480 : 576;
-
-                        /* set vertical mpeg encoder resolution */
-                        if (itv->height != tmp_height / 2) {
-                                itv->height = tmp_height / 2;
-                        }
-                        /* mpeg1 is cbr */
-                        codec->bitrate_mode = 1;
-
-                        IVTV_DEBUG_INFO(
-                                   "ivtv ioctl: set codec: "
-                                   "stream_type is MPEG1 or VCD. resolution %dx%d.\n",
-                                   itv->width, itv->height);
-
-                        /* fix videodecoder resolution */
-			fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-                        fmt.fmt.pix.width = itv->width;
-                        fmt.fmt.pix.height = tmp_height;
-                        if (atomic_read(&itv->capturing) > 0) {
-                                IVTV_DEBUG_WARN(
-                                        "v4l2 ioctl: set size failed, "
-                                        "capture in progress\n");
-                        } else {
-                                itv->video_dec_func(itv, VIDIOC_S_FMT, &fmt);
-                        }
-                }
-
-                /* Passed the garbage check */
-                itv->codec = *codec;
-
-		ivtv_audio_set_audio_clock_freq(itv, codec->audio_bitmask & 0x03);
-		break;
-	}
-
 	case IVTV_IOC_S_GOP_END:{
 		IVTV_DEBUG_IOCTL("IVTV_IOC_S_GOP_END\n");
 		itv->end_gop = *(int *)arg;
@@ -1562,7 +1439,7 @@ static int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		if (!atomic_read(&itv->capturing))
 			return 0;
 		ivtv_mute(itv);
-		ivtv_vapi(itv, CX2341X_ENC_PAUSE_ENCODER, 1);
+		ivtv_vapi(itv, CX2341X_ENC_PAUSE_ENCODER, 0);
 		break;
 	}
 
@@ -1570,7 +1447,7 @@ static int ivtv_ivtv_ioctls(struct ivtv *itv, struct ivtv_open_id *id,
 		IVTV_DEBUG_IOCTL("IVTV_IOC_RESUME_ENCODE\n");
 		if (!atomic_read(&itv->capturing))
 			return 0;
-		ivtv_vapi(itv, CX2341X_ENC_PAUSE_ENCODER, 0);
+		ivtv_vapi(itv, CX2341X_ENC_PAUSE_ENCODER, 1);
 		ivtv_unmute(itv);
 		break;
 	}
@@ -1644,20 +1521,10 @@ static int ivtv_v4l2_do_ioctl(struct inode *inode, struct file *filp,
 	IVTV_DEBUG_IOCTL("v4l2 ioctl 0x%08x\n", cmd);
 
 	switch (cmd) {
-	case IVTV_IOC_EVENT_SETUP:
-	case IVTV_IOC_RELOAD_FW:
-	case IVTV_IOC_G_ITVC_REG:
-	case IVTV_IOC_S_ITVC_REG:
-	case IVTV_IOC_G_ENCODER_REG:
-	case IVTV_IOC_S_ENCODER_REG:
-	case IVTV_IOC_G_DECODER_REG:
-	case IVTV_IOC_S_DECODER_REG:
-	case IVTV_IOC_ZCOUNT:
-	case IVTV_IOC_S_AUDIO_ROUTING:
-	case IVTV_IOC_G_DEBUG_LEVEL:
-	case IVTV_IOC_S_DEBUG_LEVEL:
-	case IVTV_IOC_FWAPI:
-	case IVTV_IOC_RESET_IR:
+	case VIDIOC_INT_G_REGISTER:
+	case IVTV_INT_S_REGISTER:
+	case VIDIOC_INT_S_AUDIO_ROUTING:
+	case VIDIOC_INT_RESET:
 		return ivtv_internal_ioctls(itv, streamtype, cmd, arg);
 
 	case VIDIOC_QUERYCAP:
@@ -1670,6 +1537,7 @@ static int ivtv_v4l2_do_ioctl(struct inode *inode, struct file *filp,
 	case VIDIOC_G_FMT:
 	case VIDIOC_S_FMT:
 	case VIDIOC_TRY_FMT:
+	case VIDIOC_ENUM_FMT:
 	case VIDIOC_G_FREQUENCY:
 	case VIDIOC_S_FREQUENCY:
 	case VIDIOC_ENUMSTD:
@@ -1686,22 +1554,26 @@ static int ivtv_v4l2_do_ioctl(struct inode *inode, struct file *filp,
         case VIDIOC_G_SLICED_VBI_CAP:
         case VIDIOC_G_FBUF:
         case VIDIOC_LOG_STATUS:
+        case VIDIOC_STREAMOFF:
+        case VIDIOC_STREAMON:
 		return ivtv_v4l2_ioctls(itv, id, streamtype, cmd, arg);
 
 	case VIDIOC_QUERYMENU:
 	case VIDIOC_QUERYCTRL:
 	case VIDIOC_S_CTRL:
 	case VIDIOC_G_CTRL:
+	case VIDIOC_S_EXT_CTRLS:
+	case VIDIOC_G_EXT_CTRLS:
+	case VIDIOC_TRY_EXT_CTRLS:
 		return ivtv_control_ioctls(itv, cmd, arg);
 
 	case IVTV_IOC_PASSTHROUGH:
 	case IVTV_IOC_GET_FB:
 	case IVTV_IOC_FRAMESYNC:
 	case IVTV_IOC_PLAY:
+	case IVTV_IOC_STOP:
 	case IVTV_IOC_START_DECODE:
-	case IVTV_IOC_S_START_DECODE:
 	case IVTV_IOC_STOP_DECODE:
-	case IVTV_IOC_S_STOP_DECODE:
 	case IVTV_IOC_DEC_FLUSH:
 	case IVTV_IOC_DEC_STEP:
 	case IVTV_IOC_DEC_SPLICE:
@@ -1710,13 +1582,10 @@ static int ivtv_v4l2_do_ioctl(struct inode *inode, struct file *filp,
 	case IVTV_IOC_S_SPEED:
 	case IVTV_IOC_S_SLOW_FAST:
 	case IVTV_IOC_PAUSE:
+	case IVTV_IOC_PAUSE_BLACK:
 	case IVTV_IOC_GET_TIMING:
-	case IVTV_IOC_S_VBI_EMBED:
-	case IVTV_IOC_G_VBI_EMBED:
 	case IVTV_IOC_S_VBI_PASSTHROUGH:
 	case IVTV_IOC_G_VBI_PASSTHROUGH:
-	case IVTV_IOC_G_CODEC:
-	case IVTV_IOC_S_CODEC:
 	case IVTV_IOC_S_GOP_END:
 	case IVTV_IOC_PAUSE_ENCODE:
 	case IVTV_IOC_RESUME_ENCODE:
@@ -1738,31 +1607,22 @@ static int ivtv_v4l2_do_ioctl(struct inode *inode, struct file *filp,
 int ivtv_v4l2_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
 		    unsigned long arg)
 {
-        struct ivtv_open_id *id = (struct ivtv_open_id *)filp->private_data;
-        struct ivtv *itv = id->itv;
-	int streamtype = id->type;
-
 	switch (cmd) {
 
 		/* This is temporary, I guess, it won't work unless here, so it's
 		   here till someone can show me a EOS mpeg captured without it here */
+		/*
 	case VIDIOC_STREAMOFF:
 		IVTV_DEBUG_IOCTL("VIDIOC_STREAMOFF\n");
-                if (id->open_id != itv->streams[streamtype].id)
-                        return -EBUSY;
+//                if (id->open_id != itv->streams[streamtype].id)
+//                        return -EBUSY;
                 ivtv_stream_off(itv, &itv->streams[streamtype]);
-		return 0;
+		return 0;*/
 
 	/* Converts old (0.1.9) non-conforming ioctls that were using
 	   'just some values I picked for now'. I hoped this would not be
 	   necessary, but too many people were already using existing apps
 	   (MythTV) written for this version of the driver. */
-	case 0xFFEE7703:
-		cmd = IVTV_IOC_G_CODEC;
-		break;
-	case 0xFFEE7704:
-		cmd = IVTV_IOC_S_CODEC;
-		break;
 	case 0xFFEE7781:
 		cmd = IVTV_IOC_PLAY;
 		break;
@@ -1778,124 +1638,9 @@ int ivtv_v4l2_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
 	case 0xFFEE7785:
 		cmd = IVTV_IOC_S_SLOW_FAST;
 		break;
-	case 0xFFEE7786:
-		cmd = IVTV_IOC_S_START_DECODE;
-		break;
-	case 0xFFEE7787:
-		cmd = IVTV_IOC_S_STOP_DECODE;
-		break;
 	case 0xFFEE7789:
 		cmd = IVTV_IOC_GET_FB;
 		break;
-	case IVTV_IOC_G_DRIVER_INFO:{
-		// AEW - put here becuase we can't do this after video_usercopy
-		// has done its work of copying a tiny portion of what we really
-		// need.
-		struct ivtv_driver_info driver_info;
-		uint32_t *driver_info_size_ptr = (uint32_t *) arg;
-		uint32_t driver_info_size = 0;
-
-		IVTV_DEBUG_IOCTL("IVTV_IOC_G_DRIVER_INFO\n");
-		if (driver_info_size_ptr == NULL) {
-			IVTV_DEBUG_WARN(
-				   "Error: IVTV_IOC_G_DRIVER_INFO called with "
-				   "invalid ivtv_driver_info address.\n");
-			return -EINVAL;
-		}
-
-                memset(&driver_info, 0,
-                       sizeof(struct ivtv_driver_info));
-                get_user(driver_info_size, driver_info_size_ptr);
-                if (driver_info_size > sizeof(struct ivtv_driver_info))
-                        driver_info_size = sizeof(struct ivtv_driver_info);
-
-                switch (driver_info_size) {
-                case IVTV_DRIVER_INFO_V3_SIZE:
-                        driver_info.hw_flags = itv->hw_flags;
-                        /* fall through */
-
-                case IVTV_DRIVER_INFO_V2_SIZE:
-                        driver_info.cardnr = itv->num;
-                        /* fall through */
-
-                case IVTV_DRIVER_INFO_V1_SIZE:
-                        driver_info.size = driver_info_size;
-                        driver_info.version =
-                            IVTV_VERSION_NUMBER(IVTV_VERSION_INFO_NAME);
-                        strncpy(driver_info.comment,
-                                IVTV_VERSION_COMMENT
-                                (IVTV_VERSION_INFO_NAME),
-                                IVTV_DRIVER_INFO_MAX_COMMENT_LENGTH);
-                        driver_info.
-                            comment[IVTV_DRIVER_INFO_MAX_COMMENT_LENGTH
-                                    - 1] = '\0';
-                        break;
-                
-                default:
-                        IVTV_DEBUG_WARN(
-                                   "Error: Unknown size "
-                                   "(0x%08x) passed to IVTV_IOC_G_DRIVER_INFO.\n",
-                                   driver_info_size);
-                        return -EINVAL;
-                }
-
-                if (copy_to_user((struct ivtv_driver_info *)arg,
-                                 &driver_info,
-                                 driver_info_size)) {
-                        IVTV_DEBUG_WARN(
-                                   "Error: IVTV_IOC_G_DRIVER_INFO Unable "
-                                   "to copy data to user space.\n");
-                        return -EINVAL;
-                }
-		return 0;
-	}
-
-	case IVTV_IOC_G_STREAM_INFO:{
-		// AEW - put here because we can't do this after video_usercopy
-		// has done its work of copying a tiny portion of what we really
-		// need.
-		struct ivtv_stream_info stream_info;
-		uint32_t *stream_info_size_ptr = (uint32_t *) arg;
-		uint32_t stream_info_size = 0;
-
-		IVTV_DEBUG_IOCTL("IVTV_IOC_G_STREAM_INFO\n");
-		if (stream_info_size_ptr == NULL) {
-			IVTV_DEBUG_WARN(
-				   "Error: IVTV_IOC_G_STREAM_INFO called with "
-				   "invalid ivtv_stream_info address.\n");
-			return -EINVAL;
-		}
-
-                memset(&stream_info, 0,
-                       sizeof(struct ivtv_stream_info));
-                get_user(stream_info_size, stream_info_size_ptr);
-                if (stream_info_size > sizeof(struct ivtv_stream_info))
-                        stream_info_size = sizeof(struct ivtv_stream_info);
-
-                switch (stream_info_size) {
-                case IVTV_STREAM_INFO_V1_SIZE:
-                        stream_info.size = stream_info_size;
-                        stream_info.type = streamtype;
-                        break;
-                
-                default:
-                        IVTV_DEBUG_WARN(
-                                   "Error: Unknown size "
-                                   "(0x%08x) passed to IVTV_IOC_G_STREAM_INFO.\n",
-                                   stream_info_size);
-                        return -EINVAL;
-                }
-
-                if (copy_to_user((struct ivtv_stream_info *)arg,
-                                 &stream_info,
-                                 stream_info_size)) {
-                        IVTV_DEBUG_WARN(
-                                   "Error: IVTV_IOC_G_STREAM_INFO Unable "
-                                   "to copy data to user space.\n");
-                        return -EINVAL;
-                }
-		return 0;
-	}
 	}
 	return video_usercopy(inode, filp, cmd, arg, ivtv_v4l2_do_ioctl);
 }
