@@ -26,11 +26,10 @@
 #include "ivtv-ioctl.h"
 #include "ivtv-mailbox.h"
 #include "ivtv-vbi.h"
-#include "ivtv-reset.h"
 
 typedef unsigned long uintptr_t;
 
-static void ivtv_FROM_DMA_done(struct ivtv *itv, int type);
+static void ivtv_FROM_DMA_done(struct ivtv *itv, int type, void *src);
 static int dma_from_device(struct ivtv *itv, struct ivtv_stream *st);
 
 IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
@@ -71,7 +70,7 @@ IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
 
 	/* Exclude 0x400 from the output, otherwise the log is flooded with
 	   these messages */
-	if (combo != 0x400)
+	if (combo & ~(0x88000400))
 		IVTV_DEBUG_IRQ("======= valid IRQ bits: 0x%08x ======\n", combo);
 
 	if (combo & IVTV_IRQ_DEC_VBI_RE_INSERT2) {
@@ -102,14 +101,13 @@ IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
 		wake_up(&itv->r_intr_wq);
 	}
 	if (combo & IVTV_IRQ_ENC_DMA_COMPLETE) {
-		IVTV_DEBUG_IRQ("IRQ: IVTV_IRQ_ENC_DMA_COMPLETE\n");
+		//IVTV_DEBUG_IRQ("IRQ: IVTV_IRQ_ENC_DMA_COMPLETE\n");
 		atomic_set(&itv->w_intr, 1);
 		wake_up(&itv->w_intr_wq);
 	}
 	if (combo & IVTV_IRQ_DMA_ERR) {
-		IVTV_DEBUG_WARN(
-			   "IRQ: IVTV_IRQ_DEC_DMA_ERR\n");
-		writel(0x0, itv->reg_mem + IVTV_REG_DMAXFER);
+		atomic_set(&itv->w_intr, 1);
+		wake_up(&itv->w_intr_wq);
 	}
 
 	/* Define irq bits */
@@ -122,7 +120,7 @@ IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
 
 	if (combo & IVTV_IRQ_ENC_DMA) {
 		if (combo & IVTV_IRQ_ENC_START_CAP) {
-			IVTV_DEBUG_IRQ("IRQ ENC DMA\n");
+			//IVTV_DEBUG_IRQ("IRQ ENC DMA\n");
 
 			/* set dma needed flag */
 			set_bit(IVTV_F_T_ENC_DMA_NEEDED, &itv->t_flags);
@@ -269,11 +267,13 @@ IRQRETURN_T ivtv_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
 		return IRQ_HANDLED;
 }
 
-static void ivtv_FROM_DMA_done(struct ivtv *itv, int stmtype)
+static void ivtv_FROM_DMA_done(struct ivtv *itv, int stmtype, void *src)
 {
 	struct ivtv_stream *stream = NULL;
 	struct ivtv_buffer *buf;
 	int freed = 0;
+	int x = 0;
+	int offset = 0;
 
 	IVTV_DEBUG_DMA("ENC: DMA Done\n");
 
@@ -284,13 +284,43 @@ static void ivtv_FROM_DMA_done(struct ivtv *itv, int stmtype)
 		/* Sync Buffer */
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 8)
 		pci_dma_sync_single_for_cpu((struct pci_dev *)itv->dev,
-					    buf->dma_handle, buf->buffer.length,
+					    buf->dma_handle, buf->buffer.length + 256,
 					    stream->dma);
 #else
 		pci_dma_sync_single(itv->dev, buf->dma_handle,
-				    buf->buffer.length, stream->dma);
+				    buf->buffer.length + 256, stream->dma);
 #endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 8) */
 
+		if (x == 0) {
+			offset = stream->last_offset;
+			if (((u32 *)(buf->buffer.m.userptr))[offset / 4] != 0x000001fe) {
+				for (offset = 0; offset < 64; offset++) {
+					if (((u32 *)(buf->buffer.m.userptr))[offset] == 0x000001fe) {
+						break;
+					}
+				}
+				offset *= 4;
+				if (offset == 256) {
+					IVTV_DEBUG_WARN("Couldn't find start of buffer within the first 256 bytes\n");
+					offset = 0;
+				}
+				if (stream->last_offset != offset)
+					IVTV_DEBUG_WARN("offset %d -> %d\n", stream->last_offset, offset);
+				stream->last_offset = offset;
+			}
+			writel(0, src);
+			if (offset) {
+				buf->buffer.bytesused -= offset;
+				memcpy((void *)buf->buffer.m.userptr,
+					(void *)(buf->buffer.m.userptr + offset), 
+					buf->buffer.bytesused + offset);
+			}
+			*(u32 *)(buf->buffer.m.userptr) = stream->buf_backup;
+		}
+		if (list_empty(&stream->dma_q.list)) {
+			buf->buffer.bytesused += stream->last_offset;
+		}
+		x++;
 		/* flag byteswap ABCD -> DCBA for MPG & VBI data outside irq */
 		if (stmtype == IVTV_ENC_STREAM_TYPE_MPG) 
 			set_bit(IVTV_F_B_NEED_BUF_SWAP, &buf->b_flags);
@@ -329,18 +359,18 @@ static void ivtv_FROM_DMA_done(struct ivtv *itv, int stmtype)
 
 void ivtv_sched_DMA(struct ivtv *itv)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
-	u32 type, size, offset, streamtype, pad = 0;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
+	u32 type, size, offset, streamtype;
 	u32 UVsize = 0, UVoffset = 0;
 	u64 pts_stamp = 0;
 	struct ivtv_stream *st;
 	int x;
+	void *src = (void *)itv->enc_mem;
 	int uvflag = 0;
 	long sequence;
 	u32 bytes_needed = 0, bytes_read = 0, bytes_received = 0;
 	struct ivtv_buffer *buf;
 	LIST_HEAD(free_list);
-	int xfer_pad;
 	unsigned long then;
 	DECLARE_WAITQUEUE(wait, current);
 	int rc = 0;
@@ -403,10 +433,11 @@ void ivtv_sched_DMA(struct ivtv *itv)
 	case 2:		/* PCM (audio) */
 		offset = data[1] + 12;
 		size = data[2] - 12;
-		pts_stamp = *(u32 *) (itv->dec_mem + offset - 8);
+		pts_stamp = readl(itv->dec_mem + offset - 8);
 		pts_stamp |=
-		    (u64) (*(u32 *) (itv->dec_mem + offset - 12)) << 32;
+		    (u64) (readl(itv->dec_mem + offset - 12)) << 32;
 		offset += IVTV_DECODER_OFFSET;
+		src = (void *)(itv->dec_mem - IVTV_DECODER_OFFSET);
 		IVTV_DEBUG_INFO(
 			   "DMA/PCM type 0x%08x,size 0x%08x,offset 0x%08x "
 			   "PTS 0x%09llx\n", type, size, offset, pts_stamp);
@@ -522,6 +553,9 @@ void ivtv_sched_DMA(struct ivtv *itv)
 	sequence = ++st->seq;
 
 	st->ubytes = size;
+	src += offset;
+	st->buf_backup = readl(src);
+	writel(0x000001fe, src);
 
 	for (x = 0; bytes_read < bytes_needed; x++) {
 		/* extract the buffers we procured earlier */
@@ -537,18 +571,10 @@ void ivtv_sched_DMA(struct ivtv *itv)
 		bytes_read += buf->buffer.length;
 
 		if (size < buf->buffer.length) {
-			xfer_pad = buf->buffer.length /*256 */ ;
-			pad = size;
 			buf->buffer.bytesused = size;
-			if (size > xfer_pad && size % xfer_pad)	/* Align */
-				size = size + (size % xfer_pad);
-			if (size < xfer_pad)	/* Too small */
-				size = xfer_pad;
-			st->SGarray[x].size = size;
+			st->SGarray[x].size = buf->buffer.length;
 			size = 0;
-
 		} else {
-			pad = 0;
 			buf->buffer.bytesused = buf->buffer.length;
 			st->SGarray[x].size = buf->buffer.length;
 			size -= st->SGarray[x].size;
@@ -568,11 +594,11 @@ void ivtv_sched_DMA(struct ivtv *itv)
 		/* Sync SG buffers */
 		pci_dma_sync_single_for_device((struct pci_dev *)itv->dev,
 					       buf->dma_handle,
-					       buf->buffer.length, st->dma);
+					       buf->buffer.length + 256, st->dma);
 #else
 		/* Sync SG buffers */
 		pci_dma_sync_single(itv->dev, buf->dma_handle,
-				    buf->buffer.length, st->dma);
+				    buf->buffer.length + 256, st->dma);
 #endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 8) */
 
 		ivtv_enq_buf_nolock(&st->dma_q, buf);
@@ -585,6 +611,7 @@ void ivtv_sched_DMA(struct ivtv *itv)
 		}
 	}
 	st->SG_length = x;
+	st->SGarray[x - 1].size += 256;
 
 	/* This should wrap gracefully */
 	st->trans_id++;
@@ -615,7 +642,7 @@ void ivtv_sched_DMA(struct ivtv *itv)
 		ivtv_vapi(itv, CX2341X_ENC_SCHED_DMA_TO_HOST, 3, 0, 0, st->type);
 
 		/* Gather Buffers from DMA xfer to read */
-		ivtv_FROM_DMA_done(itv, st->type);
+		ivtv_FROM_DMA_done(itv, st->type, src);
 	} else {
 		//down(&itv->DMA_lock);
 		if (dma_from_device(itv, st) != 0) {
@@ -624,7 +651,7 @@ void ivtv_sched_DMA(struct ivtv *itv)
 		//up(&itv->DMA_lock);
 
 		/* Gather Buffers from DMA xfer to read */
-		ivtv_FROM_DMA_done(itv, st->type);
+		ivtv_FROM_DMA_done(itv, st->type, src);
 	}
 
 	clear_bit(IVTV_F_S_DMAP, &st->s_flags);
@@ -638,14 +665,15 @@ static int dma_from_device(struct ivtv *itv, struct ivtv_stream *st)
 	unsigned long flags;
 	int redo_dma = 0;
 	int ret = 0;
-	int sg_offset;
+	int err;
 
 	/* wait for DMA complete status */
 	then = jiffies;
+	if (1)
 	while (!(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x03)) {
 		ivtv_sleep_timeout(HZ / 100, 1);
-		if ((jiffies - then) > (HZ * 3)) {
-			IVTV_DEBUG_WARN("ENC: REG_DMASTATUS wait failed\n");
+		if ((jiffies - then) > (HZ * 1)) {
+			IVTV_DEBUG_WARN("ENC: REG_DMASTATUS 1 wait failed\n");
 			break;
 		}
 	}
@@ -654,14 +682,14 @@ static int dma_from_device(struct ivtv *itv, struct ivtv_stream *st)
 	then = jiffies;
 	while ((readl(itv->reg_mem + IVTV_REG_DMAXFER) & 0x03)) {
 		ivtv_sleep_timeout(HZ / 100, 1);
-		if ((jiffies - then) > (HZ * 3)) {
-			IVTV_DEBUG_WARN("ENC: REG_DMAXFER wait failed\n");
+		if ((jiffies - then) > (HZ * 1)) {
+			IVTV_DEBUG_WARN("ENC: REG_DMAXFER 1 wait failed\n");
 			break;
 		}
 	}
 
-      redo_dma:
-
+redo_dma:
+	err = 0;
 	spin_lock_irqsave(&itv->DMA_slock, flags);
 	/* put SG Handle into register 0x0c */
 	ivtv_write_reg(st->SG_handle, itv->reg_mem + IVTV_REG_ENCDMAADDR);
@@ -675,49 +703,36 @@ static int dma_from_device(struct ivtv *itv, struct ivtv_stream *st)
 		ivtv_write_reg(readl(itv->reg_mem + IVTV_REG_DMAXFER) |
 			       0x02, itv->reg_mem + IVTV_REG_DMAXFER);
 	}
+	else {
+		err = 1;
+	}
 	spin_unlock_irqrestore(&itv->DMA_slock, flags);
 
 	/* wait for DMA to start */
 	then = jiffies;
-	while ((readl(itv->reg_mem + IVTV_REG_DMAXFER) & 0x02)) {
-		/* DMA Error */
-		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1A) {
-			break;
-		}
 
-		ivtv_sleep_timeout(HZ / 100, 1);
-		if ((jiffies - then) > (HZ * 3)) {
+	while (!err && (readl(itv->reg_mem + IVTV_REG_DMAXFER) & 0x02)) {
+		u32 stat = readl(itv->reg_mem + IVTV_REG_DMASTATUS);
+		/* DMA Error */
+		if (stat & 0x1a)
+			break;
+
+		//ivtv_sleep_timeout(HZ / 100, 1);
+		if ((jiffies - then) > (HZ / 5)) {
 			IVTV_DEBUG_WARN("ENC: REG_DMAXFER 2 wait failed\n");
-			break;
-		}
-	}
-
-	/* wait for DMA to be finished, check interrupt bit */
-	if (st->SG_length <= 8)
-		sg_offset = ((12 * st->SG_length) - 4);
-	else
-		sg_offset = ((12 * (st->SG_length % 8)) - 4);
-	then = jiffies;
-	while (!(readl(itv->reg_mem +
-			       IVTV_REG_ENCSG1SRC + sg_offset) & 0x80000000)) {
-		/* DMA Error */
-		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1A)
-			break;
-
-		ivtv_sleep_timeout(HZ / 100, 1);
-		if ((jiffies - then) > (HZ * 3)) {
-			IVTV_DEBUG_WARN("DEC: REG_ENCSG1LEN wait failed\n");
+			err = 1;
 			break;
 		}
 	}
 
 	/* Wait for Write Interrupt */
-	while (!(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1A) &&
+	while (!err && !(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1a) &&
 	       wait_event_interruptible(itv->w_intr_wq,
 					atomic_read(&itv->w_intr))) {
 		/* DMA Error */
-		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1A)
+		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1a) {
 			break;
+		}
 
 		if (atomic_read(&itv->w_intr))
 			break;
@@ -727,24 +742,27 @@ static int dma_from_device(struct ivtv *itv, struct ivtv_stream *st)
 
 	/* wait for DMA complete status */
 	then = jiffies;
-	while (!(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x02)) {
+
+	while (!err && !(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x02)) {
 		/* DMA Error */
-		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x1A) {
+		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x18) {
 			break;
 		}
 
 		ivtv_sleep_timeout(HZ / 100, 1);
-		if ((jiffies - then) > (HZ * 3)) {
-			IVTV_DEBUG_WARN("ENC: REG_DMASTATUS2 wait failed\n");
+		if ((jiffies - then) > (HZ / 50)) {
+			err = 1;
+			IVTV_DEBUG_WARN("ENC: REG_DMASTATUS 2 wait failed\n");
 			break;
 		}
 	}
 
 	/* DMA Error */
-	if ((readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x18)) {
-		IVTV_DEBUG_WARN("ENC: (%d) DMA Error 0x%08x\n",
+	if (err || (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x18)) {
+		IVTV_DEBUG_WARN("ENC: (%d) DMA Error 0x%08x %08x\n",
 			   redo_dma, readl(itv->reg_mem +
-						   IVTV_REG_DMASTATUS));
+						   IVTV_REG_DMASTATUS),
+			   readl(itv->reg_mem + IVTV_REG_IRQSTATUS));
 
 		/* Reset DMA Error, cancel last DMA? */
 		spin_lock_irqsave(&itv->DMA_slock, flags);
@@ -759,7 +777,6 @@ static int dma_from_device(struct ivtv *itv, struct ivtv_stream *st)
 		IVTV_DEBUG_WARN("ENC: REDO DMA took too many tries.\n");
 		ret = -ENOMEM;
 	}
-
 	return ret;
 }
 
@@ -769,6 +786,7 @@ void ivtv_sched_VBI(struct ivtv *itv, int streamtype)
 	u64 pts_stamp;
 	struct ivtv_stream *st;
 	int x;
+	void *src;
 	struct ivtv_buffer *buf = 0;
 	LIST_HEAD(free_list);
 	long sequence;
@@ -793,12 +811,14 @@ void ivtv_sched_VBI(struct ivtv *itv, int streamtype)
 			return;
 		}
 
-                if (itv->vbi_insert_mpeg && itv->vbi_sliced_in->service_set)
+		// Always use PIO for sliced VBI.
+                if (itv->vbi_sliced_in->service_set)
                         pio_mode = 1;
 
 		itv->vbi_frame += itv->vbi_fpi;
 		size = itv->vbi_enc_size * itv->vbi_fpi;
 		offset = bufptr;
+		src = (void *)(itv->enc_mem + offset);
 
 		IVTV_DEBUG_DMA(
 			"VBI(%d) BufPTR: 0x%08x offset 0x%08x size 0x%08x\n",
@@ -820,6 +840,7 @@ void ivtv_sched_VBI(struct ivtv *itv, int streamtype)
 		pio_mode = 1; /* Seems DMA mode has errors */
 		offset = readl(itv->dec_mem + itv->vbi_dec_start);
 		offset += itv->vbi_dec_start;
+		src = (void *)(itv->dec_mem + offset);
 		size = readl(itv->dec_mem + itv->vbi_dec_start + 4);
 		size += 8;
 		pts_stamp = 0;
@@ -917,6 +938,8 @@ skip_vbi_wait:
 	sequence = ++st->seq;
 
 	st->ubytes = bytes_received;
+	st->buf_backup = readl(src);
+	writel(0x000001fe, src);
 
 	for (x = 0; bytes_read < bytes_received; x++) {
 		/* extract the buffers we procured earlier */
@@ -969,11 +992,11 @@ skip_vbi_wait:
 		/* Sync SG buffers */
 		pci_dma_sync_single_for_device((struct pci_dev *)itv->dev,
 					       buf->dma_handle,
-					       buf->buffer.length, st->dma);
+					       buf->buffer.length + 256, st->dma);
 #else
 		/* Sync SG buffers */
 		pci_dma_sync_single(itv->dev, buf->dma_handle,
-				    buf->buffer.length, st->dma);
+				    buf->buffer.length + 256, st->dma);
 #endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 8) */
 
 		ivtv_enq_buf_nolock(&st->dma_q, buf);
@@ -1007,7 +1030,7 @@ skip_vbi_wait:
 
 	if (pio_mode) {
 		/* Gather Buffers from DMA xfer to read */
-		ivtv_FROM_DMA_done(itv, st->type);
+		ivtv_FROM_DMA_done(itv, st->type, src);
 	} else {
 		//down(&itv->DMA_lock);
 		if (dma_from_device(itv, st) != 0) {
@@ -1015,7 +1038,7 @@ skip_vbi_wait:
 		}
 		//up(&itv->DMA_lock);
 
-		ivtv_FROM_DMA_done(itv, st->type);
+		ivtv_FROM_DMA_done(itv, st->type, src);
 	}
 
 	clear_bit(IVTV_F_S_DMAP, &st->s_flags);
