@@ -25,7 +25,6 @@
 #include "ivtv-driver.h"
 #include "ivtv-queue.h"
 #include "ivtv-streams.h"
-#include "ivtv-reset.h"
 #include "ivtv-mailbox.h"
 #include "ivtv-i2c.h"
 #include "ivtv-kthreads.h"
@@ -86,7 +85,7 @@ static void ivtv_dualwatch_set_dual(struct ivtv *itv)
 	}
 
 	if (stereo_flag == stereo &&
-	    ((itv->codec.audio_bitmask & stereo_mask) == joint)) {
+	    ((itv->params.audio_properties & stereo_mask) == joint)) {
 		goal_stereo_flag = joint;
 	}
 
@@ -133,7 +132,7 @@ void ivtv_dualwatch_start_encoding(struct ivtv *itv)
 	if (itv->dualwatch.encoding)
 		return;
 	itv->dualwatch.encoding = 1;
-	itv->dualwatch.audio_bitmask = itv->codec.audio_bitmask;
+	itv->dualwatch.audio_bitmask = itv->params.audio_properties;
 	ivtv_dualwatch_update(itv);
 }
 
@@ -398,7 +397,7 @@ static void dec_work_handler(void *arg)
 	int rc = 0;
 	int x = 0, bytes_written = 0;
 	struct ivtv_buffer *buf;
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	u32 mem_offset, mem_size, orig_mem_size, hw_stream_type, buffer_bytes;
 	int stmtype = type;
 	u32 bytes_needed = 0;
@@ -406,11 +405,11 @@ static void dec_work_handler(void *arg)
 	LIST_HEAD(free_list);
 	long sequence;
 	u32 offset, sg_size = 0;
-	u32 uv_offset = (itv->height * itv->width);
+	u32 uv_offset = (itv->params.height * itv->params.width);
 	u32 yuv_buf = 0;
 	int yuv_num = 0;	
-	int yuv_frame_size = ((itv->width*itv->height)+ 
-				(itv->width*(itv->height/2)));
+	int yuv_frame_size = ((itv->params.width*itv->params.height)+ 
+				(itv->params.width*(itv->params.height/2)));
 
 	int y_done = 0;
 	int retrys;
@@ -477,13 +476,8 @@ static void dec_work_handler(void *arg)
 
 		/* Lock Decoder */
 		if (ivtv_buf_fill_lock(stream, 0, BUF_USED) >= data[2] &&
-		    (readl
-		     ((unsigned char *)itv->reg_mem +
-		      IVTV_REG_DMASTATUS) & 0x01)
-		    &&
-		    !(readl
-		      ((unsigned char *)itv->reg_mem +
-		       IVTV_REG_DMASTATUS) & 0x14)
+		    (readl ((unsigned char *)itv->reg_mem + IVTV_REG_DMASTATUS) & 0x01) &&
+		    !(readl ((unsigned char *)itv->reg_mem + IVTV_REG_DMASTATUS) & 0x14)
 		    && !test_and_set_bit(IVTV_F_S_DMAP, &stream->s_flags)) {
 			break;
 		}
@@ -563,7 +557,8 @@ static void dec_work_handler(void *arg)
 					   data[2]);
 				goto clear_dmap;
 			}
-			if (!test_bit(IVTV_F_T_DEC_STARTED, &itv->t_flags))					goto clear_dmap;	
+			if (!test_bit(IVTV_F_T_DEC_STARTED, &itv->t_flags))
+				goto clear_dmap;	
 		}
 
 		/* Get DMA Xfer Info to know what to send */
@@ -622,8 +617,8 @@ static void dec_work_handler(void *arg)
 
 		/* Sanity Checks for the Decoder DMA Xfer Info */
 		if (type == IVTV_DEC_STREAM_TYPE_MPG) {
-			if (buffer_bytes < stream->buf_max &&
-			    (stream->buf_max - buffer_bytes) <
+			if (buffer_bytes < stream->buf_min &&
+			    (stream->buf_min - buffer_bytes) <
 			    itv->dma_cfg.dec_min_xfer) {
 				IVTV_DEBUG_WARN(
 					   "DEC: Wants more data 0x%08x left 0x%08x\n",
