@@ -21,7 +21,6 @@
 #include <stdarg.h>
 
 #include "ivtv-driver.h"
-#include "ivtv-reset.h"
 #include "ivtv-fileops.h"
 #include "ivtv-mailbox.h"
 
@@ -139,7 +138,7 @@ static int ivtv_api_getresult(struct ivtv *itv, struct ivtv_mailbox *mbox, u32 *
 	if (readl((unsigned char *)&mbox->retval) != 0x00)
 		return -EBUSY;
 
-	for (count = 0; count < IVTV_MBOX_MAX_DATA; count++)
+	for (count = 0; count < CX2341X_MBOX_MAX_DATA; count++)
 		data[count] =
 		    readl((unsigned char *)&mbox->data[count]);
 
@@ -295,7 +294,7 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 	local_box = &mbox[0];	/* normal commands use mbox 1 */
 
 	/* check args */
-	if (args > IVTV_MBOX_MAX_DATA)
+	if (args > CX2341X_MBOX_MAX_DATA)
 		return -EINVAL;
 
 	switch (cmd) {
@@ -361,7 +360,7 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 		ivtv_write_reg(IVTV_API_STD_TIMEOUT,
 			       (unsigned char *)&local_box->timeout);
 
-		for (i = 0; i < IVTV_MBOX_MAX_DATA; i++) {
+		for (i = 0; i < CX2341X_MBOX_MAX_DATA; i++) {
 			if (i < args) {
 				ivtv_write_reg(data[i],
 					       (unsigned char *)&local_box->
@@ -460,7 +459,7 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 		ivtv_write_reg(IVTV_API_STD_TIMEOUT,
 			       (unsigned char *)&local_box->timeout);
 
-		for (i = 0; i < IVTV_MBOX_MAX_DATA; i++) {
+		for (i = 0; i < CX2341X_MBOX_MAX_DATA; i++) {
 			if (i < args) {
 				ivtv_write_reg(data[i],
 					       (unsigned char *)&local_box->
@@ -543,7 +542,7 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
                                         IVTV_DEBUG_API(
                                                    "cmd: 0x%08x arg %d stored\n",
                                                    cmd, i);
-				for (i = 0; i < IVTV_MBOX_MAX_DATA; i++) {
+				for (i = 0; i < CX2341X_MBOX_MAX_DATA; i++) {
 					data[i] = itv->api[cmd].r_data[i];
 				}
 
@@ -554,7 +553,7 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 			itv->api[cmd].marked = 1;	/* dirty mbox */
 			itv->api[cmd].jiffies = jiffies;	/* timestamp */
 
-			for (i = 0; i < IVTV_MBOX_MAX_DATA; i++) {
+			for (i = 0; i < CX2341X_MBOX_MAX_DATA; i++) {
 				if (i < args) {
 					itv->api[cmd].s_data[i] = data[i];
 				} else {
@@ -567,7 +566,7 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 			itv->api[cmd].marked = 1;	/* dirty mbox */
 			itv->api[cmd].jiffies = jiffies;	/* timestamp */
 
-			for (i = 0; i < IVTV_MBOX_MAX_DATA; i++) {
+			for (i = 0; i < CX2341X_MBOX_MAX_DATA; i++) {
 				if (i < args) {
 					itv->api[cmd].s_data[i] = data[i];
 				} else {
@@ -587,7 +586,7 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 		ivtv_write_reg(IVTV_API_STD_TIMEOUT,
 			       (unsigned char *)&local_box->timeout);
 
-		for (i = 0; i < IVTV_MBOX_MAX_DATA; i++) {
+		for (i = 0; i < CX2341X_MBOX_MAX_DATA; i++) {
 			if (i < args) {
 				ivtv_write_reg(data[i],
 					       (unsigned char *)&local_box->
@@ -620,7 +619,7 @@ int ivtv_api(struct ivtv *itv, struct ivtv_mailbox *mbox, struct semaphore *sem,
 			itv->api[cmd].marked = 0;
 		} else {
 			/* Store results */
-			for (count = 0; count < IVTV_MBOX_MAX_DATA; count++)
+			for (count = 0; count < CX2341X_MBOX_MAX_DATA; count++)
 				itv->api[cmd].r_data[count] = data[count];
 		}
 
@@ -670,9 +669,14 @@ static int ivtv_api_result(struct ivtv *itv, int cmd, int args, u32 * data)
 			cmd, &result, args, data);
 }
 
+int ivtv_api_func(void *priv, int cmd, int in, int out, u32 data[CX2341X_MBOX_MAX_DATA])
+{
+	return ivtv_api_result(priv, cmd, in, data);
+}
+
 int ivtv_vapi(struct ivtv *itv, int cmd, int args, ...)
 {
-	u32 data[IVTV_MBOX_MAX_DATA];
+	u32 data[CX2341X_MBOX_MAX_DATA];
 	va_list ap;
 	int i;
 
