@@ -225,7 +225,7 @@ static int ivtvfb_sleep_timeout(int timeout, int intr)
 static int ivtv_api_fb_get_framebuffer(struct ivtv *itv, u32 * fbbase,
 				       u32 * fblength)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	int rc;
 
 	rc = ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
@@ -237,7 +237,7 @@ static int ivtv_api_fb_get_framebuffer(struct ivtv *itv, u32 * fbbase,
 
 static int ivtv_api_fb_get_pixel_format(struct ivtv *itv)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
 		 CX2341X_OSD_GET_PIXEL_FORMAT, &result, 0, &data[0]);
@@ -246,7 +246,7 @@ static int ivtv_api_fb_get_pixel_format(struct ivtv *itv)
 
 static int ivtv_api_fb_set_pixel_format(struct ivtv *itv, int format)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	data[0] = format;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
@@ -257,7 +257,7 @@ static int ivtv_api_fb_set_pixel_format(struct ivtv *itv, int format)
 #if 0
 static int ivtv_api_fb_get_status(struct ivtv *itv)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, CX2341X_OSD_GET_STATE,
 		 &result, 0, &data[0]);
@@ -272,7 +272,7 @@ static int ivtv_api_fb_get_status(struct ivtv *itv)
 
 static int ivtv_api_fb_get_state(struct ivtv *itv)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, CX2341X_OSD_GET_STATE,
 		 &result, 0, &data[0]);
@@ -281,7 +281,7 @@ static int ivtv_api_fb_get_state(struct ivtv *itv)
 
 static int ivtv_api_fb_set_state(struct ivtv *itv, int enabled)
 {
-	u32 params[IVTV_MBOX_MAX_DATA], result;
+	u32 params[CX2341X_MBOX_MAX_DATA], result;
 	params[0] = enabled;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, CX2341X_OSD_SET_STATE,
@@ -293,7 +293,7 @@ static int ivtv_api_fb_set_framebuffer_window(struct ivtv *itv,
 					      int left, int top,
 					      int width, int height)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	
         data[0] = width;
 	data[1] = height;
@@ -308,7 +308,7 @@ static int ivtv_api_fb_set_framebuffer_window(struct ivtv *itv,
 static int ivtv_api_fb_get_osd_coords(struct ivtv *itv,
 				      struct ivtv_osd_coords *osd)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem, CX2341X_OSD_GET_OSD_COORDS,
 		 &result, 0, &data[0]);
@@ -326,7 +326,7 @@ static int ivtv_api_fb_get_osd_coords(struct ivtv *itv,
 static int ivtv_api_fb_set_osd_coords(struct ivtv *itv, const struct ivtv_osd_coords
 				      *osd)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 	data[0] = osd->offset + itv->osd_info->video_rbase;
 	data[1] = osd->pixel_stride;
@@ -343,7 +343,7 @@ static int ivtv_api_fb_set_osd_coords(struct ivtv *itv, const struct ivtv_osd_co
 
 static int ivtv_api_fb_get_screen_coords(struct ivtv *itv, struct rectangle *r)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
 		 CX2341X_OSD_GET_SCREEN_COORDS, &result, 0, &data[0]);
@@ -359,7 +359,7 @@ static int ivtv_api_fb_get_screen_coords(struct ivtv *itv, struct rectangle *r)
 static int ivtv_api_fb_set_screen_coords(struct ivtv *itv,
 					 const struct rectangle *r)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 	data[0] = r->x0;
 	data[1] = r->y0;
@@ -373,7 +373,7 @@ static int ivtv_api_fb_set_screen_coords(struct ivtv *itv,
 
 static int ivtv_api_fb_get_global_alpha(struct ivtv *itv)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
 		 CX2341X_OSD_GET_GLOBAL_ALPHA, &result, 0, &data[0]);
@@ -384,7 +384,7 @@ static int ivtv_api_fb_set_global_alpha(struct ivtv *itv,
 					int enable_global,
 					int alpha, int enable_local)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 	/* Save settings if Firmware reload */
 	itv->osd_info->global_alpha = alpha;
@@ -402,7 +402,7 @@ static int ivtv_api_fb_set_global_alpha(struct ivtv *itv,
 
 static void  ivtv_api_fb_set_colorKey(struct ivtv *itv,int state,  uint32_t color)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 
         itv->osd_info->color_key_state = state;
         itv->osd_info->color_key = color;
@@ -414,7 +414,7 @@ static void  ivtv_api_fb_set_colorKey(struct ivtv *itv,int state,  uint32_t colo
 }
 static int ivtv_api_fb_get_flicker_state(struct ivtv *itv)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 	ivtv_api(itv, itv->dec_mbox, &itv->dec_msem,
 		 CX2341X_OSD_GET_FLICKER_STATE, &result, 0, &data[0]);
@@ -423,7 +423,7 @@ static int ivtv_api_fb_get_flicker_state(struct ivtv *itv)
 
 static int ivtv_api_fb_set_flicker_state(struct ivtv *itv, int enabled)
 {
-	u32 params[IVTV_MBOX_MAX_DATA], result;
+	u32 params[CX2341X_MBOX_MAX_DATA], result;
 
 	params[0] = enabled;
 
@@ -438,7 +438,7 @@ static int ivtv_api_fb_blt_fill(struct ivtv *itv, int rasterop,
 				int width, int height, int destmask,
 				u32 destaddr, int deststride, u32 value)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 
 	data[0] = rasterop;
 	data[1] = alpha_mode;
@@ -463,7 +463,7 @@ static int ivtv_api_fb_blt_copy(struct ivtv *itv, int rasterop,
 				u32 destaddr, int deststride,
 				int sourcestride, int sourceaddr)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	IVTV_OSD_DEBUG_INFO(
 		       "ivtv_api_fb_blt_copy: width = %d, height = %d, destaddr = %d, deststride = %d, sourcestride = %d, sourceaddr = %d\n",
 		       width, height, destaddr, deststride, sourcestride,
@@ -771,7 +771,7 @@ int ivtvfb_prep_dec_dma_to_device(struct ivtv *itv,
 				  int size_in_bytes)
 {
 	struct ivtv_stream *stream = &itv->streams[IVTV_DEC_STREAM_TYPE_OSD];
-	u32 data[IVTV_MBOX_MAX_DATA];
+	u32 data[CX2341X_MBOX_MAX_DATA];
 	int ret = 0;
 
 	/* Map User DMA */
