@@ -127,7 +127,7 @@ void ivtv_free_buffer(struct ivtv *itv, struct ivtv_buffer *item,
 		/* UnMap Buffer */
 		if (st->dma != PCI_DMA_NONE) {
 			pci_unmap_single(itv->dev, item->dma_handle,
-					 item->buffer.length, st->dma);
+					 item->buffer.length + 256, st->dma);
 		}
 
 		kfree((void *)item->buffer.m.userptr);
@@ -237,7 +237,7 @@ struct ivtv_buffer *ivtv_init_buffer(struct ivtv *itv, struct ivtv_stream *st)
 		IVTV_DEBUG_WARN("No memory on ivtv_buffer alloc!\n");
 		return NULL;
 	}
-	buf->buffer.m.userptr = (unsigned long)kmalloc(st->bufsize, GFP_KERNEL);
+	buf->buffer.m.userptr = (unsigned long)kmalloc(st->bufsize+256, GFP_KERNEL);
 
 	if (buf->buffer.m.userptr == 0) {
 		kfree(buf);
@@ -288,7 +288,7 @@ struct ivtv_buffer *ivtv_init_buffer(struct ivtv *itv, struct ivtv_stream *st)
 	if (st->dma != PCI_DMA_NONE) {
 		buf->dma_handle = pci_map_single(itv->dev,
 						 (void *)buf->buffer.m.userptr,
-						 buf->buffer.length, st->dma);
+						 buf->buffer.length + 256, st->dma);
 	}
 
 	return buf;
@@ -728,17 +728,6 @@ int enc_gather_free_buffers(struct ivtv *itv, int streamtype,
 			continue;
 		}
 			
-		/* allocate a new buffer unless the maximum has already been allocated */
-		if (st->buf_total < st->buf_max) {
-			buf = ivtv_init_buffer(itv, st);
-			/* Out of memory? Then break off */
-			if (buf == NULL)
-				break;
-			bytes_received += buf->buffer.length;
-			list_add_tail(&buf->list, free_list);
-			continue;
-		}
-
 		/* Drop Buffers to allow things to continue to work */
 		buf = ivtv_deq_buf_nolock(itv, &st->full_q, streamtype);
 		if (!buf)
@@ -850,6 +839,7 @@ int dma_to_device(struct ivtv *itv, struct ivtv_stream *st,
 	int ret = 0;
 	/* wait for DMA complete status */
 	then = jiffies;
+	if (1)
 	while (!(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x03)) {
 		ivtv_sleep_timeout(HZ / 100, 1);
 		if ((jiffies - then) > (HZ * 3)) {
@@ -863,7 +853,7 @@ int dma_to_device(struct ivtv *itv, struct ivtv_stream *st,
 	then = jiffies;
 	while ((readl(itv->reg_mem + IVTV_REG_DMAXFER) & 0x03)) {
 		ivtv_sleep_timeout(HZ / 100, 1);
-		if ((jiffies - then) > (HZ * 3)) {
+		if ((jiffies - then) > (HZ / 5)) {
 			IVTV_DEBUG_WARN(
 				   "DMA_TO: REG_DMAXFER wait failed\n");
 			break;
@@ -894,6 +884,7 @@ int dma_to_device(struct ivtv *itv, struct ivtv_stream *st,
 
 	/* wait for DMA to start */
 	then = jiffies;
+
 	while ((readl(itv->reg_mem + IVTV_REG_DMAXFER) & 0x01)) {
 		/* DMA Error */
 		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x15) {
@@ -901,26 +892,13 @@ int dma_to_device(struct ivtv *itv, struct ivtv_stream *st,
 		}
 
 		ivtv_sleep_timeout(HZ / 100, 1);
-		if ((jiffies - then) > (HZ * 3)) {
+		if ((jiffies - then) > (HZ /5)) {
 			IVTV_DEBUG_WARN(
 				   "DMA_TO: REG_DMAXFER 2 wait failed\n");
 			break;
 		}
 	}
 
-	then = jiffies;
-	while (!(readl(itv->reg_mem + IVTV_REG_DECSG1LEN) & 0x80000000)) {
-		/* DMA Error */
-		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x15)
-			break;
-
-		ivtv_sleep_timeout(HZ / 100, 1);
-		if ((jiffies - then) > (HZ * 3)) {
-			IVTV_DEBUG_WARN(
-				   "DEC: REG_DECSG1LEN wait failed\n");
-			break;
-		}
-	}
 	/* Wait for Read Interrupt */
 	while (!(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x15) &&
 	       wait_event_interruptible(itv->r_intr_wq,
@@ -935,6 +913,7 @@ int dma_to_device(struct ivtv *itv, struct ivtv_stream *st,
 	atomic_set(&itv->r_intr, 0);
 	/* wait for DMA complete status */
 	then = jiffies;
+
 	while (!(readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x01)) {
 		/* DMA Error */
 		if (readl(itv->reg_mem + IVTV_REG_DMASTATUS) & 0x15) {
@@ -986,7 +965,7 @@ int ivtv_stream_alloc(struct ivtv *itv, int streamtype)
 
 	if (s->buffers) {
                 SGsize = sizeof(struct ivtv_SG_element) * (s->buf_min / s->bufsize);
-		IVTV_INFO("Allocate %s%s stream: "
+		IVTV_DEBUG_INFO("Allocate %s%s stream: "
 				"%d x %d buffers (%dKB total)\n",
 				s->dma != PCI_DMA_NONE ? "DMA " : "",
 				ivtv_stream_name(streamtype), 
