@@ -34,10 +34,8 @@
  */
 
 #include "ivtv-driver.h"
-#include "ivtv-reset.h"
 #include "ivtv-fileops.h"
 #include "ivtv-i2c.h"
-#include "ivtv-reset.h"
 #include "ivtv-queue.h"
 #include "ivtv-mailbox.h"
 #include "ivtv-audio.h"
@@ -85,17 +83,8 @@ static int ivtv_stream_init(struct ivtv *itv, int streamtype,
 {
 	struct ivtv_stream *s = &itv->streams[streamtype];
 
-	if (buffers) {
-		if (itv->options.dynbuf) {
-		        IVTV_INFO("Create %s%s stream: "
-				"%d x %d buffers (%dKB total)\n",
-				dma != PCI_DMA_NONE ? "DMA " : "",
-				ivtv_stream_name(streamtype), 
-				(buffers / bufsize), bufsize,
-				(buffers / 1024));
-                }
-	} else {
-		IVTV_INFO("Create %s%s stream\n",
+	if (buffers == 0) {
+		IVTV_DEBUG_INFO("Create %s%s stream\n",
 			       dma != PCI_DMA_NONE ? "DMA " : "",
 			       ivtv_stream_name(streamtype)); 
 	}
@@ -103,34 +92,24 @@ static int ivtv_stream_init(struct ivtv *itv, int streamtype,
 	/* Translate streamtype to buffers limit */
 	if (streamtype == IVTV_ENC_STREAM_TYPE_MPG) {
 		s->buf_min = (int)(itv->options.mpg_buffers * ((1024 * 1024)));
-		s->buf_max = (int)(itv->options.max_mpg_buffers * ((1024 * 1024)));
 	} else if (streamtype == IVTV_ENC_STREAM_TYPE_YUV) {
 		s->buf_min = (int)(itv->options.yuv_buffers * ((1024 * 1024)));
-		s->buf_max = (int)(itv->options.max_yuv_buffers * ((1024 * 1024)));
 	} else if (streamtype == IVTV_ENC_STREAM_TYPE_VBI) {
 		s->buf_min = (int)(itv->options.vbi_buffers * ((1024 * 1024)));
-		s->buf_max = (int)(itv->options.max_vbi_buffers * ((1024 * 1024)));
 	} else if (streamtype == IVTV_ENC_STREAM_TYPE_PCM) {
 		s->buf_min = (int)(itv->options.pcm_buffers * ((1024 * 1024)));
-		s->buf_max = (int)(itv->options.max_pcm_buffers * ((1024 * 1024)));
 	} else if (streamtype == IVTV_ENC_STREAM_TYPE_RAD) {
 		s->buf_min = (int)(itv->options.pcm_buffers * ((1024 * 1024)));
-		s->buf_max = (int)(itv->options.max_pcm_buffers * ((1024 * 1024)));
 	} else if (streamtype == IVTV_DEC_STREAM_TYPE_MPG) {
 		s->buf_min = (int)(itv->options.dec_mpg_buffers * ((1024 * 1024)));
-		s->buf_max = (int)(itv->options.max_dec_mpg_buffers * ((1024 * 1024)));
 	} else if (streamtype == IVTV_DEC_STREAM_TYPE_VBI) {
 		s->buf_min = (int)(itv->options.dec_vbi_buffers * ((1024 * 1024)));
-		s->buf_max = (int)(itv->options.max_dec_vbi_buffers * ((1024 * 1024)));
 	} else if (streamtype == IVTV_DEC_STREAM_TYPE_VOUT) {
 		s->buf_min = (int)(itv->options.dec_vbi_buffers * ((1024 * 1024)));
-		s->buf_max = (int)(itv->options.max_dec_vbi_buffers * ((1024 * 1024)));
 	} else if (streamtype == IVTV_DEC_STREAM_TYPE_YUV) {
 		s->buf_min = (int)(itv->options.dec_yuv_buffers * ((1024 * 1024)));
-		s->buf_max = (int)(itv->options.max_dec_yuv_buffers * ((1024 * 1024)));
 	} else if (streamtype == IVTV_DEC_STREAM_TYPE_OSD) {
 		s->buf_min = (int)(itv->options.dec_osd_buffers * ((1024 * 1024)));
-		s->buf_max = (int)(itv->options.max_dec_osd_buffers * ((1024 * 1024)));
 	} else {
 		IVTV_DEBUG_WARN(
 			   "Stream Init: Unknown stream: %d\n", streamtype);
@@ -217,8 +196,22 @@ static int ivtv_reg_dev(struct ivtv *itv, int streamtype, int minor, int reg_typ
 
 		return -ENOMEM;
 	}
-	IVTV_DEBUG_INFO("Registered v4l2 device for %s minor %d\n",
-		       ivtv_stream_name(streamtype), s->v4l2dev->minor);
+	switch (reg_type) {
+		case VFL_TYPE_GRABBER:
+			IVTV_INFO("Registered device video%d for %s\n", 
+				s->v4l2dev->minor, ivtv_stream_name(streamtype));
+			break;
+		case VFL_TYPE_RADIO:
+			IVTV_INFO("Registered device radio%d for %s\n", 
+				s->v4l2dev->minor - MINOR_VFL_TYPE_RADIO_MIN,
+			       	ivtv_stream_name(streamtype));
+			break;
+		case VFL_TYPE_VBI:
+			IVTV_INFO("Registered device vbi%d for %s\n", 
+				s->v4l2dev->minor - MINOR_VFL_TYPE_VBI_MIN,
+			       	ivtv_stream_name(streamtype));
+			break;
+	}
 
 	/* Success! All done. */
 
@@ -377,13 +370,10 @@ int ivtv_streams_setup(struct ivtv *itv)
 		if (ivtv_stream_setup(itv, x))
 			break;
 
-		if (!itv->options.dynbuf) 
-		{
-			/* Allocate Stream */
-			if (x != IVTV_DEC_STREAM_TYPE_OSD) {
-				if (ivtv_stream_alloc(itv, x))
-					break;
-			}
+		/* Allocate Stream */
+		if (x != IVTV_DEC_STREAM_TYPE_OSD) {
+			if (ivtv_stream_alloc(itv, x))
+				break;
 		}
 	}
 	if (x == itv->streamcount) {
@@ -450,7 +440,6 @@ void ivtv_setup_v4l2_encode_stream(struct ivtv *itv, int type)
 {
 	struct ivtv_stream *s = ivtv_stream_safeget(
 		"ivtv_setup_v4l2_encode_stream", itv, type);
-        int dnr_temporal;
 
 	if (s == NULL)
 		return;
@@ -458,73 +447,9 @@ void ivtv_setup_v4l2_encode_stream(struct ivtv *itv, int type)
 	/* streams_lock must be held */
 	IVTV_ASSERT(ivtv_sem_count(&s->mlock) <= 0);
 
-	/* assign stream type */
-	ivtv_vapi(itv, CX2341X_ENC_SET_STREAM_TYPE, 1, itv->codec.stream_type);
+	cx2341x_update(itv, ivtv_api_func, NULL, &itv->params);
 
-	/* assign output port */
-	ivtv_vapi(itv, CX2341X_ENC_SET_OUTPUT_PORT, 1, 0);	/* 0 = Memory */
-
-	/* assign framerate */
-	ivtv_vapi(itv, CX2341X_ENC_SET_FRAME_RATE, 1, itv->is_50hz);
-
-	/* assign frame size */
-	ivtv_vapi(itv, CX2341X_ENC_SET_FRAME_SIZE, 2, itv->height, itv->width);
-
-	/* assign aspect ratio */
-	ivtv_vapi(itv, CX2341X_ENC_SET_ASPECT_RATIO, 1, itv->codec.aspect);
-
-	/* automatically calculate bitrate on the fly */
-	if (itv->codec.bitrate > 15000000) {
-		IVTV_DEBUG_WARN(
-			   "Bitrate too high, adjusted 15mbit, see mpeg specs\n");
-		itv->codec.bitrate = 15000000;
-	}
-
-	/* assign bitrates */
-	ivtv_vapi(itv, CX2341X_ENC_SET_BIT_RATE, 5, itv->codec.bitrate_mode,
-		  itv->codec.bitrate,	/* bps */
-		  itv->codec.bitrate_peak / 400,	/* peak/400 */
-		  0x00, 0x00, 0x70);	/* encoding buffer, ckennedy */
-
-	/* assign gop properties */
-	ivtv_vapi(itv, CX2341X_ENC_SET_GOP_PROPERTIES, 2,
-		  itv->codec.framespergop, itv->codec.bframes);
-
-	/* assign 3 2 pulldown */
-	ivtv_vapi(itv, CX2341X_ENC_SET_3_2_PULLDOWN, 1, itv->codec.pulldown);
-
-	/* assign gop closure */
-	ivtv_vapi(itv, CX2341X_ENC_SET_GOP_CLOSURE, 1, itv->codec.gop_closure);
-
-	/* assign audio properties */
-	ivtv_vapi(itv, CX2341X_ENC_SET_AUDIO_PROPERTIES, 1,
-		  itv->codec.audio_bitmask);
 	ivtv_dualwatch_start_encoding(itv);
-
-	/* assign dnr filter mode */
-	ivtv_vapi(itv, CX2341X_ENC_SET_DNR_FILTER_MODE, 2,
-		  itv->codec.dnr_mode, itv->codec.dnr_type);
-
-	/* assign dnr filter props */
-        dnr_temporal = itv->codec.dnr_temporal;
-        if (itv->width != 720 || 
-            (itv->is_50hz && itv->height != 576) ||
-            (itv->is_60hz && itv->height != 480)) {
-                // dnr_temporal != 0 for scaled images gives ghosting effect.
-                // Always set to 0 for scaled images.
-                dnr_temporal = 0;
-        }
-	ivtv_vapi(itv, CX2341X_ENC_SET_DNR_FILTER_PROPS, 2,
-		  itv->codec.dnr_spatial, dnr_temporal);
-
-	/* assign coring levels (luma_h, luma_l, chroma_h, chroma_l) */
-	ivtv_vapi(itv, CX2341X_ENC_SET_CORING_LEVELS, 4, 0, 255, 0, 255);
-
-	/* assign spatial filter type: luma_t: 1 = horiz_only, chroma_t: 1 = horiz_only */
-	ivtv_vapi(itv, CX2341X_ENC_SET_SPATIAL_FILTER_TYPE, 2, 1, 1);
-
-	/* assign frame drop rate */
-	ivtv_vapi(itv, CX2341X_ENC_SET_FRAME_DROP_RATE, 1, 0);
 }
 
 int ivtv_passthrough_mode(struct ivtv *itv, int enable)
@@ -617,7 +542,7 @@ int ivtv_passthrough_mode(struct ivtv *itv, int enable)
 static void ivtv_vbi_setup(struct ivtv *itv, int mode)
 {
 	int raw = itv->vbi_sliced_in->service_set == 0;
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	int lines;
 	int x;
 	int i;
@@ -627,18 +552,18 @@ static void ivtv_vbi_setup(struct ivtv *itv, int mode)
 	if (!(mode & 0x04) && (mode & 0x02) && 
 		!raw && itv->vbi_insert_mpeg) 
 	{
-		itv->codec.stream_type = 0;
+		itv->params.stream_type = 0;
 
         	/* assign stream type */
         	ivtv_vapi(itv, CX2341X_ENC_SET_STREAM_TYPE, 1, 
-			itv->codec.stream_type);
+			itv->params.stream_type);
 	}
 
 	/* Reset VBI */
 	ivtv_vapi(itv, CX2341X_ENC_SET_VBI_LINE, 5, 0xffff , 0, 0, 0, 0);
 
 	/* Don't use VBI if scaling */
-	if (raw && itv->height != h) {
+	if (raw && itv->params.height != h) {
 		itv->video_dec_func(itv, VIDIOC_S_FMT, &itv->vbi_in);
 		return;
 	}
@@ -776,30 +701,9 @@ static void unknown_setup_api(struct ivtv *itv)
 	ivtv_vapi(itv, CX2341X_ENC_MISC, 2, 12, 0);
 }
 
-int ivtv_init_digitizer(struct ivtv *itv)
-{
-	int dummy;
-
-        /* Disable digitizer */
-        IVTV_DEBUG_INFO("Disabling digitizer\n");
-	itv->video_dec_func(itv, VIDIOC_STREAMOFF, &dummy);
-
-        /* initialize or refresh input */
-        if (atomic_read(&itv->capturing) == 0)
-                ivtv_vapi(itv, CX2341X_ENC_INITIALIZE_INPUT, 0);
-        else
-                ivtv_vapi(itv, CX2341X_ENC_REFRESH_INPUT, 0);
-
-        /* enable digitizer (saa7115) */
-        IVTV_DEBUG_INFO("Enabling digitizer\n");
-	itv->video_dec_func(itv, VIDIOC_STREAMON, &dummy);
-
-	return 0;
-}
-
 int ivtv_start_v4l2_encode_stream(struct ivtv *itv, int type)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	int x = 0;
 	int captype = 0, subtype = 0;
 	struct ivtv_stream *st = ivtv_stream_safeget(
@@ -908,7 +812,7 @@ int ivtv_start_v4l2_encode_stream(struct ivtv *itv, int type)
 			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
 
 		/* Setup VBI */
-        	if ((itv->v4l2_cap & V4L2_CAP_VBI_CAPTURE)) {
+        	if (itv->v4l2_cap & V4L2_CAP_VBI_CAPTURE) {
                 	int pkt_type = 0x02;
 
                 	/* Raw mode, no private packet */
@@ -974,9 +878,8 @@ int ivtv_start_v4l2_encode_stream(struct ivtv *itv, int type)
         	clear_bit(IVTV_F_I_EOS, &itv->i_flags);
 
 		/* Initialize Digitizer for Capture */
-		ivtv_init_digitizer(itv);	
+                ivtv_vapi(itv, CX2341X_ENC_INITIALIZE_INPUT, 0);
 
-        	/*IVTV_DEBUG_INFO("Sleeping for 100ms\n"); */
         	ivtv_sleep_timeout(HZ / 10, 0);
         }
 
@@ -1008,7 +911,7 @@ int ivtv_start_v4l2_encode_stream(struct ivtv *itv, int type)
 
 int ivtv_setup_v4l2_decode_stream(struct ivtv *itv, int type)
 {
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	int x;
 	struct ivtv_stream *s = ivtv_stream_safeget(
 		"ivtv_setup_v4l2_decode_stream", itv, type);
@@ -1071,9 +974,9 @@ int ivtv_setup_v4l2_decode_stream(struct ivtv *itv, int type)
 		data[0] = 0;
 		break;
 	}
-	data[1] = itv->width;	/* YUV source width */
-	data[2] = itv->height;
-	data[3] = itv->codec.audio_bitmask & 0xffff;	/* Audio settings to use,
+	data[1] = itv->params.width;	/* YUV source width */
+	data[2] = itv->params.height;
+	data[3] = itv->params.audio_properties;	/* Audio settings to use,
 							   bitmap. see docs. */
 	x = ivtv_api(itv, itv->dec_mbox,
 		     &itv->dec_msem,
@@ -1222,7 +1125,7 @@ int ivtv_stop_capture(struct ivtv *itv, int type)
 	unsigned long then;
 	int x;
 	int stopmode;
-	u32 data[IVTV_MBOX_MAX_DATA], result;
+	u32 data[CX2341X_MBOX_MAX_DATA], result;
 	int thread_to_stop = 0;
 
 	if (st == NULL)
@@ -1329,9 +1232,8 @@ int ivtv_stop_capture(struct ivtv *itv, int type)
 			set_current_state(TASK_INTERRUPTIBLE);
 
 			/* wait 2s for EOS interrupt */
-			while ((!test_bit(IVTV_F_I_EOS,
-					  &itv->i_flags)) &&
-			       (jiffies < (then + (1 * HZ)))) {
+			while ((!test_bit(IVTV_F_I_EOS, &itv->i_flags)) &&
+			       (jiffies < (then + (2 * HZ)))) {
 				schedule_timeout(HZ / 100);
 			}
 
@@ -1410,8 +1312,7 @@ int ivtv_stop_capture(struct ivtv *itv, int type)
 	atomic_dec(&itv->capturing);
 
 	/* Clear capture and no-read bits */
-	if (!test_bit(IVTV_F_S_STREAMOFF, &st->s_flags))
-		clear_bit(IVTV_F_S_CAPTURING, &st->s_flags);
+	clear_bit(IVTV_F_S_CAPTURING, &st->s_flags);
 
 	/* Clear capture started bit */
         if (type == IVTV_ENC_STREAM_TYPE_VBI) {
