@@ -23,7 +23,6 @@
 #include "ivtv-i2c.h"
 #include "ivtv-vbi.h"
 #include "ivtv-ioctl.h"
-#include "v4l2-common.h"
 
 typedef unsigned long uintptr_t;
 
@@ -79,11 +78,11 @@ void vbi_setup_lcr(struct ivtv *itv, int set, int is_pal, struct v4l2_sliced_vbi
 
 void vbi_schedule_work(struct ivtv *itv)
 {
-#ifdef LINUX26
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	queue_work(itv->vbi_work_queues, &itv->vbi_work_queue);
 #else
 	ivtv_schedule_work(&itv->vbi_work_queue);
-#endif /* LINUX26 */
+#endif
 }
 
 static void passthrough_vbi_data(struct ivtv *itv, u8 *p, int cnt)
@@ -388,10 +387,10 @@ ssize_t ivtv_write_vbi(struct ivtv *itv, const char *ubuf, size_t count)
 
         if (found_cc && itv->vbi_cc_pos < sizeof(itv->vbi_cc_data_even)) {
                 down(&itv->vbi_dec_lock);
-                itv->vbi_cc_data_even[itv->vbi_cc_pos] = cc[2];
-                itv->vbi_cc_data_even[itv->vbi_cc_pos + 1] = cc[3];
                 itv->vbi_cc_data_odd[itv->vbi_cc_pos] = cc[0];
                 itv->vbi_cc_data_odd[itv->vbi_cc_pos + 1] = cc[1];
+                itv->vbi_cc_data_even[itv->vbi_cc_pos] = cc[2];
+                itv->vbi_cc_data_even[itv->vbi_cc_pos + 1] = cc[3];
                 itv->vbi_cc_pos += 2;
                 up(&itv->vbi_dec_lock);
                 set_bit(IVTV_F_T_DEC_VBI_NEEDED, &itv->t_flags);
@@ -449,7 +448,8 @@ static u32 compress_sliced_buf(struct ivtv *itv, u32 line, u8 *buf, u32 size)
                 if (*buf == 0xff)
                         break;
         }
-        for (i = 0; i < size / line_size; i++) {
+
+        for (i = 0; i < 1 + (size - 1) / line_size; i++) {
                 p = buf + i * line_size;
 
                 // Look for SAV code 
@@ -458,7 +458,7 @@ static u32 compress_sliced_buf(struct ivtv *itv, u32 line, u8 *buf, u32 size)
                         break;
                 }
                 vbi.p = p + 4;
-		itv->card->video_dec_func(itv, VIDIOC_INT_DECODE_VBI_LINE, &vbi);
+		itv->video_dec_func(itv, VIDIOC_INT_DECODE_VBI_LINE, &vbi);
                 if (vbi.type) {
                         itv->vbi_sliced_data[line].id = vbi.type;
                         itv->vbi_sliced_data[line].field = vbi.is_second_field;
@@ -613,7 +613,7 @@ void vbi_work_handler(void *arg)
 			data.id = V4L2_SLICED_WSS_625;
 			data.field = 0;
 
-			if (itv->card->video_dec_func(itv, VIDIOC_INT_G_VBI_DATA, &data) == 0) {
+			if (itv->video_dec_func(itv, VIDIOC_INT_G_VBI_DATA, &data) == 0) {
 				ivtv_set_wss(itv, 1, data.data[0] & 0xf);
 				itv->vbi_wss_no_update = 0;
 			} else if (itv->vbi_wss_no_update == 4) {
@@ -628,13 +628,13 @@ void vbi_work_handler(void *arg)
 
 			data.id = V4L2_SLICED_CAPTION_525;
 			data.field = 0;
-			if (itv->card->video_dec_func(itv, VIDIOC_INT_G_VBI_DATA, &data) == 0) {
+			if (itv->video_dec_func(itv, VIDIOC_INT_G_VBI_DATA, &data) == 0) {
 				mode |= 1;
 				c1 = data.data[0];
 				c2 = data.data[1];
 			}
 			data.field = 1;
-			if (itv->card->video_dec_func(itv, VIDIOC_INT_G_VBI_DATA, &data) == 0) {
+			if (itv->video_dec_func(itv, VIDIOC_INT_G_VBI_DATA, &data) == 0) {
 				mode |= 2;
 				c3 = data.data[0];
 				c4 = data.data[1];
