@@ -139,8 +139,11 @@ void ivtv_video_set_io(struct ivtv *itv)
 			route.input |= UPD64083_YCNR_MODE;
 		}
 		else if (itv->card->hw_video & IVTV_HW_UPD64031A) {
-			/* Use upd64031a output for tuner and composite inputs */
-			route.input |= UPD64083_EXT_Y_ADC;
+		  /* Use upd64031a output for tuner and composite(CX23416GYC only) inputs */
+		  if ((type == IVTV_CARD_INPUT_VID_TUNER)||
+		      (itv->card->type == IVTV_CARD_CX23416GYC)) {
+		    route.input |= UPD64083_EXT_Y_ADC;
+		  }
 		}
 		ivtv_upd64083(itv, VIDIOC_INT_S_VIDEO_ROUTING, &route);
 	}
