@@ -21,6 +21,7 @@
 #include <features.h>		/* Uses _GNU_SOURCE to define getsubopt in stdlib.h */
 #include <stdlib.h>
 #include <stdio.h>
+#include <signal.h>
 #include <string.h>
 #include <inttypes.h>
 #include <getopt.h>
@@ -33,7 +34,8 @@
 #include <math.h>
 #include <linux/types.h>
 
-#include <videodev2.h>
+#define __user
+#include <linux/videodev2.h>
 
 #include "ivtv.h"
 
@@ -418,16 +420,44 @@ void process(struct v4l2_sliced_vbi_data *s)
 	}
 }
 
+static void sigalrm(int s)
+{
+	exit(0);
+}
+
 int main(int argc, char **argv)
 {
-	char *device = "/dev/vbi0";
+	char device[100] = "/dev/vbi0";
 	struct v4l2_format fmt;
         v4l2_std_id std;
 	struct v4l2_sliced_vbi_data *buf;
+	int timeout = 0;
 	int fh;
 
-	if (argc == 2)
-		device = argv[1];
+	while (1) {
+		int ch = getopt(argc, argv, "d:t:h");
+
+		if (ch == -1)
+			break;
+		switch (ch) {
+		case 'd':
+		        strncpy(device, optarg , 99 );
+			device[99] = 0;
+			if (strlen(optarg) > 99 ) {
+				fprintf(stderr, "device too long [max 99 char], truncated to %s" , device);
+			}
+			if (optarg[0] >= '0' && optarg[0] <= '9')
+				sprintf(device, "/dev/vbi%s", optarg);
+			break;
+		case 't':
+			timeout = atol(optarg);
+			break;
+		default:
+			fprintf(stderr, "usage: %s [-d <dev>] [-t <secs>] [-h]\n"
+					"defaults are /dev/vbi0 and no timeout\n", argv[0]);
+			exit(-1);
+		}
+	}
 	fh = open(device, O_RDONLY);
 
 	if (fh == -1) {
@@ -450,6 +480,10 @@ int main(int argc, char **argv)
 
 	fprintf(stderr, "%08x, %d\n", fmt.fmt.sliced.service_set, fmt.fmt.sliced.io_size);
 	buf = malloc(fmt.fmt.sliced.io_size);
+	if (timeout) {
+		signal(SIGALRM, sigalrm);
+		alarm(timeout);
+	}
 	for (;;) {
 		int size = read(fh, buf, fmt.fmt.sliced.io_size);
 		int i;
