@@ -96,20 +96,6 @@ void print_usage(void)
 		cfg.play_cmd_tmpl);
 }
 
-static int get_cardnr(int fd)
-{
-        struct ivtv_driver_info info;
-	int res;
-
-	memset(&info, 0, sizeof(info));
-	info.size = sizeof(info);
-	res = ioctl(fd, IVTV_IOC_G_DRIVER_INFO, &info);
-	if (res == 0 && info.size >= IVTV_DRIVER_INFO_V2_SIZE) {
-		return info.cardnr;
-	}
-	return -1;
-}
-
 int scan_channels(int allfreqs)
 {
 	int i;
@@ -338,16 +324,18 @@ int main(int argc, char **argv)
 			pause();
 		}
 	} else {
-		int radio_cardnr = get_cardnr(cfg.fh);
-		int audio_cardnr = -1;
+		struct v4l2_capability radio_cap;
+		struct v4l2_capability audio_cap;
 		int fd;
 		
+		ioctl(cfg.fh, VIDIOC_QUERYCAP, &radio_cap);
 		fd = open(cfg.audio_in, O_RDONLY);
+		audio_cap.bus_info[0] = 0;
 		if (fd != -1) {
-			audio_cardnr = get_cardnr(fd);
+			ioctl(fd, VIDIOC_QUERYCAP, &audio_cap);
 			close(fd);
 		}
-		if (radio_cardnr != audio_cardnr) {
+		if (strcmp((char *)radio_cap.bus_info, (char *)audio_cap.bus_info)) {
 			fprintf(stderr, "%s belongs to a different ivtv driver then %s.\n",
 					cfg.audio_in, cfg.radio_dev);
 			fprintf(stderr,
